% Compute the acoustic mass matrices of the individual gcells.
%
% function ems = acoustic_mass(self, geom, P)
%
% Return an array of the element matrices so they may be assembled.
%    Arguments
%            geom=geometry field
%            P=acoustic pressure field
function ems = acoustic_mass(self, geom, P)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells,'nfens');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute basis f. values + basis f. gradients wrt parametric coord
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material
    mat = get(self.feblock, 'mater');
    % Prepare some data: connectivity and the element matrices
    conns = get(gcells, 'conn'); % connectivity
    Me = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(P,conns(i,:),'eqnums');
        Me{i} =zeros(nfens);
    end
    xs =get(geom,'values');% retrieve the geometry information
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        for j=1:npts_per_gcell
            J = Jacobian_matrix(gcells,Nders{j},x);
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
            Ndersp = Nders{j}/J;
            Me{i} = Me{i} + Ndersp*(Jac*w(j))*Ndersp' ;
        end% Loop over quadrature points
    end% Loop over elements
    ems  = elematset(struct('mat',{Me}, 'eqnums',{eqnums}));
end
