% Compute the acoustic stiffness matrices of the individual gcells.
%
% function ems = acoustic_stiffness(self, geom, ddP)
%
% Return an array of them so they may be assembled.
%     Arguments
%       geom=geometry field
%       ddP=second-order time derivative of the acoustic pressure field
%
function ems = acoustic_stiffness(self, geom, ddP)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells(1),'nfens');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute basis f. values + basis f. gradients wrt parametric coor
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material
    mat = get(self.feblock, 'mater');
    bulk = get(get(mat,'property'),'bulk');
    rho = get(get(mat,'property'),'rho');
    c2 = bulk/rho ;
    % Prepare some data: connectivity and the element matrices
    conns = get(gcells, 'conn'); % connectivity
    Se = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(ddP,conns(i,:),'eqnums');
        Se{i} =zeros(nfens);
    end
    xs =get(geom,'values');
     % Now loop over all gcells in the block
     for i=1:size(conns,1)
         conn =conns(i,:);
         x=xs(conn,:);
         for j=1:npts_per_gcell
             J = Jacobian_matrix(gcells,Nders{j},x);
             Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
             Se{i} = Se{i} + Ns{j}*(Ns{j}' * (Jac/c2 * w(j)));
         end
    end
    ems  = elematset(struct('mat',{Se}, 'eqnums',{eqnums}));
end
