% Compute the stiffness matrices of the individual gcells.
% Return an array of them so they may be assembled.
%    Call as
% ems = stiffness(feb, geom, u1, u)
%     geom=reference geometry field
%     u1=displacement field in the *current* configuration
%     u=displacement field in the last known configuration
%    Returns an array of elemat objects that may be assembled
%    into the system stiffness matrix.
%
function ems = stiffness (self, geom, u)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells,'nfens');
    dim = get(geom,'dim');
    conns = get(gcells, 'conn'); % connectivity
    Kes= cell(ngcells,1);
    eqnums = cell(ngcells,1);
    for i=1:ngcells
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Kes{i} = [];
    end
    iem=1;
    for i=1:ngcells                % Now loop over all gcells in the block
        conn = conns(i,:);                 % connectivity
        X = gather(geom, conn, 'values', 'noreshape'); % reference coordinates
        U = gather(u, conn, 'values', 'noreshape'); % displacement in step n
        x = X + U; % previous known coordinates
        Ke = zeros(dim*nfens);       % element stiffness matrix
        gap = self.get_gap (self.surface_data,x);
        if (gap<0)
            normal = self.get_normal (self.surface_data,x);
            normal = normal/norm(normal);
            Ke = Ke + self.penalty*(normal'*normal);
        end
        Kes{iem} =  Ke;
        iem=iem+1;
    end
    ems  = elematset(struct('mat',{Kes(1:iem-1)}, 'eqnums',{eqnums(1:iem-1)}));
end
