%  Draw graphic representation for all gcells.
%
% function draw(self, gv, context)
%
%      self= finite element block, 
%      gv=graphic viewer, 
%      context = structure with optional fields u (displacement field), R
%      (rotation-matrix field), rot (rotation-vector field).
% 
% As a convenience, both the displacement (u) and the rotation field (R) may be
% supplied as empty.
% In that case the displacement is initialized to be a zero field, and the
% rotation field is initialized to identities as rotations.
% 
% Also, if  the rot field is supplied, the R field is computed from it.
% 
% See: draw_mesh method of the feblock class.
% 
function draw(self, gv, context)
    if (isfield(context,'rot'))
        context.R = update_rotation_field(field(struct ('name',['R'], 'dim', 9, 'data', ones(get(context.rot,'nfens'),1)*reshape(eye(3),1,9))),context.rot);
    end
    if (isempty(context.u))
        context.u=0*context.x;
    end
    if (isempty(context.R))
        context.R = update_rotation_field(field(struct ('name',['R'], 'dim', 9, 'data', ones(get(context.x,'nfens'),1)*reshape(eye(3),1,9))),0*context.x);
    end
    draw_mesh(self, gv, context);
end
