% Compute the infsup_Gh matrices of the individual entities in the block
% (in this case nodes) for use in the numerical LBB test.
% 
% function ems = infsup_Gh (self, geom, u)
% 
% Return an array of the element matrices so they may be assembled.
% 
%     geom=geometry field
%     u=displacement field
%
function ems = infsup_Gh (self, geom, u)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    tangents =eye(dim);
    %     Evaluate the nodal basis function gradients
    bfun_gradients = nodal_bfun_gradients (self, geom);
    %     Preallocate
    Kes= cell(length(bfun_gradients),1);
    eqnums = cell(length(bfun_gradients),1);
    for i=1:length(bfun_gradients)
        eqnums{i} =[];
        Kes{i} = [];
    end
    xs =gather(geom, 1:length(bfun_gradients), 'values', 'noreshape'); % coordinates of all nodes
    eqnumss =gather(u, 1:length(bfun_gradients), 'eqnums','noreshape');% equation numbers for all nodes
    iem=1;
    % Now loop over all finite element nodes
    for nix=1:length(bfun_gradients)
        if bfun_gradients{nix}.Vpatch>0 % This node has a element patch in this block
            np=length(bfun_gradients{nix}.patchconn);
            c = xs(nix, :); % coordinates of central node
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,tangents);%RAW: tangent vectors?
            end
            Bnodal= self.hdivmat (self,bfun_gradients{nix}.Nspd,c,Rm);
            Ke = Bnodal'*Bnodal * bfun_gradients{nix}.Vpatch;
            Kes{iem} =  Ke;
            eqnums{iem} =  reshape(eqnumss(bfun_gradients{nix}.patchconn,:)',(np)*size(eqnumss,2),1);
            iem=iem+1;
        end
    end
    ems  = elematset(struct('mat',{Kes(1:iem-1)}, 'eqnums',{eqnums(1:iem-1)}));
end

% %     If there are any interior integration points, execute the loop over
%     %     all the cells and perform the integrations
%     if (npts > length(get(gcells(1), 'conn')))
%         ems(length(gmap)+1:length(gmap)+length(gcells)) = deal(elemat);
%         Ke = zeros(get(x,'dim')*nfens); 
%         for i=1:length(gcells)
%             conn = get(gcells(i), 'conn'); % connectivity
%             lx = gather(x, conn, 'values', 'noreshape'); % coord
%             Ke = 0*Ke; % zero out element stiffness
%             % Loop over all integration points
%             for j=nconn+1:npts
%                 N = bfun(gcells(i),pc(j,:));
%                 Rm = material_directions(self,gcells(i),pc(j,:),lx);
%                 detJ = Jacobian_volume(gcells(i),pc(j,:),lx);
%                 B = divmat(gcells(i), pc(j,:), lx, Rm);
%                 Ke = Ke + B'*B * detJ * w(j);
%             end
%             ems(iem) = set(ems(iem), 'mat', Ke);
%             ems(iem) = set(ems(iem), 'eqnums', gather(u,conn,'eqnums'));
%             iem=iem+1;
%         end
%     end
