% Compute the source-term load vectors of the individual gcells.
%
% function evs = source_loads(self, geom, temp)
%
% Return an array of them so they may be assembled.
%    Call as
%       evs = source_loads(feb, geom, temp)
%    where
%       geom=geometry field
%       temp=temperature field
%
function evs = source_loads(self, geom, temp)
    gcells = get(self.feblock,'gcells');
    dim = get(geom,'dim');
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    tangents =eye(dim);
    % Material
    mat = get(self.feblock, 'mater');
    Q = get(get(mat,'property'),'source');
    if (isempty(Q)) % Any loads due to internal heat sources?
       evs  = elevecset(struct('vec',{{}}, 'eqnums',{{}}));
       return;
    end
    if (strcmp(class(Q),'function_handle'))
        need_to_eval=1;
    else
        need_to_eval=0;
    end
    %     Evaluate the nodal basis function gradients
    bfun_gradients = nodal_bfun_gradients (self, geom);
    %     Preallocate
    Fe = cell(length(bfun_gradients),1);
    eqnums = cell(length(bfun_gradients),1);
    for i=1:length(bfun_gradients)
        eqnums{i} =[];
        Fe{i} =[];
    end
    xs =gather(geom, 1:length(bfun_gradients), 'values', 'noreshape'); % coordinates of all nodes
    eqnumss =gather(temp, 1:length(bfun_gradients), 'eqnums','noreshape');% equation numbers for all nodes
    % Now loop over all finite element nodes
    iem =1;
    for nix=1:length(bfun_gradients)
        if bfun_gradients{nix}.Vpatch>0 % This node has a element patch in this block
            np=length(bfun_gradients{nix}.patchconn);
            c = xs(nix, :); % coordinates of central node
            if (need_to_eval)
                val = feval(Q,c);
            else
                val = Q;
            end
            Fe{iem} =  (val * bfun_gradients{nix}.Vpatch);
            eqnums{iem} =  eqnumss(nix,:);
            iem=iem+1;
        end
    end
    evs  = elevecset(struct('vec',{Fe(1:iem-1)}, 'eqnums',{eqnums(1:iem-1)}));
end
