% Constructor.
%
% function retobj = mater_defor_ss_linviscel_biax (varargin)
%
% This class represents deformable materials, linear elastic,
% for biaxial strain/stress states, with viscous damping.
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing parameters.
% Parameters:
%     property = property object
%   optional field
%     reduction = type of stress state reduction, 'stress' (for plane stress), or 'strain'
%          (for plane strain), or 'axisymm' (for axially symmetric). Default is 'strain'.
%
function retobj = mater_defor_ss_linviscel_biax (varargin)
    class_name='mater_defor_ss_linviscel_biax';
   if (nargin == 0)
        parent=mater_defor_ss_linel_biax;
        self.property = [];
        self.reduction='strain';
        retobj = class(self,class_name, parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            options =varargin{1};
            parent=mater_defor_ss_linel_biax(options);
            self.property = options.property;
            self.reduction ='strain';
            if isfield(options,'reduction')
                self.reduction =options.reduction;
            end
            switch self.reduction
                case 'axisymm'
                case 'strain'
                case 'stress'
                otherwise
                    error([' Reduction ' self.reduction ' not recognized']);
            end
            retobj = class(self,class_name, parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end