% Advance the state by specified number of steps
%   Call as:
%      advance(obj, torque, dt, nsteps)
%   where
%      obj       - object
%      torque    - handle to function defining spatial torque
%      dt        - time step
%      nsteps    - number of steps to take
%
%   The first form returns the list of property names as a string,
%   the second form returns the value of the named property.
%
function retobj = advance(self,torque,dt,nsteps)
    Omegan1 = self.Omega;
    Rn1 = self.R;
    t=get(self.rotint,'t');
    I=self.I;
    tol = 100*eps;
    maxiter = 20;
    for i=1:nsteps
        f = dt*Omegan1;
        r = f+100*tol;
        iter=1;
        while norm(r)>tol
            nf= norm(f);
            r = dt*I*Omegan1-sin(nf)/nf*I*f-(1-cos(nf))/nf^2*skewmat(f)*I*f;
            f=f+DG(f,I)\r;
            if iter >maxiter
                if (max(abs(r)) > 1e-9)
                    warning(['mleok - Failed to converge: ||residual||=' num2str(norm(r))]);
                end
                break;
            end
        end
        F=rotmat(skewmat(f));
        Rn=Rn1*F;
        Tn = Rn'*feval(torque, t+dt, Rn);
        Omegan = I\(F'*I*Omegan1+dt*Tn);
        Rn1=Rn;
        Omegan1=Omegan;
        t=t+dt;
    end
    self.R = Rn;
    self.Omega = Omegan;
    self.rotint = set(self.rotint,'t',t);
    retobj=self;
end

function d=DG(f,I)
    nf= norm(f);
    d = (cos(nf)*nf-sin(nf))/(nf^3)*I*f*f'+...
        sin(nf)/nf*I+...
        (sin(nf)*nf-2*(1-cos(nf)))/(nf^4)*skewmat(f)*I*f*f'+...
        (1-cos(nf))/nf^2*(-skewmat(I*f)+skewmat(f)*I);
    return;
end