%% Free vibration of cantilevered drill pipe with massive collar.
%  
% Long and slender drill pipe with a massive collar at the bottom. Torsional vibration only.
% Ansys verification problem (http://www.kxcad.net/ansys/ANSYS/ansyshelp/Hlp_V_VM57.html).
%
function CB_drill_pipe_vibration
%%
% Material parameters (steel).
G=12000000;%29,000ksi
nu=0.3;
E=2*(1+nu)*G;
g=32.17*12; % in/second^2
% rho=490/12^3/g;% Pound/g mass density per unit volume
rho=15.2174/12^4;% Pound/g mass density per unit volume

%%
% Dimensions
L=5000*12;% in
L0=1;% in
%%
% Cross-sectional properties
Parameters=Beam_cross_section(struct('type','hollow_circle','Rint',3.83/2,'Rext',4.5/2));
% Figure of the dimension of the collar given its moment of inertia.
J10=29.3*12;
R0=(J10/rho/(pi/2))^(1/4);
Parameters0=Beam_cross_section(struct('type','hollow_circle','Rint',0,'Rext',R0));


%%
% Simulation control parameters.
utol=1e-2;% convergence tolerance
graphics=true;
neigvs=12;

%%
% The mesh.
Parameters.x1x2_vector=[1 0 0];
[fens1,gcells1] = Beam_member([[0 0 0];[0 0 -L]],12,@gcellset_beam3,Parameters);
Parameters0.x1x2_vector=[1 0 0];
[fens2,gcells2] = Beam_member([0,0,-L;0,0,-L-L0],1,@gcellset_beam3,Parameters0);
[fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens2, gcells2, L0/100);
gcells= cat(gcells1,gcells2);
topn=fenode_select(fens, struct('box', update_box([],[0 0 0])));

%%
% Material properties
prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
mater = mater_defor_ss_linel_triax (struct('property',prop));
%%
% Finite element block
feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells));


%%
% Construct the requisite fields, geometry and displacement
% Initialize configuration variables
geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
u0=0*geom0;
Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
Rfield0=Rfield;
% Define the displacement increment field
dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));

% Apply EBC's
ebc_fenids=topn;
ebc_prescribed=[1];
ebc_comp=[];% clamped
ebc_val=ebc_fenids*0;
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=1:count(fens);
ebc_prescribed=0*ebc_fenids+1;
ebc_val=0*ebc_fenids+0;
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, 1, ebc_val);
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, 2, ebc_val);
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, 3, ebc_val);
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, 4, ebc_val);
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, 5, ebc_val);
dchi  = apply_ebc (dchi);


%%
% Number equations
dchi  = numbereqns (dchi);
neqns=get(dchi, 'neqns');

%%
% Solver

if (graphics )
    gv=graphic_viewer;
end


%%
% Free-vibration.
K = start (sparse_sysmat, neqns);
K = assemble (K, cat(2,stiffness(feb,geom0,u0,Rfield0,dchi)));
M = start (sparse_sysmat, neqns);
M = assemble (M, mass(feb,geom0,u0,Rfield0,dchi));
Km=get(K, 'mat');
Mm=get(M, 'mat');


[W,Omega]=eig(full(Km),full(Mm));
[Omegas2,ix]=sort((diag(Omega))) ;
for j=1:neigvs
    Frequencies(j)= sqrt(abs(Omegas2(j)))/2/pi;
end

W=W(:,ix);
disp(['  Lowest natural frequencies '  num2str(Frequencies) '  [Hz]']);

if (graphics )
    %%
    % Graphic display of the mesh and the deformed beam
    for j = 1:neigvs
        gv=reset (gv,[]);
        draw(feb, gv, struct ('x', geom0, 'u', u0, 'R', Rfield0, 'facecolor','none'));
        dchi = scatter_sysvec(dchi, W(:,ix(j)));
        scale=1.2/norm(slice(dchi,(4:6),'rotat'));
        wu1 = u0 + scale*slice(dchi,(1:3),'transl');   % increment displacement
        wRfield1=update_rotation_field(Rfield0,scale*slice(dchi,(4:6),'rotat'));
        title(['  Frequency ' num2str(Frequencies(j)) ]);
        draw(feb, gv, struct ('x', geom0, 'u', wu1, 'R', wRfield1, 'facecolor','yellow'));
        draw(fens,gv, struct ('x', geom0, 'u', 0*u0, 'color','blue','offset',0.35));
        draw_local_coordinates(feb, gv, struct ('x', geom0, 'u', wu1, 'R', wRfield1, 'length',500));
        
        
        xlabel('X');
        ylabel('Y');
        zlabel('Z');
        hold on
        
        %     draw_axes(gv, struct([]));
        camset (gv, 1.0e+005 *[-0.229569466394230  -0.289274975454620   4.965386370192127  -0.000145765729689  -0.000849854263685  -0.305030711572438    0                   0   0.000010000000000   0.000004622834789]);
        pause(1)
    end
end

end