disp('Taut wire: statics, uniform load');
L=6;
P=4;
q =0.1;% This is consistent now with the sign convention in the textbook
n=2; % number of elements
% Mesh
% finite element node set
fens=fenodeset(struct ('id',(1:n+1)','xyz',linspace(0, L, n+1)'));
% set of geometric cells
gcells = gcellset_L2(struct('id',1,'conn',[(1:n)', (2:n+1)']));
% Finite element block
feb = feblock_defor_taut_wire(struct ('mater',mater,...
    'gcells',gcells,...
    'integration_rule',simpson_1_3_rule,...
    'P',P));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 1, 'fens',fens));
% Define the displacement field
w   = 0*clone(geom,'w');
% Apply EBC's
fenids=[1]; prescribed=[1]; component=[1]; val=0;
w   = set_ebc(w, fenids, prescribed, component, val);
w   = apply_ebc (w);
% Number equations
w   = numbereqns (w);
% Assemble the system matrix
K = start (dense_sysmat, get(w, 'neqns'));
K = assemble (K, stiffness(feb, geom, w));
% Load
fi = force_intensity(struct ('magn',inline(num2str(q))));
F = start (sysvec, get(w, 'neqns'));
F = assemble (F, body_loads(feb, geom, w, fi));
% Solve
w = scatter_sysvec(w, get(K,'mat')\get(F,'vec'));
% Plot
xs= (0:0.01:L);
plot (xs/L, (2*P/q/L^2)*q/P*xs.*(L-xs/2),'r-','linewidth', 3); 
hold on
plot (gather(geom, (1:n+ 1),'values')/L, ...
    (2*P/q/L^2)*gather(w, (1:n+ 1),'values'),'bo-','linewidth', 3); 
left_handed_axes;% Plotting is consistent with the textbook sign convention
xlabel('x/L'),ylabel('(2*P/q/L^2)*w')
set_graphics_defaults

figure
plot (xs/L, (q).*(L-xs)/q/L,'r-','linewidth', 3); 
hold on
pc=linspace(-1,+1,10);
for j=1:count(gcells)
    ix=isoparamipol1d(gcells,pc,gather(geom,get(subset(gcells,j),'conn'),'values'));
    iwp=isoparamipol1ddersp(gcells,pc,gather(geom,get(subset(gcells,j),'conn'),'values'),gather(w,get(subset(gcells,j),'conn'),'values'));
    plot(ix/L,P*iwp/q/L,'b-','linewidth',3);hold on
end
left_handed_axes;% Plotting is consistent with the textbook sign convention
xlabel('x/L'),ylabel('S/q/L')
set_graphics_defaults


assignin('caller','faesor_test_passed',((norm([    0;0.7500;1.0000]-(2*P/q/L^2)*gather(w, (1:n+ 1),'values')))<1e-12));