% Mesh of a 3-D block located in a given range, H8 cells
%
% function [fens,gcells] = H8_block(Length,Width,Height,nL,nW,nH)
%
% Range xyz =<0,Length> x <0,Width> x <0,Height>
% Divided into elements: nL, nW, nH in the first, second, and
% third direction (x,y,z). Geometric cells H8.
function [fens,gcells] = H8_block(Length,Width,Height,nL,nW,nH)
    nnodes=(nL+1)*(nW+1)*(nH+1);
    xs=zeros(nnodes,3);
    ncells=(nL)*(nW)*(nH);
    conns=zeros(ncells,8);

    f=1;
    for k=1:(nH+1)
        for j=1:(nW+1)
            for i=1:(nL+1)
                xyz=[(i-1)*Length/nL (j-1)*Width/nW (k-1)*Height/nH];
                xs(f,:)=xyz;
                f=f+1;
            end
        end
    end
    fens=fenodeset(struct('id',(1:nnodes)','xyz',xs));
    gc=1;
    for i=1:nL
        for j=1:nW
            for k=1:nH
                nn=node_numbers(i,j,k,nL,nW,nH);
                conns(gc,:)=nn;
                gc=gc+1;
            end
        end
    end
    gcells =gcellset_H8(struct ('conn',conns));
end

% first go along Length, then Width, and finally Height
function nn=node_numbers(i,j,k,nL,nW,nH)
    f=(k-1)*((nL+1)*(nW+1))+(j-1)*(nL+1)+i;
    nn=[f (f+1)  f+(nL+1)+1 f+(nL+1)];
    nn=[nn nn+((nL+1)*(nW+1))];
end
