% Convert a mesh of quadrilateral Q4 to quadrilateral Q8.
%
% function [fens,gcells] = Q4_to_Q8(fens,gcells,options)
%
% options =attributes recognized by the constructor gcell_Q8
function [fens,gcells] = Q4_to_Q8(fens,gcells,options)
    if ~isstruct(options)
        options = struct('id',1);
    end
    nedges=4;
    ec = [1, 2; 2, 3; 3, 4; 4, 1];
     conns = get (gcells,'conn');
       % make a search structure for edges
    edges={};
    for i= 1:count(gcells)
        conn = conns(i,:);
        for J = 1:nedges
            ev=conn(ec(J,:));
            anchor=min(ev);
            if length(edges)<anchor
                edges{anchor}=[];
            end
            edges{anchor}=unique([edges{anchor} max(ev)]);
        end
    end
    % now generate new node number for each edge
    n=count(fens);
    nodes=edges;
    for i= 1:length(edges)
        e=edges{i};
        for J = 1:length(e)
            n=n+1;
            e(J)=n;
        end
        nodes{i}=e;
    end
    %     fens(length(fens)+1:n)=deal(fenode(struct('id',0,'xyz',[0 0 0])));
    xyz1=get(fens,'xyz');
    xyz =zeros(n,size(xyz1,2));
    xyz(1:size(xyz1,1),:)=xyz1;
    % calculate the locations of the new nodes
    % and construct the new nodes
    for i= 1:length(edges)
        e=edges{i};
        n=nodes{i};
        for J = 1:length(e)
            xyz(n(J),:)=(xyz(i,:)+xyz(e(J),:))/2;
        end
    end
    nfens=size(xyz,2);% number of nodes in the original mesh plus number of the edge nodes
    % construct new geometry cells
    nconns =zeros(count(gcells),8);
    nc=1;
    for i= 1:count(gcells)
        conn = conns(i,:);
        econn=zeros(1,nedges);
        for J = 1:nedges
            ev=conn(ec(J,:));
            anchor=min(ev);
            e=edges{anchor};
            n=nodes{anchor};
            econn(J)=n(find(e==max(ev)));
        end
        nconns(nc,:) =[conn econn];
        nc= nc+ 1;
    end
    Thickness =get(gcells,'other_dimension');
    gcells =gcellset_Q8(struct('id',1:nc-1,'conn',nconns,...
        'other_dimension',Thickness));
    fens =fenodeset(struct('id',1:size(xyz,1),'xyz',xyz));
end
