% Convert a mesh of Tetrahedron T4 (four-node) to Tetrahedron T10.
%
% function [fens,gcells] = T4_to_T10(fens,gcells)
%
function [fens,gcells] = T4_to_T10(fens,gcells)
    conn =get(gcells,'conn') ;
    nedges=6;
    ec = [1, 2; 2, 3; 3, 1; 4, 1; 4, 2; 4, 3];
    % make a search structure for edges
    edges={};
    for i= 1:size(conn,1)
        for J = 1:nedges
            ev=conn(i,ec(J,:));
            anchor=min(ev);
            if length(edges)<anchor
                edges{anchor}=[];
            end
            edges{anchor}=unique([edges{anchor} max(ev)]);
        end
    end
    % now generate new node number for each edge
    id =get(fens,'id');
    xyz =get(fens,'xyz');
    n=length(id);
    nodes=edges;
    for i= 1:length(edges)
        e=edges{i};
        for J = 1:length(e)
            n=n+1;
            e(J)=n;
        end
        nodes{i}=e;
    end
    id(length(id)+1:n)=0;
    xyz(length(id)+1:n,1:3)=0;
    % calculate the locations of the new nodes
    % and construct the new nodes
    for i= 1:length(edges)
        e=edges{i};
        n=nodes{i};
        for J = 1:length(e)
            id(n(J))=n(J);
            xyz(n(J),:)=(xyz(i,:)+xyz(e(J),:))/2;
        end
    end
    nfens=length(id);% number of nodes in the original mesh plus number of the edge nodes
    fens=fenodeset(struct('id',id,'xyz',xyz));
    % construct new geometry cells
    nconn=zeros(size(conn,1),10);
    nc=1;
    for i= 1:size(conn,1)
        econn=zeros(1,nedges);
        for J = 1:nedges
            ev=conn(i,ec(J,:));
            anchor=min(ev);
            e=edges{anchor};
            n=nodes{anchor};
            econn(J)=n(find(e==max(ev)));
        end
        nconn(nc,:) =[conn(i,:) econn];
        nc= nc+ 1;
    end
    gcells=gcellset_T10(struct('conn',nconn));
    return;
end
