% Merge two meshes together by gluing together nodes within tolerance.
%
% function [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens2,
%                     gcells2, tolerance)
%
% The two meshes, fens1, gcells1, and fens2, gcells2, are glued together by
% merging the nodes that fall within a box of size "tolerance". If tolerance 
% is set to zero, no merging of nodes is performed; the two meshes are 
% simply concatenated together.
% 
% The merged node set, fens, and the two arrays of geometry cells with
% renumbered  connectivities are returned.
% 
% Important notes:
% On entry into this function the connectivity of gcells1 point into fens1
% and the connectivity of gcells2 point into fens2. 
% After this function returns the connectivity of both gcells1 and gcells2 point into
% fens. 
% The node set IDs fens1 will have changed, whereas the IDs of the node set
% fens2 all are guaranteed to be the same. Therefore, the connectivity 
% of gcells2 will in fact remain the same.
%
%  
function [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens2, gcells2, tolerance)
    % Fuse the nodes
    [fens,fens1,fens2] = fuse_nodes(fens1, fens2, tolerance);
    
    % Renumber the cells
    id1 = get(fens1,'id');
    gcells1= update_connectivity(gcells1,id1);
    id2 = get(fens2,'id');
    gcells2= update_connectivity(gcells2,id2);
    % Note that now the connectivity of both gcells1 and gcells2 point into
    % fens.
end

