% Renumber the nodes and the cells using an optimal numbering of the finite
% element nodes. 
% 
% function [fens, gcells]=renum_mesh(fens, gcells, method)
% 
% fens, gcells = arrays of finite element nodes and geometric cells
% method =  either 'symrcm' or 'symamd' (optional argument, default
%           'symrcm') 
%     
function [fens, gcells]=renum_mesh(fens, gcells, method)
    if (~exist('method','var'))
        method ='symrcm';
    end

    conns = gcells_to_conns(gcells);
    A=conn_to_sparse(conns);
    p= 1:size(A,1);
    
    if (strcmp(method,'symrcm'))
        % Reverse Cuthill-McKee
        p=symrcm(A);
    elseif (strcmp(method,'symamd'))
        p=symamd(A);
    else
        eval(['p=' method '(A);']);
    end
    clear A

    ip=(1:length(p));
    ip(p)=(1:length(p));

    conn =get(gcells,'conn');
    for C=1:size(conn,1)
        c=conn(C,:);
        for j=1:size(c,1)
            c(j,:) =ip(c(j,:));
        end 
        conn(C,:)=c;
    end
    gcells =set(gcells,'conn',conn);

    id=get(fens,'id'); nid =id;
    xyz=get(fens,'xyz'); nxyz=xyz;
    for C=1:size(xyz,1)
        nxyz(C,:)=xyz(p(C),:);
        nid(C)=C;
    end
    fens = set(fens,'id',nid);
    fens = set(fens,'xyz',nxyz);
end


