function distance3dtool(arg)
%distance3dTOOL A simple tool for interactively obtaining 3-D coordinates 
%
% distance3dTOOL(FIG) Specify figure handle
%
% Example:
%   surf(peaks);
%   distance3dtool;
%   % click on surface

if nargin<1
   arg = gcf;
end

if ~ishandle(arg)
   feval(arg);
   return;
end

%% initialize gui %%
fig = arg;
figure(fig);

uistate = uiclearmode(fig);
[tool, htext] = createUI;
hmarker1 = line('marker','o','markersize',10,'markerfacecolor','k','erasemode','xor','visible','off');
hmarker2 = line('marker','o','markersize',10,'markerfacecolor','r','erasemode','xor','visible','off');
dline = line('marker','.','markersize',10,'markerfacecolor','r','erasemode','xor','visible','off');

state.uistate = uistate;
state.text = htext;
state.tool = tool;
state.fig = fig;
state.click = 0;
state.p= zeros(2,3);
state.marker(1) = hmarker1;
state.marker(2) = hmarker2;
state.line = dline;
setappdata(fig,'distance3dtool',state);
setappdata(state.tool,'distance3dhost',fig);

set(fig,'windowbuttondownfcn','distance3dtool(''click'')');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function off

state = getappdata(gcbf,'distance3dtool');

if ~isempty(state)
    delete(state.tool);
end

fig = getappdata(gcbf,'distance3dhost');

if ~isempty(fig) & ishandle(fig)
    state = getappdata(fig,'distance3dtool');     
    uirestore(state.uistate);
    delete(state.marker(1));
    delete(state.marker(2));
    delete(state.line);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function click

[p v vi] = select3d;
state = getappdata(gcbf,'distance3dtool');

if ~ishandle(state.text)
    state.text = createUI;
end


if ~ishandle(state.marker(1))
    state.marker(1) = [];
end

if ~ishandle(state.marker(2))
    state.marker(2) = [];
end

if ~ishandle(state.line)
    state.line = [];
end

state.click=state.click+1;
if isempty(p)
    p = [nan nan nan];
    set(state.marker(state.click),'visible','off');
else
    set(state.marker(state.click),'visible','on','xdata',p(1),'ydata',p(2),'zdata',p(3));
end
state.p(state.click,:)=p;

if (state.click<2)
    set(state.line,'visible','off');
    set(state.marker(2),'visible','off');
end
% Update tool and markers
if (state.click==2)
    set(state.line,'visible','on','xdata',state.p(:,1),'ydata',state.p(:,2),'zdata',state.p(:,3));
    set(state.text,'string',createString(state.p));
    state.click=0;
end
setappdata(state.fig,'distance3dtool',state);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [fig, h] = createUI

pos = [200 200 200 200];

% Create selection tool %
fig = figure('handlevisibility','off','menubar','none','resize','off',...
    'numbertitle','off','name','Select 3-D Tool','position',pos,'deletefcn','distance3dtool(''off'')');

h = uicontrol('style','text','parent',fig,'string',createString([]),...
    'units','norm','position',[0 0 1 1],'horizontalalignment','left');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [str] = createString(p)
    if (isempty(p))
        str='Pick points';
    else
        str = sprintf('\n   Location 1:\n  X  %f\n  Y:  %f\n  Z:  %f  \n\n  Location 2:\n  X:  %f\n  Y:  %f\n  Z:  %f  \n\n   Distance:\n  %f',...
            p(1,:),p(2,:),norm(p(1,:)-p(2,:)));
    end