% Calculate the material stiffness matrix.
%   Call as:
%     D = tangent_moduli(m, context)
%   where
%     m=material
%    context=struct
% with fields
%  - mandatory
%  - optional
%     stiff_type=type of the stiffness,
%        either 'Lagrangean'
%        or 'Eulerian'
%              requires field 'F'= current deformation gradient
% the output arguments are
%     D=matrix 6x6
%
function D = tangent_moduli_nice(self, context)
    stiff_type='Eulerian';
    if isfield(context,'stiff_type')
        stiff_type= context.stiff_type;
    end
    switch (stiff_type)
        case 'Eulerian'
            E = self.E;
            nu = self.nu;
            dim=length(context.F);
            J =det(context.F);
            lambda = E * nu / (1 + nu) / (1 - 2*(nu));          % Lame constant #1
            mu     = E / (2 * (1 + nu));                        % shear modulus
            kappa  = E / 3 / (1 - 2*nu);                        % bulk modulus
            c=(context.Fbar)'*(context.Fbar);
            c=inv(c);
            for i=1:dim
                for j=1:dim
                    for k=1:dim
                        for l=1:dim
                            C(i,j,k,l)=(lambda / J) * c(i,j) * c(k,l) + 2 * (mu - lambda*log(J))/J * c(i,k)*c(j,l);
                        end
                    end
                end
            end
            F=context.F;
            Fbar=context.Fbar;
            D = 1/J*pushforward(C,F,Fbar);
            return;

        otherwise
            error('Cannot handle');
            D=[]; % return non-usable value
            return;
    end
end

function D = pushforward (C,F,Fbar)

D(1,1)=C(2,3,1,2)*F(1,2)*F(1,3)*Fbar(1,1)*Fbar(1,2)+C(3,1,2,3)*F(1,3)*F(1,1)*Fbar(1,2)*Fbar(1,3)+C(2,2,1,3)*F(1,2)^2*Fbar(1,1)*Fbar(1,3)+C(3,3,3,1)*F(1,3)^2*Fbar(1,3)*Fbar(1,1)+C(2,2,2,1)*F(1,2)^2*Fbar(1,2)*Fbar(1,1)+C(2,1,2,2)*F(1,2)*F(1,1)*Fbar(1,2)^2+C(2,1,3,3)*F(1,2)*F(1,1)*Fbar(1,3)^2+C(3,1,3,3)*F(1,3)*F(1,1)*Fbar(1,3)^2+C(1,1,1,3)*F(1,1)^2*Fbar(1,1)*Fbar(1,3)+C(1,1,2,1)*F(1,1)^2*Fbar(1,2)*Fbar(1,1)+C(1,1,1,2)*F(1,1)^2*Fbar(1,1)*Fbar(1,2)+C(1,3,3,3)*F(1,1)*F(1,3)*Fbar(1,3)^2+C(1,3,3,2)*F(1,1)*F(1,3)*Fbar(1,3)*Fbar(1,2)+C(3,3,1,3)*F(1,3)^2*Fbar(1,1)*Fbar(1,3)+C(2,3,1,1)*F(1,2)*F(1,3)*Fbar(1,1)^2+C(2,2,3,2)*F(1,2)^2*Fbar(1,3)*Fbar(1,2)+C(2,3,2,2)*F(1,2)*F(1,3)*Fbar(1,2)^2+C(3,1,1,1)*F(1,3)*F(1,1)*Fbar(1,1)^2+C(2,2,3,1)*F(1,2)^2*Fbar(1,3)*Fbar(1,1)+C(1,2,3,1)*F(1,1)*F(1,2)*Fbar(1,3)*Fbar(1,1)+C(1,3,1,3)*F(1,1)*F(1,3)*Fbar(1,1)*Fbar(1,3)+C(1,3,2,1)*F(1,1)*F(1,3)*Fbar(1,2)*Fbar(1,1)+C(3,1,1,3)*F(1,3)*F(1,1)*Fbar(1,1)*Fbar(1,3)+C(3,2,2,3)*F(1,3)*F(1,2)*Fbar(1,2)*Fbar(1,3)+C(3,1,3,1)*F(1,3)*F(1,1)*Fbar(1,3)*Fbar(1,1)+C(1,3,1,2)*F(1,1)*F(1,3)*Fbar(1,1)*Fbar(1,2)+C(2,3,2,3)*F(1,2)*F(1,3)*Fbar(1,2)*Fbar(1,3)+C(2,1,1,2)*F(1,2)*F(1,1)*Fbar(1,1)*Fbar(1,2)+C(1,2,1,3)*F(1,1)*F(1,2)*Fbar(1,1)*Fbar(1,3)+C(3,2,1,2)*F(1,3)*F(1,2)*Fbar(1,1)*Fbar(1,2)+C(1,2,2,1)*F(1,1)*F(1,2)*Fbar(1,2)*Fbar(1,1)+C(3,3,2,2)*F(1,3)^2*Fbar(1,2)^2+C(2,2,1,1)*F(1,2)^2*Fbar(1,1)^2+C(2,2,3,3)*F(1,2)^2*Fbar(1,3)^2+C(1,1,1,1)*F(1,1)^2*Fbar(1,1)^2+C(3,3,3,3)*F(1,3)^2*Fbar(1,3)^2+C(1,1,3,3)*F(1,1)^2*Fbar(1,3)^2+C(1,1,2,2)*F(1,1)^2*Fbar(1,2)^2+C(3,3,1,1)*F(1,3)^2*Fbar(1,1)^2+C(2,2,2,2)*F(1,2)^2*Fbar(1,2)^2+C(3,2,2,2)*F(1,3)*F(1,2)*Fbar(1,2)^2+C(3,1,2,2)*F(1,3)*F(1,1)*Fbar(1,2)^2+C(3,3,2,1)*F(1,3)^2*Fbar(1,2)*Fbar(1,1)+C(1,1,3,1)*F(1,1)^2*Fbar(1,3)*Fbar(1,1)+C(3,3,2,3)*F(1,3)^2*Fbar(1,2)*Fbar(1,3)+C(3,3,1,2)*F(1,3)^2*Fbar(1,1)*Fbar(1,2)+C(3,2,1,1)*F(1,3)*F(1,2)*Fbar(1,1)^2+C(2,2,1,2)*F(1,2)^2*Fbar(1,1)*Fbar(1,2)+C(2,2,2,3)*F(1,2)^2*Fbar(1,2)*Fbar(1,3)+C(1,3,2,2)*F(1,1)*F(1,3)*Fbar(1,2)^2+C(1,2,3,2)*F(1,1)*F(1,2)*Fbar(1,3)*Fbar(1,2)+C(3,2,1,3)*F(1,3)*F(1,2)*Fbar(1,1)*Fbar(1,3)+C(3,1,2,1)*F(1,3)*F(1,1)*Fbar(1,2)*Fbar(1,1)+C(2,1,3,2)*F(1,2)*F(1,1)*Fbar(1,3)*Fbar(1,2)+C(2,1,1,3)*F(1,2)*F(1,1)*Fbar(1,1)*Fbar(1,3)+C(2,3,2,1)*F(1,2)*F(1,3)*Fbar(1,2)*Fbar(1,1)+C(2,1,2,3)*F(1,2)*F(1,1)*Fbar(1,2)*Fbar(1,3)+C(2,3,3,1)*F(1,2)*F(1,3)*Fbar(1,3)*Fbar(1,1)+C(2,1,2,1)*F(1,2)*F(1,1)*Fbar(1,2)*Fbar(1,1)+C(3,2,3,1)*F(1,3)*F(1,2)*Fbar(1,3)*Fbar(1,1)+C(3,2,3,2)*F(1,3)*F(1,2)*Fbar(1,3)*Fbar(1,2)+C(2,3,1,3)*F(1,2)*F(1,3)*Fbar(1,1)*Fbar(1,3)+C(3,2,2,1)*F(1,3)*F(1,2)*Fbar(1,2)*Fbar(1,1)+C(1,2,2,3)*F(1,1)*F(1,2)*Fbar(1,2)*Fbar(1,3)+C(3,1,1,2)*F(1,3)*F(1,1)*Fbar(1,1)*Fbar(1,2)+C(2,1,3,1)*F(1,2)*F(1,1)*Fbar(1,3)*Fbar(1,1)+C(2,3,3,2)*F(1,2)*F(1,3)*Fbar(1,3)*Fbar(1,2)+C(3,1,3,2)*F(1,3)*F(1,1)*Fbar(1,3)*Fbar(1,2)+C(1,3,2,3)*F(1,1)*F(1,3)*Fbar(1,2)*Fbar(1,3)+C(1,3,3,1)*F(1,1)*F(1,3)*Fbar(1,3)*Fbar(1,1)+C(3,3,3,2)*F(1,3)^2*Fbar(1,3)*Fbar(1,2)+C(1,2,1,1)*F(1,1)*F(1,2)*Fbar(1,1)^2+C(1,2,3,3)*F(1,1)*F(1,2)*Fbar(1,3)^2+C(3,2,3,3)*F(1,3)*F(1,2)*Fbar(1,3)^2+C(1,3,1,1)*F(1,1)*F(1,3)*Fbar(1,1)^2+C(1,2,2,2)*F(1,1)*F(1,2)*Fbar(1,2)^2+C(1,1,2,3)*F(1,1)^2*Fbar(1,2)*Fbar(1,3)+C(1,1,3,2)*F(1,1)^2*Fbar(1,3)*Fbar(1,2)+C(2,1,1,1)*F(1,2)*F(1,1)*Fbar(1,1)^2+C(2,3,3,3)*F(1,2)*F(1,3)*Fbar(1,3)^2+C(1,2,1,2)*F(1,1)*F(1,2)*Fbar(1,1)*Fbar(1,2);
D(1,2)=C(1,1,1,1)*F(1,1)^2*Fbar(2,1)^2+C(3,3,2,2)*F(1,3)^2*Fbar(2,2)^2+C(2,2,2,2)*F(1,2)^2*Fbar(2,2)^2+C(2,1,3,2)*F(1,2)*F(1,1)*Fbar(2,3)*Fbar(2,2)+C(3,1,2,1)*F(1,3)*F(1,1)*Fbar(2,2)*Fbar(2,1)+C(2,3,1,2)*F(1,2)*F(1,3)*Fbar(2,1)*Fbar(2,2)+C(1,2,2,3)*F(1,1)*F(1,2)*Fbar(2,2)*Fbar(2,3)+C(2,1,2,3)*F(1,2)*F(1,1)*Fbar(2,2)*Fbar(2,3)+C(1,3,2,1)*F(1,1)*F(1,3)*Fbar(2,2)*Fbar(2,1)+C(1,2,2,1)*F(1,1)*F(1,2)*Fbar(2,2)*Fbar(2,1)+C(2,1,3,1)*F(1,2)*F(1,1)*Fbar(2,3)*Fbar(2,1)+C(2,3,2,3)*F(1,2)*F(1,3)*Fbar(2,2)*Fbar(2,3)+C(2,1,1,2)*F(1,2)*F(1,1)*Fbar(2,1)*Fbar(2,2)+C(2,3,1,3)*F(1,2)*F(1,3)*Fbar(2,1)*Fbar(2,3)+C(1,3,3,1)*F(1,1)*F(1,3)*Fbar(2,3)*Fbar(2,1)+C(3,2,1,3)*F(1,3)*F(1,2)*Fbar(2,1)*Fbar(2,3)+C(2,3,2,1)*F(1,2)*F(1,3)*Fbar(2,2)*Fbar(2,1)+C(3,2,2,1)*F(1,3)*F(1,2)*Fbar(2,2)*Fbar(2,1)+C(1,2,1,2)*F(1,1)*F(1,2)*Fbar(2,1)*Fbar(2,2)+C(1,2,1,3)*F(1,1)*F(1,2)*Fbar(2,1)*Fbar(2,3)+C(3,1,1,2)*F(1,3)*F(1,1)*Fbar(2,1)*Fbar(2,2)+C(2,1,1,3)*F(1,2)*F(1,1)*Fbar(2,1)*Fbar(2,3)+C(1,3,1,2)*F(1,1)*F(1,3)*Fbar(2,1)*Fbar(2,2)+C(1,3,3,2)*F(1,1)*F(1,3)*Fbar(2,3)*Fbar(2,2)+C(1,3,2,3)*F(1,1)*F(1,3)*Fbar(2,2)*Fbar(2,3)+C(2,1,2,1)*F(1,2)*F(1,1)*Fbar(2,2)*Fbar(2,1)+C(1,2,3,1)*F(1,1)*F(1,2)*Fbar(2,3)*Fbar(2,1)+C(3,1,3,2)*F(1,3)*F(1,1)*Fbar(2,3)*Fbar(2,2)+C(2,2,1,2)*F(1,2)^2*Fbar(2,1)*Fbar(2,2)+C(1,1,2,3)*F(1,1)^2*Fbar(2,2)*Fbar(2,3)+C(1,1,1,2)*F(1,1)^2*Fbar(2,1)*Fbar(2,2)+C(1,1,1,3)*F(1,1)^2*Fbar(2,1)*Fbar(2,3)+C(1,1,2,1)*F(1,1)^2*Fbar(2,2)*Fbar(2,1)+C(2,2,3,2)*F(1,2)^2*Fbar(2,3)*Fbar(2,2)+C(3,1,3,1)*F(1,3)*F(1,1)*Fbar(2,3)*Fbar(2,1)+C(1,3,1,3)*F(1,1)*F(1,3)*Fbar(2,1)*Fbar(2,3)+C(3,2,3,1)*F(1,3)*F(1,2)*Fbar(2,3)*Fbar(2,1)+C(2,3,3,1)*F(1,2)*F(1,3)*Fbar(2,3)*Fbar(2,1)+C(3,1,1,3)*F(1,3)*F(1,1)*Fbar(2,1)*Fbar(2,3)+C(3,1,2,3)*F(1,3)*F(1,1)*Fbar(2,2)*Fbar(2,3)+C(3,2,3,2)*F(1,3)*F(1,2)*Fbar(2,3)*Fbar(2,2)+C(1,2,3,2)*F(1,1)*F(1,2)*Fbar(2,3)*Fbar(2,2)+C(3,2,1,2)*F(1,3)*F(1,2)*Fbar(2,1)*Fbar(2,2)+C(3,2,2,3)*F(1,3)*F(1,2)*Fbar(2,2)*Fbar(2,3)+C(3,3,3,3)*F(1,3)^2*Fbar(2,3)^2+C(3,3,2,3)*F(1,3)^2*Fbar(2,2)*Fbar(2,3)+C(3,1,3,3)*F(1,3)*F(1,1)*Fbar(2,3)^2+C(1,3,2,2)*F(1,1)*F(1,3)*Fbar(2,2)^2+C(1,3,1,1)*F(1,1)*F(1,3)*Fbar(2,1)^2+C(2,2,2,1)*F(1,2)^2*Fbar(2,2)*Fbar(2,1)+C(3,1,2,2)*F(1,3)*F(1,1)*Fbar(2,2)^2+C(2,3,2,2)*F(1,2)*F(1,3)*Fbar(2,2)^2+C(3,2,2,2)*F(1,3)*F(1,2)*Fbar(2,2)^2+C(3,2,1,1)*F(1,3)*F(1,2)*Fbar(2,1)^2+C(3,3,1,2)*F(1,3)^2*Fbar(2,1)*Fbar(2,2)+C(2,1,3,3)*F(1,2)*F(1,1)*Fbar(2,3)^2+C(3,3,3,1)*F(1,3)^2*Fbar(2,3)*Fbar(2,1)+C(1,1,3,1)*F(1,1)^2*Fbar(2,3)*Fbar(2,1)+C(2,3,3,3)*F(1,2)*F(1,3)*Fbar(2,3)^2+C(1,1,3,2)*F(1,1)^2*Fbar(2,3)*Fbar(2,2)+C(1,2,2,2)*F(1,1)*F(1,2)*Fbar(2,2)^2+C(3,2,3,3)*F(1,3)*F(1,2)*Fbar(2,3)^2+C(1,2,1,1)*F(1,1)*F(1,2)*Fbar(2,1)^2+C(2,1,1,1)*F(1,2)*F(1,1)*Fbar(2,1)^2+C(1,3,3,3)*F(1,1)*F(1,3)*Fbar(2,3)^2+C(3,3,2,1)*F(1,3)^2*Fbar(2,2)*Fbar(2,1)+C(2,2,3,3)*F(1,2)^2*Fbar(2,3)^2+C(1,1,2,2)*F(1,1)^2*Fbar(2,2)^2+C(1,1,3,3)*F(1,1)^2*Fbar(2,3)^2+C(3,3,3,2)*F(1,3)^2*Fbar(2,3)*Fbar(2,2)+C(2,1,2,2)*F(1,2)*F(1,1)*Fbar(2,2)^2+C(3,1,1,1)*F(1,3)*F(1,1)*Fbar(2,1)^2+C(2,2,2,3)*F(1,2)^2*Fbar(2,2)*Fbar(2,3)+C(1,2,3,3)*F(1,1)*F(1,2)*Fbar(2,3)^2+C(2,2,1,3)*F(1,2)^2*Fbar(2,1)*Fbar(2,3)+C(2,3,1,1)*F(1,2)*F(1,3)*Fbar(2,1)^2+C(3,3,1,3)*F(1,3)^2*Fbar(2,1)*Fbar(2,3)+C(2,2,3,1)*F(1,2)^2*Fbar(2,3)*Fbar(2,1)+C(2,3,3,2)*F(1,2)*F(1,3)*Fbar(2,3)*Fbar(2,2)+C(3,3,1,1)*F(1,3)^2*Fbar(2,1)^2+C(2,2,1,1)*F(1,2)^2*Fbar(2,1)^2;
D(1,3)=C(1,1,2,2)*F(1,1)^2*Fbar(3,2)^2+C(1,1,3,3)*F(1,1)^2*Fbar(3,3)^2+C(3,2,2,3)*F(1,3)*F(1,2)*Fbar(3,2)*Fbar(3,3)+C(3,1,3,1)*F(1,3)*F(1,1)*Fbar(3,3)*Fbar(3,1)+C(2,3,3,1)*F(1,2)*F(1,3)*Fbar(3,3)*Fbar(3,1)+C(3,2,2,1)*F(1,3)*F(1,2)*Fbar(3,2)*Fbar(3,1)+C(1,3,3,1)*F(1,1)*F(1,3)*Fbar(3,3)*Fbar(3,1)+C(1,2,3,1)*F(1,1)*F(1,2)*Fbar(3,3)*Fbar(3,1)+C(3,2,1,2)*F(1,3)*F(1,2)*Fbar(3,1)*Fbar(3,2)+C(1,3,3,2)*F(1,1)*F(1,3)*Fbar(3,3)*Fbar(3,2)+C(2,1,1,3)*F(1,2)*F(1,1)*Fbar(3,1)*Fbar(3,3)+C(3,2,3,1)*F(1,3)*F(1,2)*Fbar(3,3)*Fbar(3,1)+C(3,1,1,2)*F(1,3)*F(1,1)*Fbar(3,1)*Fbar(3,2)+C(2,3,1,3)*F(1,2)*F(1,3)*Fbar(3,1)*Fbar(3,3)+C(3,1,2,3)*F(1,3)*F(1,1)*Fbar(3,2)*Fbar(3,3)+C(2,1,2,1)*F(1,2)*F(1,1)*Fbar(3,2)*Fbar(3,1)+C(1,2,2,1)*F(1,1)*F(1,2)*Fbar(3,2)*Fbar(3,1)+C(1,2,1,2)*F(1,1)*F(1,2)*Fbar(3,1)*Fbar(3,2)+C(2,1,2,3)*F(1,2)*F(1,1)*Fbar(3,2)*Fbar(3,3)+C(1,3,2,3)*F(1,1)*F(1,3)*Fbar(3,2)*Fbar(3,3)+C(1,3,1,3)*F(1,1)*F(1,3)*Fbar(3,1)*Fbar(3,3)+C(2,3,2,3)*F(1,2)*F(1,3)*Fbar(3,2)*Fbar(3,3)+C(3,1,1,3)*F(1,3)*F(1,1)*Fbar(3,1)*Fbar(3,3)+C(2,3,1,2)*F(1,2)*F(1,3)*Fbar(3,1)*Fbar(3,2)+C(3,2,1,3)*F(1,3)*F(1,2)*Fbar(3,1)*Fbar(3,3)+C(1,2,2,3)*F(1,1)*F(1,2)*Fbar(3,2)*Fbar(3,3)+C(2,1,3,2)*F(1,2)*F(1,1)*Fbar(3,3)*Fbar(3,2)+C(2,1,3,1)*F(1,2)*F(1,1)*Fbar(3,3)*Fbar(3,1)+C(3,1,3,2)*F(1,3)*F(1,1)*Fbar(3,3)*Fbar(3,2)+C(1,2,3,2)*F(1,1)*F(1,2)*Fbar(3,3)*Fbar(3,2)+C(3,2,3,2)*F(1,3)*F(1,2)*Fbar(3,3)*Fbar(3,2)+C(2,1,1,2)*F(1,2)*F(1,1)*Fbar(3,1)*Fbar(3,2)+C(2,3,3,2)*F(1,2)*F(1,3)*Fbar(3,3)*Fbar(3,2)+C(1,3,1,2)*F(1,1)*F(1,3)*Fbar(3,1)*Fbar(3,2)+C(1,2,1,3)*F(1,1)*F(1,2)*Fbar(3,1)*Fbar(3,3)+C(3,1,2,1)*F(1,3)*F(1,1)*Fbar(3,2)*Fbar(3,1)+C(3,3,3,3)*F(1,3)^2*Fbar(3,3)^2+C(2,3,2,1)*F(1,2)*F(1,3)*Fbar(3,2)*Fbar(3,1)+C(3,3,1,1)*F(1,3)^2*Fbar(3,1)^2+C(3,2,3,3)*F(1,3)*F(1,2)*Fbar(3,3)^2+C(1,1,1,3)*F(1,1)^2*Fbar(3,1)*Fbar(3,3)+C(1,1,2,1)*F(1,1)^2*Fbar(3,2)*Fbar(3,1)+C(3,3,3,1)*F(1,3)^2*Fbar(3,3)*Fbar(3,1)+C(2,2,2,2)*F(1,2)^2*Fbar(3,2)^2+C(1,1,1,1)*F(1,1)^2*Fbar(3,1)^2+C(2,2,1,1)*F(1,2)^2*Fbar(3,1)^2+C(2,2,1,2)*F(1,2)^2*Fbar(3,1)*Fbar(3,2)+C(2,1,1,1)*F(1,2)*F(1,1)*Fbar(3,1)^2+C(2,2,2,1)*F(1,2)^2*Fbar(3,2)*Fbar(3,1)+C(3,3,2,1)*F(1,3)^2*Fbar(3,2)*Fbar(3,1)+C(3,1,3,3)*F(1,3)*F(1,1)*Fbar(3,3)^2+C(1,2,3,3)*F(1,1)*F(1,2)*Fbar(3,3)^2+C(2,3,1,1)*F(1,2)*F(1,3)*Fbar(3,1)^2+C(1,3,3,3)*F(1,1)*F(1,3)*Fbar(3,3)^2+C(1,1,2,3)*F(1,1)^2*Fbar(3,2)*Fbar(3,3)+C(3,3,1,3)*F(1,3)^2*Fbar(3,1)*Fbar(3,3)+C(2,3,3,3)*F(1,2)*F(1,3)*Fbar(3,3)^2+C(1,1,3,1)*F(1,1)^2*Fbar(3,3)*Fbar(3,1)+C(2,2,3,3)*F(1,2)^2*Fbar(3,3)^2+C(3,1,1,1)*F(1,3)*F(1,1)*Fbar(3,1)^2+C(1,1,1,2)*F(1,1)^2*Fbar(3,1)*Fbar(3,2)+C(3,3,2,2)*F(1,3)^2*Fbar(3,2)^2+C(2,2,3,1)*F(1,2)^2*Fbar(3,3)*Fbar(3,1)+C(3,3,3,2)*F(1,3)^2*Fbar(3,3)*Fbar(3,2)+C(2,2,2,3)*F(1,2)^2*Fbar(3,2)*Fbar(3,3)+C(1,3,2,2)*F(1,1)*F(1,3)*Fbar(3,2)^2+C(3,2,2,2)*F(1,3)*F(1,2)*Fbar(3,2)^2+C(2,1,3,3)*F(1,2)*F(1,1)*Fbar(3,3)^2+C(1,3,1,1)*F(1,1)*F(1,3)*Fbar(3,1)^2+C(1,2,1,1)*F(1,1)*F(1,2)*Fbar(3,1)^2+C(2,3,2,2)*F(1,2)*F(1,3)*Fbar(3,2)^2+C(1,1,3,2)*F(1,1)^2*Fbar(3,3)*Fbar(3,2)+C(3,3,1,2)*F(1,3)^2*Fbar(3,1)*Fbar(3,2)+C(2,2,1,3)*F(1,2)^2*Fbar(3,1)*Fbar(3,3)+C(2,2,3,2)*F(1,2)^2*Fbar(3,3)*Fbar(3,2)+C(1,2,2,2)*F(1,1)*F(1,2)*Fbar(3,2)^2+C(3,3,2,3)*F(1,3)^2*Fbar(3,2)*Fbar(3,3)+C(3,2,1,1)*F(1,3)*F(1,2)*Fbar(3,1)^2+C(3,1,2,2)*F(1,3)*F(1,1)*Fbar(3,2)^2+C(2,1,2,2)*F(1,2)*F(1,1)*Fbar(3,2)^2+C(1,3,2,1)*F(1,1)*F(1,3)*Fbar(3,2)*Fbar(3,1);
D(1,4)=C(1,2,1,2)*F(1,1)*F(1,2)*Fbar(2,1)*Fbar(1,2)+C(1,3,2,3)*F(1,1)*F(1,3)*Fbar(2,2)*Fbar(1,3)+C(1,2,2,2)*F(1,1)*F(1,2)*Fbar(2,2)*Fbar(1,2)+C(2,3,2,1)*F(1,2)*F(1,3)*Fbar(2,2)*Fbar(1,1)+C(3,1,2,1)*F(1,3)*F(1,1)*Fbar(2,2)*Fbar(1,1)+C(1,1,1,1)*F(1,1)^2*Fbar(2,1)*Fbar(1,1)+C(2,3,2,2)*F(1,2)*F(1,3)*Fbar(2,2)*Fbar(1,2)+C(1,3,3,1)*F(1,1)*F(1,3)*Fbar(2,3)*Fbar(1,1)+C(1,2,1,3)*F(1,1)*F(1,2)*Fbar(2,1)*Fbar(1,3)+C(1,2,3,3)*F(1,1)*F(1,2)*Fbar(2,3)*Fbar(1,3)+C(1,2,3,2)*F(1,1)*F(1,2)*Fbar(2,3)*Fbar(1,2)+C(1,1,3,1)*F(1,1)^2*Fbar(2,3)*Fbar(1,1)+C(1,1,1,2)*F(1,1)^2*Fbar(2,1)*Fbar(1,2)+C(3,3,3,3)*F(1,3)^2*Fbar(2,3)*Fbar(1,3)+C(3,3,3,2)*F(1,3)^2*Fbar(2,3)*Fbar(1,2)+C(2,2,1,3)*F(1,2)^2*Fbar(2,1)*Fbar(1,3)+C(1,1,2,3)*F(1,1)^2*Fbar(2,2)*Fbar(1,3)+C(3,3,1,2)*F(1,3)^2*Fbar(2,1)*Fbar(1,2)+C(3,3,3,1)*F(1,3)^2*Fbar(2,3)*Fbar(1,1)+C(3,3,2,1)*F(1,3)^2*Fbar(2,2)*Fbar(1,1)+C(2,2,3,2)*F(1,2)^2*Fbar(2,3)*Fbar(1,2)+C(1,1,2,1)*F(1,1)^2*Fbar(2,2)*Fbar(1,1)+C(1,1,3,2)*F(1,1)^2*Fbar(2,3)*Fbar(1,2)+C(2,2,3,1)*F(1,2)^2*Fbar(2,3)*Fbar(1,1)+C(3,3,2,2)*F(1,3)^2*Fbar(2,2)*Fbar(1,2)+C(3,3,1,3)*F(1,3)^2*Fbar(2,1)*Fbar(1,3)+C(1,1,1,3)*F(1,1)^2*Fbar(2,1)*Fbar(1,3)+C(2,1,1,2)*F(1,2)*F(1,1)*Fbar(2,1)*Fbar(1,2)+C(1,3,1,1)*F(1,1)*F(1,3)*Fbar(2,1)*Fbar(1,1)+C(2,3,3,3)*F(1,2)*F(1,3)*Fbar(2,3)*Fbar(1,3)+C(1,3,3,3)*F(1,1)*F(1,3)*Fbar(2,3)*Fbar(1,3)+C(3,2,2,2)*F(1,3)*F(1,2)*Fbar(2,2)*Fbar(1,2)+C(2,1,2,2)*F(1,2)*F(1,1)*Fbar(2,2)*Fbar(1,2)+C(3,2,3,1)*F(1,3)*F(1,2)*Fbar(2,3)*Fbar(1,1)+C(3,1,3,1)*F(1,3)*F(1,1)*Fbar(2,3)*Fbar(1,1)+C(3,1,1,3)*F(1,3)*F(1,1)*Fbar(2,1)*Fbar(1,3)+C(3,1,1,2)*F(1,3)*F(1,1)*Fbar(2,1)*Fbar(1,2)+C(2,1,3,1)*F(1,2)*F(1,1)*Fbar(2,3)*Fbar(1,1)+C(3,2,1,3)*F(1,3)*F(1,2)*Fbar(2,1)*Fbar(1,3)+C(2,1,2,3)*F(1,2)*F(1,1)*Fbar(2,2)*Fbar(1,3)+C(1,3,1,3)*F(1,1)*F(1,3)*Fbar(2,1)*Fbar(1,3)+C(3,2,2,1)*F(1,3)*F(1,2)*Fbar(2,2)*Fbar(1,1)+C(1,3,2,1)*F(1,1)*F(1,3)*Fbar(2,2)*Fbar(1,1)+C(2,3,3,2)*F(1,2)*F(1,3)*Fbar(2,3)*Fbar(1,2)+C(1,3,1,2)*F(1,1)*F(1,3)*Fbar(2,1)*Fbar(1,2)+C(3,1,1,1)*F(1,3)*F(1,1)*Fbar(2,1)*Fbar(1,1)+C(3,1,2,2)*F(1,3)*F(1,1)*Fbar(2,2)*Fbar(1,2)+C(3,1,2,3)*F(1,3)*F(1,1)*Fbar(2,2)*Fbar(1,3)+C(3,2,3,2)*F(1,3)*F(1,2)*Fbar(2,3)*Fbar(1,2)+C(3,2,1,2)*F(1,3)*F(1,2)*Fbar(2,1)*Fbar(1,2)+C(2,3,2,3)*F(1,2)*F(1,3)*Fbar(2,2)*Fbar(1,3)+C(2,3,3,1)*F(1,2)*F(1,3)*Fbar(2,3)*Fbar(1,1)+C(3,1,3,3)*F(1,3)*F(1,1)*Fbar(2,3)*Fbar(1,3)+C(2,1,1,1)*F(1,2)*F(1,1)*Fbar(2,1)*Fbar(1,1)+C(2,1,3,2)*F(1,2)*F(1,1)*Fbar(2,3)*Fbar(1,2)+C(3,2,2,3)*F(1,3)*F(1,2)*Fbar(2,2)*Fbar(1,3)+C(1,3,2,2)*F(1,1)*F(1,3)*Fbar(2,2)*Fbar(1,2)+C(2,1,2,1)*F(1,2)*F(1,1)*Fbar(2,2)*Fbar(1,1)+C(1,2,2,3)*F(1,1)*F(1,2)*Fbar(2,2)*Fbar(1,3)+C(2,1,3,3)*F(1,2)*F(1,1)*Fbar(2,3)*Fbar(1,3)+C(2,3,1,2)*F(1,2)*F(1,3)*Fbar(2,1)*Fbar(1,2)+C(3,1,3,2)*F(1,3)*F(1,1)*Fbar(2,3)*Fbar(1,2)+C(1,2,2,1)*F(1,1)*F(1,2)*Fbar(2,2)*Fbar(1,1)+C(2,1,1,3)*F(1,2)*F(1,1)*Fbar(2,1)*Fbar(1,3)+C(3,2,1,1)*F(1,3)*F(1,2)*Fbar(2,1)*Fbar(1,1)+C(1,3,3,2)*F(1,1)*F(1,3)*Fbar(2,3)*Fbar(1,2)+C(3,2,3,3)*F(1,3)*F(1,2)*Fbar(2,3)*Fbar(1,3)+C(2,3,1,3)*F(1,2)*F(1,3)*Fbar(2,1)*Fbar(1,3)+C(1,2,3,1)*F(1,1)*F(1,2)*Fbar(2,3)*Fbar(1,1)+C(2,3,1,1)*F(1,2)*F(1,3)*Fbar(2,1)*Fbar(1,1)+C(1,2,1,1)*F(1,1)*F(1,2)*Fbar(2,1)*Fbar(1,1)+C(3,3,1,1)*F(1,3)^2*Fbar(2,1)*Fbar(1,1)+C(1,1,2,2)*F(1,1)^2*Fbar(2,2)*Fbar(1,2)+C(2,2,2,1)*F(1,2)^2*Fbar(2,2)*Fbar(1,1)+C(2,2,1,2)*F(1,2)^2*Fbar(2,1)*Fbar(1,2)+C(2,2,2,3)*F(1,2)^2*Fbar(2,2)*Fbar(1,3)+C(2,2,2,2)*F(1,2)^2*Fbar(2,2)*Fbar(1,2)+C(2,2,1,1)*F(1,2)^2*Fbar(2,1)*Fbar(1,1)+C(2,2,3,3)*F(1,2)^2*Fbar(2,3)*Fbar(1,3)+C(3,3,2,3)*F(1,3)^2*Fbar(2,2)*Fbar(1,3)+C(1,1,3,3)*F(1,1)^2*Fbar(2,3)*Fbar(1,3);
D(1,5)=C(3,1,1,2)*F(1,3)*F(1,1)*Fbar(3,1)*Fbar(1,2)+C(3,1,3,2)*F(1,3)*F(1,1)*Fbar(3,3)*Fbar(1,2)+C(2,1,1,1)*F(1,2)*F(1,1)*Fbar(3,1)*Fbar(1,1)+C(1,3,1,1)*F(1,1)*F(1,3)*Fbar(3,1)*Fbar(1,1)+C(3,2,1,3)*F(1,3)*F(1,2)*Fbar(3,1)*Fbar(1,3)+C(1,3,2,1)*F(1,1)*F(1,3)*Fbar(3,2)*Fbar(1,1)+C(2,2,2,2)*F(1,2)^2*Fbar(3,2)*Fbar(1,2)+C(2,2,2,3)*F(1,2)^2*Fbar(3,2)*Fbar(1,3)+C(1,1,1,2)*F(1,1)^2*Fbar(3,1)*Fbar(1,2)+C(1,1,1,3)*F(1,1)^2*Fbar(3,1)*Fbar(1,3)+C(2,2,2,1)*F(1,2)^2*Fbar(3,2)*Fbar(1,1)+C(1,1,1,1)*F(1,1)^2*Fbar(3,1)*Fbar(1,1)+C(2,2,1,3)*F(1,2)^2*Fbar(3,1)*Fbar(1,3)+C(1,1,2,2)*F(1,1)^2*Fbar(3,2)*Fbar(1,2)+C(1,1,2,1)*F(1,1)^2*Fbar(3,2)*Fbar(1,1)+C(2,2,1,2)*F(1,2)^2*Fbar(3,1)*Fbar(1,2)+C(2,2,3,1)*F(1,2)^2*Fbar(3,3)*Fbar(1,1)+C(1,3,3,2)*F(1,1)*F(1,3)*Fbar(3,3)*Fbar(1,2)+C(1,2,1,2)*F(1,1)*F(1,2)*Fbar(3,1)*Fbar(1,2)+C(3,2,2,1)*F(1,3)*F(1,2)*Fbar(3,2)*Fbar(1,1)+C(2,3,1,2)*F(1,2)*F(1,3)*Fbar(3,1)*Fbar(1,2)+C(3,2,1,2)*F(1,3)*F(1,2)*Fbar(3,1)*Fbar(1,2)+C(3,2,3,1)*F(1,3)*F(1,2)*Fbar(3,3)*Fbar(1,1)+C(3,2,3,3)*F(1,3)*F(1,2)*Fbar(3,3)*Fbar(1,3)+C(1,2,1,1)*F(1,1)*F(1,2)*Fbar(3,1)*Fbar(1,1)+C(3,1,1,1)*F(1,3)*F(1,1)*Fbar(3,1)*Fbar(1,1)+C(1,2,2,2)*F(1,1)*F(1,2)*Fbar(3,2)*Fbar(1,2)+C(3,1,1,3)*F(1,3)*F(1,1)*Fbar(3,1)*Fbar(1,3)+C(1,2,3,2)*F(1,1)*F(1,2)*Fbar(3,3)*Fbar(1,2)+C(2,1,2,1)*F(1,2)*F(1,1)*Fbar(3,2)*Fbar(1,1)+C(2,3,3,1)*F(1,2)*F(1,3)*Fbar(3,3)*Fbar(1,1)+C(1,3,3,3)*F(1,1)*F(1,3)*Fbar(3,3)*Fbar(1,3)+C(1,3,1,3)*F(1,1)*F(1,3)*Fbar(3,1)*Fbar(1,3)+C(2,1,2,2)*F(1,2)*F(1,1)*Fbar(3,2)*Fbar(1,2)+C(2,1,3,1)*F(1,2)*F(1,1)*Fbar(3,3)*Fbar(1,1)+C(3,1,3,1)*F(1,3)*F(1,1)*Fbar(3,3)*Fbar(1,1)+C(1,1,2,3)*F(1,1)^2*Fbar(3,2)*Fbar(1,3)+C(2,2,3,2)*F(1,2)^2*Fbar(3,3)*Fbar(1,2)+C(2,2,1,1)*F(1,2)^2*Fbar(3,1)*Fbar(1,1)+C(3,3,1,3)*F(1,3)^2*Fbar(3,1)*Fbar(1,3)+C(3,3,3,3)*F(1,3)^2*Fbar(3,3)*Fbar(1,3)+C(2,2,3,3)*F(1,2)^2*Fbar(3,3)*Fbar(1,3)+C(1,1,3,1)*F(1,1)^2*Fbar(3,3)*Fbar(1,1)+C(2,1,3,2)*F(1,2)*F(1,1)*Fbar(3,3)*Fbar(1,2)+C(1,3,1,2)*F(1,1)*F(1,3)*Fbar(3,1)*Fbar(1,2)+C(2,3,2,3)*F(1,2)*F(1,3)*Fbar(3,2)*Fbar(1,3)+C(2,3,3,3)*F(1,2)*F(1,3)*Fbar(3,3)*Fbar(1,3)+C(2,1,3,3)*F(1,2)*F(1,1)*Fbar(3,3)*Fbar(1,3)+C(1,3,2,3)*F(1,1)*F(1,3)*Fbar(3,2)*Fbar(1,3)+C(2,3,1,1)*F(1,2)*F(1,3)*Fbar(3,1)*Fbar(1,1)+C(3,2,2,2)*F(1,3)*F(1,2)*Fbar(3,2)*Fbar(1,2)+C(1,3,3,1)*F(1,1)*F(1,3)*Fbar(3,3)*Fbar(1,1)+C(3,2,2,3)*F(1,3)*F(1,2)*Fbar(3,2)*Fbar(1,3)+C(2,1,1,2)*F(1,2)*F(1,1)*Fbar(3,1)*Fbar(1,2)+C(2,3,2,1)*F(1,2)*F(1,3)*Fbar(3,2)*Fbar(1,1)+C(2,3,2,2)*F(1,2)*F(1,3)*Fbar(3,2)*Fbar(1,2)+C(3,3,3,2)*F(1,3)^2*Fbar(3,3)*Fbar(1,2)+C(3,3,1,2)*F(1,3)^2*Fbar(3,1)*Fbar(1,2)+C(3,3,2,2)*F(1,3)^2*Fbar(3,2)*Fbar(1,2)+C(3,1,2,3)*F(1,3)*F(1,1)*Fbar(3,2)*Fbar(1,3)+C(2,1,2,3)*F(1,2)*F(1,1)*Fbar(3,2)*Fbar(1,3)+C(2,3,3,2)*F(1,2)*F(1,3)*Fbar(3,3)*Fbar(1,2)+C(1,2,3,3)*F(1,1)*F(1,2)*Fbar(3,3)*Fbar(1,3)+C(3,2,3,2)*F(1,3)*F(1,2)*Fbar(3,3)*Fbar(1,2)+C(3,1,3,3)*F(1,3)*F(1,1)*Fbar(3,3)*Fbar(1,3)+C(3,1,2,1)*F(1,3)*F(1,1)*Fbar(3,2)*Fbar(1,1)+C(2,3,1,3)*F(1,2)*F(1,3)*Fbar(3,1)*Fbar(1,3)+C(1,1,3,2)*F(1,1)^2*Fbar(3,3)*Fbar(1,2)+C(3,3,2,3)*F(1,3)^2*Fbar(3,2)*Fbar(1,3)+C(3,3,1,1)*F(1,3)^2*Fbar(3,1)*Fbar(1,1)+C(1,1,3,3)*F(1,1)^2*Fbar(3,3)*Fbar(1,3)+C(3,3,2,1)*F(1,3)^2*Fbar(3,2)*Fbar(1,1)+C(1,2,2,3)*F(1,1)*F(1,2)*Fbar(3,2)*Fbar(1,3)+C(3,1,2,2)*F(1,3)*F(1,1)*Fbar(3,2)*Fbar(1,2)+C(2,1,1,3)*F(1,2)*F(1,1)*Fbar(3,1)*Fbar(1,3)+C(3,2,1,1)*F(1,3)*F(1,2)*Fbar(3,1)*Fbar(1,1)+C(1,2,1,3)*F(1,1)*F(1,2)*Fbar(3,1)*Fbar(1,3)+C(1,2,3,1)*F(1,1)*F(1,2)*Fbar(3,3)*Fbar(1,1)+C(1,2,2,1)*F(1,1)*F(1,2)*Fbar(3,2)*Fbar(1,1)+C(1,3,2,2)*F(1,1)*F(1,3)*Fbar(3,2)*Fbar(1,2)+C(3,3,3,1)*F(1,3)^2*Fbar(3,3)*Fbar(1,1);
D(1,6)=C(1,2,1,2)*F(1,1)*F(1,2)*Fbar(3,1)*Fbar(2,2)+C(2,3,1,2)*F(1,2)*F(1,3)*Fbar(3,1)*Fbar(2,2)+C(1,1,3,1)*F(1,1)^2*Fbar(3,3)*Fbar(2,1)+C(3,3,1,3)*F(1,3)^2*Fbar(3,1)*Fbar(2,3)+C(2,2,1,1)*F(1,2)^2*Fbar(2,1)*Fbar(3,1)+C(2,2,2,2)*F(1,2)^2*Fbar(2,2)*Fbar(3,2)+C(3,3,2,3)*F(1,3)^2*Fbar(3,2)*Fbar(2,3)+C(2,2,3,2)*F(1,2)^2*Fbar(3,3)*Fbar(2,2)+C(3,3,3,2)*F(1,3)^2*Fbar(3,3)*Fbar(2,2)+C(3,3,3,3)*F(1,3)^2*Fbar(3,3)*Fbar(2,3)+C(3,3,2,2)*F(1,3)^2*Fbar(3,2)*Fbar(2,2)+C(1,1,2,3)*F(1,1)^2*Fbar(3,2)*Fbar(2,3)+C(2,2,2,3)*F(1,2)^2*Fbar(3,2)*Fbar(2,3)+C(2,2,1,2)*F(1,2)^2*Fbar(3,1)*Fbar(2,2)+C(3,3,2,1)*F(1,3)^2*Fbar(3,2)*Fbar(2,1)+C(1,1,1,1)*F(1,1)^2*Fbar(2,1)*Fbar(3,1)+C(2,2,2,1)*F(1,2)^2*Fbar(3,2)*Fbar(2,1)+C(1,1,3,2)*F(1,1)^2*Fbar(3,3)*Fbar(2,2)+C(2,2,3,3)*F(1,2)^2*Fbar(2,3)*Fbar(3,3)+C(2,2,1,3)*F(1,2)^2*Fbar(3,1)*Fbar(2,3)+C(1,1,1,2)*F(1,1)^2*Fbar(3,1)*Fbar(2,2)+C(1,1,1,3)*F(1,1)^2*Fbar(3,1)*Fbar(2,3)+C(1,1,2,1)*F(1,1)^2*Fbar(3,2)*Fbar(2,1)+C(1,1,2,2)*F(1,1)^2*Fbar(2,2)*Fbar(3,2)+C(3,3,3,1)*F(1,3)^2*Fbar(3,3)*Fbar(2,1)+C(3,3,1,2)*F(1,3)^2*Fbar(3,1)*Fbar(2,2)+C(3,3,1,1)*F(1,3)^2*Fbar(3,1)*Fbar(2,1)+C(2,2,3,1)*F(1,2)^2*Fbar(3,3)*Fbar(2,1)+C(1,2,3,1)*F(1,1)*F(1,2)*Fbar(3,3)*Fbar(2,1)+C(1,3,2,3)*F(1,1)*F(1,3)*Fbar(3,2)*Fbar(2,3)+C(1,2,1,1)*F(1,1)*F(1,2)*Fbar(2,1)*Fbar(3,1)+C(3,1,2,2)*F(1,3)*F(1,1)*Fbar(3,2)*Fbar(2,2)+C(3,2,1,3)*F(1,3)*F(1,2)*Fbar(3,1)*Fbar(2,3)+C(2,1,1,1)*F(1,2)*F(1,1)*Fbar(2,1)*Fbar(3,1)+C(3,1,3,3)*F(1,3)*F(1,1)*Fbar(3,3)*Fbar(2,3)+C(3,2,2,3)*F(1,3)*F(1,2)*Fbar(3,2)*Fbar(2,3)+C(3,2,3,1)*F(1,3)*F(1,2)*Fbar(3,3)*Fbar(2,1)+C(3,1,3,1)*F(1,3)*F(1,1)*Fbar(3,3)*Fbar(2,1)+C(3,1,1,2)*F(1,3)*F(1,1)*Fbar(3,1)*Fbar(2,2)+C(1,1,3,3)*F(1,1)^2*Fbar(2,3)*Fbar(3,3)+C(3,1,2,1)*F(1,3)*F(1,1)*Fbar(3,2)*Fbar(2,1)+C(1,2,3,3)*F(1,1)*F(1,2)*Fbar(2,3)*Fbar(3,3)+C(1,3,2,2)*F(1,1)*F(1,3)*Fbar(2,2)*Fbar(3,2)+C(3,1,3,2)*F(1,3)*F(1,1)*Fbar(3,3)*Fbar(2,2)+C(1,3,3,3)*F(1,1)*F(1,3)*Fbar(2,3)*Fbar(3,3)+C(2,3,1,1)*F(1,2)*F(1,3)*Fbar(2,1)*Fbar(3,1)+C(1,3,1,1)*F(1,1)*F(1,3)*Fbar(2,1)*Fbar(3,1)+C(3,2,2,2)*F(1,3)*F(1,2)*Fbar(3,2)*Fbar(2,2)+C(3,2,3,3)*F(1,3)*F(1,2)*Fbar(3,3)*Fbar(2,3)+C(3,2,1,1)*F(1,3)*F(1,2)*Fbar(3,1)*Fbar(2,1)+C(3,1,2,3)*F(1,3)*F(1,1)*Fbar(3,2)*Fbar(2,3)+C(3,2,3,2)*F(1,3)*F(1,2)*Fbar(3,3)*Fbar(2,2)+C(1,2,3,2)*F(1,1)*F(1,2)*Fbar(3,3)*Fbar(2,2)+C(3,2,1,2)*F(1,3)*F(1,2)*Fbar(3,1)*Fbar(2,2)+C(2,1,1,2)*F(1,2)*F(1,1)*Fbar(3,1)*Fbar(2,2)+C(3,1,1,3)*F(1,3)*F(1,1)*Fbar(3,1)*Fbar(2,3)+C(2,1,2,2)*F(1,2)*F(1,1)*Fbar(2,2)*Fbar(3,2)+C(1,3,1,3)*F(1,1)*F(1,3)*Fbar(3,1)*Fbar(2,3)+C(1,3,3,2)*F(1,1)*F(1,3)*Fbar(3,3)*Fbar(2,2)+C(3,1,1,1)*F(1,3)*F(1,1)*Fbar(2,1)*Fbar(3,1)+C(2,1,2,1)*F(1,2)*F(1,1)*Fbar(3,2)*Fbar(2,1)+C(2,3,2,3)*F(1,2)*F(1,3)*Fbar(3,2)*Fbar(2,3)+C(3,2,2,1)*F(1,3)*F(1,2)*Fbar(3,2)*Fbar(2,1)+C(1,3,1,2)*F(1,1)*F(1,3)*Fbar(3,1)*Fbar(2,2)+C(2,3,1,3)*F(1,2)*F(1,3)*Fbar(3,1)*Fbar(2,3)+C(2,3,3,3)*F(1,2)*F(1,3)*Fbar(2,3)*Fbar(3,3)+C(2,3,3,1)*F(1,2)*F(1,3)*Fbar(3,3)*Fbar(2,1)+C(2,3,2,2)*F(1,2)*F(1,3)*Fbar(2,2)*Fbar(3,2)+C(1,2,2,1)*F(1,1)*F(1,2)*Fbar(3,2)*Fbar(2,1)+C(2,1,3,1)*F(1,2)*F(1,1)*Fbar(3,3)*Fbar(2,1)+C(2,1,3,2)*F(1,2)*F(1,1)*Fbar(3,3)*Fbar(2,2)+C(1,2,2,2)*F(1,1)*F(1,2)*Fbar(2,2)*Fbar(3,2)+C(2,1,3,3)*F(1,2)*F(1,1)*Fbar(2,3)*Fbar(3,3)+C(2,3,3,2)*F(1,2)*F(1,3)*Fbar(3,3)*Fbar(2,2)+C(2,1,2,3)*F(1,2)*F(1,1)*Fbar(3,2)*Fbar(2,3)+C(1,2,1,3)*F(1,1)*F(1,2)*Fbar(3,1)*Fbar(2,3)+C(2,3,2,1)*F(1,2)*F(1,3)*Fbar(3,2)*Fbar(2,1)+C(2,1,1,3)*F(1,2)*F(1,1)*Fbar(3,1)*Fbar(2,3)+C(1,2,2,3)*F(1,1)*F(1,2)*Fbar(3,2)*Fbar(2,3)+C(1,3,2,1)*F(1,1)*F(1,3)*Fbar(3,2)*Fbar(2,1)+C(1,3,3,1)*F(1,1)*F(1,3)*Fbar(3,3)*Fbar(2,1);
D(2,1)=C(2,1,3,1)*F(2,2)*F(2,1)*Fbar(1,3)*Fbar(1,1)+C(2,3,3,3)*F(2,2)*F(2,3)*Fbar(1,3)^2+C(1,3,2,2)*F(2,1)*F(2,3)*Fbar(1,2)^2+C(2,3,1,1)*F(2,2)*F(2,3)*Fbar(1,1)^2+C(2,3,2,2)*F(2,2)*F(2,3)*Fbar(1,2)^2+C(2,1,3,2)*F(2,2)*F(2,1)*Fbar(1,3)*Fbar(1,2)+C(1,2,1,2)*F(2,1)*F(2,2)*Fbar(1,1)*Fbar(1,2)+C(1,3,1,2)*F(2,1)*F(2,3)*Fbar(1,1)*Fbar(1,2)+C(1,3,3,1)*F(2,1)*F(2,3)*Fbar(1,3)*Fbar(1,1)+C(3,2,1,3)*F(2,3)*F(2,2)*Fbar(1,1)*Fbar(1,3)+C(3,1,2,3)*F(2,3)*F(2,1)*Fbar(1,2)*Fbar(1,3)+C(1,3,1,3)*F(2,1)*F(2,3)*Fbar(1,1)*Fbar(1,3)+C(3,2,2,1)*F(2,3)*F(2,2)*Fbar(1,2)*Fbar(1,1)+C(2,2,2,3)*F(2,2)^2*Fbar(1,2)*Fbar(1,3)+C(3,1,1,1)*F(2,3)*F(2,1)*Fbar(1,1)^2+C(3,1,3,3)*F(2,3)*F(2,1)*Fbar(1,3)^2+C(3,3,2,3)*F(2,3)^2*Fbar(1,2)*Fbar(1,3)+C(2,1,2,2)*F(2,2)*F(2,1)*Fbar(1,2)^2+C(1,1,3,2)*F(2,1)^2*Fbar(1,3)*Fbar(1,2)+C(2,1,1,1)*F(2,2)*F(2,1)*Fbar(1,1)^2+C(1,2,3,3)*F(2,1)*F(2,2)*Fbar(1,3)^2+C(1,1,1,3)*F(2,1)^2*Fbar(1,1)*Fbar(1,3)+C(2,3,2,1)*F(2,2)*F(2,3)*Fbar(1,2)*Fbar(1,1)+C(1,3,2,3)*F(2,1)*F(2,3)*Fbar(1,2)*Fbar(1,3)+C(3,3,2,1)*F(2,3)^2*Fbar(1,2)*Fbar(1,1)+C(2,2,3,2)*F(2,2)^2*Fbar(1,3)*Fbar(1,2)+C(2,2,3,1)*F(2,2)^2*Fbar(1,3)*Fbar(1,1)+C(1,2,2,2)*F(2,1)*F(2,2)*Fbar(1,2)^2+C(3,2,1,1)*F(2,3)*F(2,2)*Fbar(1,1)^2+C(2,2,2,1)*F(2,2)^2*Fbar(1,2)*Fbar(1,1)+C(3,2,2,3)*F(2,3)*F(2,2)*Fbar(1,2)*Fbar(1,3)+C(1,2,2,3)*F(2,1)*F(2,2)*Fbar(1,2)*Fbar(1,3)+C(2,3,3,1)*F(2,2)*F(2,3)*Fbar(1,3)*Fbar(1,1)+C(2,1,2,3)*F(2,2)*F(2,1)*Fbar(1,2)*Fbar(1,3)+C(1,2,3,1)*F(2,1)*F(2,2)*Fbar(1,3)*Fbar(1,1)+C(3,1,1,2)*F(2,3)*F(2,1)*Fbar(1,1)*Fbar(1,2)+C(2,1,1,3)*F(2,2)*F(2,1)*Fbar(1,1)*Fbar(1,3)+C(1,2,1,3)*F(2,1)*F(2,2)*Fbar(1,1)*Fbar(1,3)+C(2,1,1,2)*F(2,2)*F(2,1)*Fbar(1,1)*Fbar(1,2)+C(3,2,2,2)*F(2,3)*F(2,2)*Fbar(1,2)^2+C(2,2,1,3)*F(2,2)^2*Fbar(1,1)*Fbar(1,3)+C(2,2,1,2)*F(2,2)^2*Fbar(1,1)*Fbar(1,2)+C(1,1,2,3)*F(2,1)^2*Fbar(1,2)*Fbar(1,3)+C(1,1,3,1)*F(2,1)^2*Fbar(1,3)*Fbar(1,1)+C(1,1,1,2)*F(2,1)^2*Fbar(1,1)*Fbar(1,2)+C(1,3,1,1)*F(2,1)*F(2,3)*Fbar(1,1)^2+C(3,3,3,2)*F(2,3)^2*Fbar(1,3)*Fbar(1,2)+C(3,2,3,3)*F(2,3)*F(2,2)*Fbar(1,3)^2+C(2,3,1,3)*F(2,2)*F(2,3)*Fbar(1,1)*Fbar(1,3)+C(3,2,3,2)*F(2,3)*F(2,2)*Fbar(1,3)*Fbar(1,2)+C(2,1,2,1)*F(2,2)*F(2,1)*Fbar(1,2)*Fbar(1,1)+C(1,3,3,2)*F(2,1)*F(2,3)*Fbar(1,3)*Fbar(1,2)+C(2,3,3,2)*F(2,2)*F(2,3)*Fbar(1,3)*Fbar(1,2)+C(3,2,3,1)*F(2,3)*F(2,2)*Fbar(1,3)*Fbar(1,1)+C(3,1,2,1)*F(2,3)*F(2,1)*Fbar(1,2)*Fbar(1,1)+C(2,3,2,3)*F(2,2)*F(2,3)*Fbar(1,2)*Fbar(1,3)+C(3,3,1,2)*F(2,3)^2*Fbar(1,1)*Fbar(1,2)+C(1,3,3,3)*F(2,1)*F(2,3)*Fbar(1,3)^2+C(2,1,3,3)*F(2,2)*F(2,1)*Fbar(1,3)^2+C(1,1,2,1)*F(2,1)^2*Fbar(1,2)*Fbar(1,1)+C(3,1,3,2)*F(2,3)*F(2,1)*Fbar(1,3)*Fbar(1,2)+C(1,3,2,1)*F(2,1)*F(2,3)*Fbar(1,2)*Fbar(1,1)+C(3,2,1,2)*F(2,3)*F(2,2)*Fbar(1,1)*Fbar(1,2)+C(3,1,1,3)*F(2,3)*F(2,1)*Fbar(1,1)*Fbar(1,3)+C(3,1,2,2)*F(2,3)*F(2,1)*Fbar(1,2)^2+C(3,3,1,3)*F(2,3)^2*Fbar(1,1)*Fbar(1,3)+C(3,3,3,1)*F(2,3)^2*Fbar(1,3)*Fbar(1,1)+C(1,2,1,1)*F(2,1)*F(2,2)*Fbar(1,1)^2+C(1,1,2,2)*F(2,1)^2*Fbar(1,2)^2+C(1,1,3,3)*F(2,1)^2*Fbar(1,3)^2+C(1,2,2,1)*F(2,1)*F(2,2)*Fbar(1,2)*Fbar(1,1)+C(1,2,3,2)*F(2,1)*F(2,2)*Fbar(1,3)*Fbar(1,2)+C(3,3,2,2)*F(2,3)^2*Fbar(1,2)^2+C(2,2,1,1)*F(2,2)^2*Fbar(1,1)^2+C(3,1,3,1)*F(2,3)*F(2,1)*Fbar(1,3)*Fbar(1,1)+C(2,2,3,3)*F(2,2)^2*Fbar(1,3)^2+C(3,3,3,3)*F(2,3)^2*Fbar(1,3)^2+C(2,2,2,2)*F(2,2)^2*Fbar(1,2)^2+C(3,3,1,1)*F(2,3)^2*Fbar(1,1)^2+C(2,3,1,2)*F(2,2)*F(2,3)*Fbar(1,1)*Fbar(1,2)+C(1,1,1,1)*F(2,1)^2*Fbar(1,1)^2;
D(2,2)=C(2,2,1,1)*F(2,2)^2*Fbar(2,1)^2+C(2,2,2,2)*F(2,2)^2*Fbar(2,2)^2+C(3,3,3,3)*F(2,3)^2*Fbar(2,3)^2+C(1,1,3,3)*F(2,1)^2*Fbar(2,3)^2+C(1,2,2,2)*F(2,1)*F(2,2)*Fbar(2,2)^2+C(2,2,2,3)*F(2,2)^2*Fbar(2,2)*Fbar(2,3)+C(3,1,3,3)*F(2,3)*F(2,1)*Fbar(2,3)^2+C(2,1,3,3)*F(2,2)*F(2,1)*Fbar(2,3)^2+C(1,1,2,3)*F(2,1)^2*Fbar(2,2)*Fbar(2,3)+C(1,1,3,1)*F(2,1)^2*Fbar(2,3)*Fbar(2,1)+C(1,3,3,3)*F(2,1)*F(2,3)*Fbar(2,3)^2+C(2,2,1,2)*F(2,2)^2*Fbar(2,1)*Fbar(2,2)+C(1,2,3,3)*F(2,1)*F(2,2)*Fbar(2,3)^2+C(3,2,2,2)*F(2,3)*F(2,2)*Fbar(2,2)^2+C(2,1,1,1)*F(2,2)*F(2,1)*Fbar(2,1)^2+C(2,2,3,1)*F(2,2)^2*Fbar(2,3)*Fbar(2,1)+C(3,3,2,2)*F(2,3)^2*Fbar(2,2)^2+C(1,1,1,1)*F(2,1)^2*Fbar(2,1)^2+C(3,3,3,1)*F(2,3)^2*Fbar(2,3)*Fbar(2,1)+C(2,2,3,2)*F(2,2)^2*Fbar(2,3)*Fbar(2,2)+C(1,1,1,2)*F(2,1)^2*Fbar(2,1)*Fbar(2,2)+C(1,1,1,3)*F(2,1)^2*Fbar(2,1)*Fbar(2,3)+C(1,1,2,1)*F(2,1)^2*Fbar(2,2)*Fbar(2,1)+C(1,1,3,2)*F(2,1)^2*Fbar(2,3)*Fbar(2,2)+C(2,2,2,1)*F(2,2)^2*Fbar(2,2)*Fbar(2,1)+C(2,3,1,1)*F(2,2)*F(2,3)*Fbar(2,1)^2+C(3,1,1,1)*F(2,3)*F(2,1)*Fbar(2,1)^2+C(3,3,1,3)*F(2,3)^2*Fbar(2,1)*Fbar(2,3)+C(2,3,3,3)*F(2,2)*F(2,3)*Fbar(2,3)^2+C(3,1,2,2)*F(2,3)*F(2,1)*Fbar(2,2)^2+C(2,1,2,2)*F(2,2)*F(2,1)*Fbar(2,2)^2+C(3,2,3,3)*F(2,3)*F(2,2)*Fbar(2,3)^2+C(3,3,2,1)*F(2,3)^2*Fbar(2,2)*Fbar(2,1)+C(1,2,1,1)*F(2,1)*F(2,2)*Fbar(2,1)^2+C(2,2,1,3)*F(2,2)^2*Fbar(2,1)*Fbar(2,3)+C(2,3,2,2)*F(2,2)*F(2,3)*Fbar(2,2)^2+C(3,3,3,2)*F(2,3)^2*Fbar(2,3)*Fbar(2,2)+C(1,3,1,1)*F(2,1)*F(2,3)*Fbar(2,1)^2+C(3,3,2,3)*F(2,3)^2*Fbar(2,2)*Fbar(2,3)+C(3,3,1,2)*F(2,3)^2*Fbar(2,1)*Fbar(2,2)+C(3,3,1,1)*F(2,3)^2*Fbar(2,1)^2+C(2,2,3,3)*F(2,2)^2*Fbar(2,3)^2+C(3,1,3,1)*F(2,3)*F(2,1)*Fbar(2,3)*Fbar(2,1)+C(3,1,1,2)*F(2,3)*F(2,1)*Fbar(2,1)*Fbar(2,2)+C(3,2,1,2)*F(2,3)*F(2,2)*Fbar(2,1)*Fbar(2,2)+C(3,2,3,1)*F(2,3)*F(2,2)*Fbar(2,3)*Fbar(2,1)+C(2,3,3,2)*F(2,2)*F(2,3)*Fbar(2,3)*Fbar(2,2)+C(2,3,1,2)*F(2,2)*F(2,3)*Fbar(2,1)*Fbar(2,2)+C(3,1,3,2)*F(2,3)*F(2,1)*Fbar(2,3)*Fbar(2,2)+C(1,3,2,1)*F(2,1)*F(2,3)*Fbar(2,2)*Fbar(2,1)+C(2,3,3,1)*F(2,2)*F(2,3)*Fbar(2,3)*Fbar(2,1)+C(2,3,1,3)*F(2,2)*F(2,3)*Fbar(2,1)*Fbar(2,3)+C(3,2,3,2)*F(2,3)*F(2,2)*Fbar(2,3)*Fbar(2,2)+C(2,3,2,1)*F(2,2)*F(2,3)*Fbar(2,2)*Fbar(2,1)+C(3,2,2,3)*F(2,3)*F(2,2)*Fbar(2,2)*Fbar(2,3)+C(3,1,2,3)*F(2,3)*F(2,1)*Fbar(2,2)*Fbar(2,3)+C(2,1,1,3)*F(2,2)*F(2,1)*Fbar(2,1)*Fbar(2,3)+C(2,1,3,1)*F(2,2)*F(2,1)*Fbar(2,3)*Fbar(2,1)+C(1,3,3,2)*F(2,1)*F(2,3)*Fbar(2,3)*Fbar(2,2)+C(1,2,2,1)*F(2,1)*F(2,2)*Fbar(2,2)*Fbar(2,1)+C(1,2,3,2)*F(2,1)*F(2,2)*Fbar(2,3)*Fbar(2,2)+C(1,2,1,3)*F(2,1)*F(2,2)*Fbar(2,1)*Fbar(2,3)+C(3,2,2,1)*F(2,3)*F(2,2)*Fbar(2,2)*Fbar(2,1)+C(1,2,3,1)*F(2,1)*F(2,2)*Fbar(2,3)*Fbar(2,1)+C(1,2,2,3)*F(2,1)*F(2,2)*Fbar(2,2)*Fbar(2,3)+C(1,1,2,2)*F(2,1)^2*Fbar(2,2)^2+C(1,3,3,1)*F(2,1)*F(2,3)*Fbar(2,3)*Fbar(2,1)+C(2,1,3,2)*F(2,2)*F(2,1)*Fbar(2,3)*Fbar(2,2)+C(1,3,2,3)*F(2,1)*F(2,3)*Fbar(2,2)*Fbar(2,3)+C(2,1,1,2)*F(2,2)*F(2,1)*Fbar(2,1)*Fbar(2,2)+C(3,2,1,3)*F(2,3)*F(2,2)*Fbar(2,1)*Fbar(2,3)+C(1,3,1,3)*F(2,1)*F(2,3)*Fbar(2,1)*Fbar(2,3)+C(2,1,2,1)*F(2,2)*F(2,1)*Fbar(2,2)*Fbar(2,1)+C(1,3,1,2)*F(2,1)*F(2,3)*Fbar(2,1)*Fbar(2,2)+C(3,2,1,1)*F(2,3)*F(2,2)*Fbar(2,1)^2+C(1,3,2,2)*F(2,1)*F(2,3)*Fbar(2,2)^2+C(3,1,2,1)*F(2,3)*F(2,1)*Fbar(2,2)*Fbar(2,1)+C(1,2,1,2)*F(2,1)*F(2,2)*Fbar(2,1)*Fbar(2,2)+C(3,1,1,3)*F(2,3)*F(2,1)*Fbar(2,1)*Fbar(2,3)+C(2,3,2,3)*F(2,2)*F(2,3)*Fbar(2,2)*Fbar(2,3)+C(2,1,2,3)*F(2,2)*F(2,1)*Fbar(2,2)*Fbar(2,3);
D(2,3)=C(2,3,3,1)*F(2,2)*F(2,3)*Fbar(3,3)*Fbar(3,1)+C(1,1,1,1)*F(2,1)^2*Fbar(3,1)^2+C(2,2,2,2)*F(2,2)^2*Fbar(3,2)^2+C(2,3,1,2)*F(2,2)*F(2,3)*Fbar(3,1)*Fbar(3,2)+C(3,3,1,1)*F(2,3)^2*Fbar(3,1)^2+C(1,2,3,2)*F(2,1)*F(2,2)*Fbar(3,3)*Fbar(3,2)+C(3,3,2,2)*F(2,3)^2*Fbar(3,2)^2+C(3,2,2,3)*F(2,3)*F(2,2)*Fbar(3,2)*Fbar(3,3)+C(2,1,3,2)*F(2,2)*F(2,1)*Fbar(3,3)*Fbar(3,2)+C(2,3,1,1)*F(2,2)*F(2,3)*Fbar(3,1)^2+C(2,2,3,2)*F(2,2)^2*Fbar(3,3)*Fbar(3,2)+C(3,3,3,1)*F(2,3)^2*Fbar(3,3)*Fbar(3,1)+C(1,1,3,3)*F(2,1)^2*Fbar(3,3)^2+C(2,3,3,2)*F(2,2)*F(2,3)*Fbar(3,3)*Fbar(3,2)+C(3,1,3,1)*F(2,3)*F(2,1)*Fbar(3,3)*Fbar(3,1)+C(2,3,2,1)*F(2,2)*F(2,3)*Fbar(3,2)*Fbar(3,1)+C(1,2,1,2)*F(2,1)*F(2,2)*Fbar(3,1)*Fbar(3,2)+C(2,1,1,2)*F(2,2)*F(2,1)*Fbar(3,1)*Fbar(3,2)+C(1,1,2,2)*F(2,1)^2*Fbar(3,2)^2+C(2,3,1,3)*F(2,2)*F(2,3)*Fbar(3,1)*Fbar(3,3)+C(3,1,2,1)*F(2,3)*F(2,1)*Fbar(3,2)*Fbar(3,1)+C(3,2,1,2)*F(2,3)*F(2,2)*Fbar(3,1)*Fbar(3,2)+C(1,3,1,3)*F(2,1)*F(2,3)*Fbar(3,1)*Fbar(3,3)+C(1,3,1,2)*F(2,1)*F(2,3)*Fbar(3,1)*Fbar(3,2)+C(2,3,2,3)*F(2,2)*F(2,3)*Fbar(3,2)*Fbar(3,3)+C(3,2,2,1)*F(2,3)*F(2,2)*Fbar(3,2)*Fbar(3,1)+C(3,2,2,2)*F(2,3)*F(2,2)*Fbar(3,2)^2+C(1,2,1,1)*F(2,1)*F(2,2)*Fbar(3,1)^2+C(1,1,3,2)*F(2,1)^2*Fbar(3,3)*Fbar(3,2)+C(1,1,1,2)*F(2,1)^2*Fbar(3,1)*Fbar(3,2)+C(3,1,2,2)*F(2,3)*F(2,1)*Fbar(3,2)^2+C(2,2,1,2)*F(2,2)^2*Fbar(3,1)*Fbar(3,2)+C(1,1,1,3)*F(2,1)^2*Fbar(3,1)*Fbar(3,3)+C(1,1,2,1)*F(2,1)^2*Fbar(3,2)*Fbar(3,1)+C(2,2,1,3)*F(2,2)^2*Fbar(3,1)*Fbar(3,3)+C(2,2,2,1)*F(2,2)^2*Fbar(3,2)*Fbar(3,1)+C(3,3,1,3)*F(2,3)^2*Fbar(3,1)*Fbar(3,3)+C(3,1,3,3)*F(2,3)*F(2,1)*Fbar(3,3)^2+C(1,2,2,2)*F(2,1)*F(2,2)*Fbar(3,2)^2+C(1,2,3,3)*F(2,1)*F(2,2)*Fbar(3,3)^2+C(2,2,2,3)*F(2,2)^2*Fbar(3,2)*Fbar(3,3)+C(3,3,1,2)*F(2,3)^2*Fbar(3,1)*Fbar(3,2)+C(3,3,3,2)*F(2,3)^2*Fbar(3,3)*Fbar(3,2)+C(2,3,3,3)*F(2,2)*F(2,3)*Fbar(3,3)^2+C(1,1,2,3)*F(2,1)^2*Fbar(3,2)*Fbar(3,3)+C(1,3,3,3)*F(2,1)*F(2,3)*Fbar(3,3)^2+C(1,3,1,1)*F(2,1)*F(2,3)*Fbar(3,1)^2+C(1,1,3,1)*F(2,1)^2*Fbar(3,3)*Fbar(3,1)+C(2,2,3,1)*F(2,2)^2*Fbar(3,3)*Fbar(3,1)+C(2,1,2,2)*F(2,2)*F(2,1)*Fbar(3,2)^2+C(3,1,1,1)*F(2,3)*F(2,1)*Fbar(3,1)^2+C(3,2,3,1)*F(2,3)*F(2,2)*Fbar(3,3)*Fbar(3,1)+C(3,1,3,2)*F(2,3)*F(2,1)*Fbar(3,3)*Fbar(3,2)+C(2,1,2,1)*F(2,2)*F(2,1)*Fbar(3,2)*Fbar(3,1)+C(1,3,3,2)*F(2,1)*F(2,3)*Fbar(3,3)*Fbar(3,2)+C(2,1,2,3)*F(2,2)*F(2,1)*Fbar(3,2)*Fbar(3,3)+C(1,2,2,3)*F(2,1)*F(2,2)*Fbar(3,2)*Fbar(3,3)+C(2,1,3,1)*F(2,2)*F(2,1)*Fbar(3,3)*Fbar(3,1)+C(2,1,1,3)*F(2,2)*F(2,1)*Fbar(3,1)*Fbar(3,3)+C(3,1,2,3)*F(2,3)*F(2,1)*Fbar(3,2)*Fbar(3,3)+C(1,3,2,3)*F(2,1)*F(2,3)*Fbar(3,2)*Fbar(3,3)+C(1,2,1,3)*F(2,1)*F(2,2)*Fbar(3,1)*Fbar(3,3)+C(1,2,2,1)*F(2,1)*F(2,2)*Fbar(3,2)*Fbar(3,1)+C(3,2,1,3)*F(2,3)*F(2,2)*Fbar(3,1)*Fbar(3,3)+C(1,3,3,1)*F(2,1)*F(2,3)*Fbar(3,3)*Fbar(3,1)+C(3,1,1,3)*F(2,3)*F(2,1)*Fbar(3,1)*Fbar(3,3)+C(1,2,3,1)*F(2,1)*F(2,2)*Fbar(3,3)*Fbar(3,1)+C(3,2,3,2)*F(2,3)*F(2,2)*Fbar(3,3)*Fbar(3,2)+C(1,3,2,1)*F(2,1)*F(2,3)*Fbar(3,2)*Fbar(3,1)+C(3,1,1,2)*F(2,3)*F(2,1)*Fbar(3,1)*Fbar(3,2)+C(3,2,3,3)*F(2,3)*F(2,2)*Fbar(3,3)^2+C(2,1,3,3)*F(2,2)*F(2,1)*Fbar(3,3)^2+C(3,3,2,3)*F(2,3)^2*Fbar(3,2)*Fbar(3,3)+C(3,2,1,1)*F(2,3)*F(2,2)*Fbar(3,1)^2+C(2,1,1,1)*F(2,2)*F(2,1)*Fbar(3,1)^2+C(2,3,2,2)*F(2,2)*F(2,3)*Fbar(3,2)^2+C(3,3,2,1)*F(2,3)^2*Fbar(3,2)*Fbar(3,1)+C(1,3,2,2)*F(2,1)*F(2,3)*Fbar(3,2)^2+C(2,2,3,3)*F(2,2)^2*Fbar(3,3)^2+C(2,2,1,1)*F(2,2)^2*Fbar(3,1)^2+C(3,3,3,3)*F(2,3)^2*Fbar(3,3)^2;
D(2,4)=C(1,1,3,3)*F(2,1)^2*Fbar(1,3)*Fbar(2,3)+C(3,3,1,2)*F(2,3)^2*Fbar(2,1)*Fbar(1,2)+C(2,3,3,3)*F(2,2)*F(2,3)*Fbar(1,3)*Fbar(2,3)+C(2,3,2,1)*F(2,2)*F(2,3)*Fbar(2,2)*Fbar(1,1)+C(3,1,1,3)*F(2,3)*F(2,1)*Fbar(2,1)*Fbar(1,3)+C(1,2,2,2)*F(2,1)*F(2,2)*Fbar(1,2)*Fbar(2,2)+C(3,1,3,2)*F(2,3)*F(2,1)*Fbar(2,3)*Fbar(1,2)+C(3,1,2,2)*F(2,3)*F(2,1)*Fbar(1,2)*Fbar(2,2)+C(3,1,1,1)*F(2,3)*F(2,1)*Fbar(1,1)*Fbar(2,1)+C(1,3,2,2)*F(2,1)*F(2,3)*Fbar(1,2)*Fbar(2,2)+C(1,2,3,3)*F(2,1)*F(2,2)*Fbar(1,3)*Fbar(2,3)+C(2,3,2,2)*F(2,2)*F(2,3)*Fbar(1,2)*Fbar(2,2)+C(2,1,1,1)*F(2,2)*F(2,1)*Fbar(1,1)*Fbar(2,1)+C(2,3,3,2)*F(2,2)*F(2,3)*Fbar(2,3)*Fbar(1,2)+C(1,2,3,1)*F(2,1)*F(2,2)*Fbar(2,3)*Fbar(1,1)+C(1,3,3,1)*F(2,1)*F(2,3)*Fbar(2,3)*Fbar(1,1)+C(2,1,1,3)*F(2,2)*F(2,1)*Fbar(2,1)*Fbar(1,3)+C(3,2,2,1)*F(2,3)*F(2,2)*Fbar(2,2)*Fbar(1,1)+C(3,2,3,2)*F(2,3)*F(2,2)*Fbar(2,3)*Fbar(1,2)+C(2,1,3,3)*F(2,2)*F(2,1)*Fbar(1,3)*Fbar(2,3)+C(1,2,1,2)*F(2,1)*F(2,2)*Fbar(2,1)*Fbar(1,2)+C(3,2,1,1)*F(2,3)*F(2,2)*Fbar(1,1)*Fbar(2,1)+C(1,3,1,1)*F(2,1)*F(2,3)*Fbar(1,1)*Fbar(2,1)+C(2,3,3,1)*F(2,2)*F(2,3)*Fbar(2,3)*Fbar(1,1)+C(2,3,1,3)*F(2,2)*F(2,3)*Fbar(2,1)*Fbar(1,3)+C(2,1,3,1)*F(2,2)*F(2,1)*Fbar(2,3)*Fbar(1,1)+C(3,1,1,2)*F(2,3)*F(2,1)*Fbar(2,1)*Fbar(1,2)+C(3,2,3,1)*F(2,3)*F(2,2)*Fbar(2,3)*Fbar(1,1)+C(1,3,1,3)*F(2,1)*F(2,3)*Fbar(2,1)*Fbar(1,3)+C(2,1,2,1)*F(2,2)*F(2,1)*Fbar(2,2)*Fbar(1,1)+C(3,2,2,2)*F(2,3)*F(2,2)*Fbar(1,2)*Fbar(2,2)+C(2,1,1,2)*F(2,2)*F(2,1)*Fbar(2,1)*Fbar(1,2)+C(2,3,2,3)*F(2,2)*F(2,3)*Fbar(2,2)*Fbar(1,3)+C(1,1,2,1)*F(2,1)^2*Fbar(2,2)*Fbar(1,1)+C(1,1,1,3)*F(2,1)^2*Fbar(2,1)*Fbar(1,3)+C(1,3,3,3)*F(2,1)*F(2,3)*Fbar(1,3)*Fbar(2,3)+C(1,2,3,2)*F(2,1)*F(2,2)*Fbar(2,3)*Fbar(1,2)+C(3,2,2,3)*F(2,3)*F(2,2)*Fbar(2,2)*Fbar(1,3)+C(2,3,1,2)*F(2,2)*F(2,3)*Fbar(2,1)*Fbar(1,2)+C(3,1,3,1)*F(2,3)*F(2,1)*Fbar(2,3)*Fbar(1,1)+C(1,3,3,2)*F(2,1)*F(2,3)*Fbar(2,3)*Fbar(1,2)+C(1,3,2,3)*F(2,1)*F(2,3)*Fbar(2,2)*Fbar(1,3)+C(2,1,2,2)*F(2,2)*F(2,1)*Fbar(1,2)*Fbar(2,2)+C(2,1,2,3)*F(2,2)*F(2,1)*Fbar(2,2)*Fbar(1,3)+C(3,2,3,3)*F(2,3)*F(2,2)*Fbar(1,3)*Fbar(2,3)+C(3,1,2,3)*F(2,3)*F(2,1)*Fbar(2,2)*Fbar(1,3)+C(1,2,1,1)*F(2,1)*F(2,2)*Fbar(1,1)*Fbar(2,1)+C(3,1,3,3)*F(2,3)*F(2,1)*Fbar(1,3)*Fbar(2,3)+C(3,1,2,1)*F(2,3)*F(2,1)*Fbar(2,2)*Fbar(1,1)+C(2,3,1,1)*F(2,2)*F(2,3)*Fbar(1,1)*Fbar(2,1)+C(3,2,1,2)*F(2,3)*F(2,2)*Fbar(2,1)*Fbar(1,2)+C(2,1,3,2)*F(2,2)*F(2,1)*Fbar(2,3)*Fbar(1,2)+C(3,2,1,3)*F(2,3)*F(2,2)*Fbar(2,1)*Fbar(1,3)+C(1,2,2,3)*F(2,1)*F(2,2)*Fbar(2,2)*Fbar(1,3)+C(1,3,1,2)*F(2,1)*F(2,3)*Fbar(2,1)*Fbar(1,2)+C(1,2,1,3)*F(2,1)*F(2,2)*Fbar(2,1)*Fbar(1,3)+C(1,2,2,1)*F(2,1)*F(2,2)*Fbar(2,2)*Fbar(1,1)+C(1,3,2,1)*F(2,1)*F(2,3)*Fbar(2,2)*Fbar(1,1)+C(2,2,1,3)*F(2,2)^2*Fbar(2,1)*Fbar(1,3)+C(1,1,3,2)*F(2,1)^2*Fbar(2,3)*Fbar(1,2)+C(1,1,1,2)*F(2,1)^2*Fbar(2,1)*Fbar(1,2)+C(2,2,2,3)*F(2,2)^2*Fbar(2,2)*Fbar(1,3)+C(3,3,2,3)*F(2,3)^2*Fbar(2,2)*Fbar(1,3)+C(1,1,2,3)*F(2,1)^2*Fbar(2,2)*Fbar(1,3)+C(3,3,2,2)*F(2,3)^2*Fbar(1,2)*Fbar(2,2)+C(2,2,1,2)*F(2,2)^2*Fbar(2,1)*Fbar(1,2)+C(1,1,2,2)*F(2,1)^2*Fbar(1,2)*Fbar(2,2)+C(3,3,3,3)*F(2,3)^2*Fbar(1,3)*Fbar(2,3)+C(2,2,3,2)*F(2,2)^2*Fbar(2,3)*Fbar(1,2)+C(1,1,3,1)*F(2,1)^2*Fbar(2,3)*Fbar(1,1)+C(2,2,1,1)*F(2,2)^2*Fbar(1,1)*Fbar(2,1)+C(3,3,1,1)*F(2,3)^2*Fbar(1,1)*Fbar(2,1)+C(3,3,2,1)*F(2,3)^2*Fbar(2,2)*Fbar(1,1)+C(3,3,3,2)*F(2,3)^2*Fbar(2,3)*Fbar(1,2)+C(2,2,3,1)*F(2,2)^2*Fbar(2,3)*Fbar(1,1)+C(2,2,3,3)*F(2,2)^2*Fbar(1,3)*Fbar(2,3)+C(3,3,1,3)*F(2,3)^2*Fbar(2,1)*Fbar(1,3)+C(1,1,1,1)*F(2,1)^2*Fbar(1,1)*Fbar(2,1)+C(3,3,3,1)*F(2,3)^2*Fbar(2,3)*Fbar(1,1)+C(2,2,2,2)*F(2,2)^2*Fbar(1,2)*Fbar(2,2)+C(2,2,2,1)*F(2,2)^2*Fbar(2,2)*Fbar(1,1);
D(2,5)=C(1,2,2,1)*F(2,1)*F(2,2)*Fbar(3,2)*Fbar(1,1)+C(1,3,2,3)*F(2,1)*F(2,3)*Fbar(3,2)*Fbar(1,3)+C(3,1,3,1)*F(2,3)*F(2,1)*Fbar(3,3)*Fbar(1,1)+C(1,2,3,2)*F(2,1)*F(2,2)*Fbar(3,3)*Fbar(1,2)+C(2,1,3,3)*F(2,2)*F(2,1)*Fbar(3,3)*Fbar(1,3)+C(2,1,1,2)*F(2,2)*F(2,1)*Fbar(3,1)*Fbar(1,2)+C(2,1,2,1)*F(2,2)*F(2,1)*Fbar(3,2)*Fbar(1,1)+C(2,1,2,2)*F(2,2)*F(2,1)*Fbar(3,2)*Fbar(1,2)+C(2,3,3,2)*F(2,2)*F(2,3)*Fbar(3,3)*Fbar(1,2)+C(1,3,1,3)*F(2,1)*F(2,3)*Fbar(3,1)*Fbar(1,3)+C(1,2,2,2)*F(2,1)*F(2,2)*Fbar(3,2)*Fbar(1,2)+C(3,2,1,1)*F(2,3)*F(2,2)*Fbar(3,1)*Fbar(1,1)+C(3,1,1,2)*F(2,3)*F(2,1)*Fbar(3,1)*Fbar(1,2)+C(3,2,1,2)*F(2,3)*F(2,2)*Fbar(3,1)*Fbar(1,2)+C(1,3,3,1)*F(2,1)*F(2,3)*Fbar(3,3)*Fbar(1,1)+C(1,3,3,2)*F(2,1)*F(2,3)*Fbar(3,3)*Fbar(1,2)+C(1,2,1,3)*F(2,1)*F(2,2)*Fbar(3,1)*Fbar(1,3)+C(2,1,1,1)*F(2,2)*F(2,1)*Fbar(3,1)*Fbar(1,1)+C(3,2,2,2)*F(2,3)*F(2,2)*Fbar(3,2)*Fbar(1,2)+C(2,2,2,2)*F(2,2)^2*Fbar(3,2)*Fbar(1,2)+C(1,1,2,3)*F(2,1)^2*Fbar(3,2)*Fbar(1,3)+C(3,3,1,1)*F(2,3)^2*Fbar(3,1)*Fbar(1,1)+C(3,3,1,2)*F(2,3)^2*Fbar(3,1)*Fbar(1,2)+C(2,2,2,1)*F(2,2)^2*Fbar(3,2)*Fbar(1,1)+C(2,2,1,2)*F(2,2)^2*Fbar(3,1)*Fbar(1,2)+C(1,1,1,3)*F(2,1)^2*Fbar(3,1)*Fbar(1,3)+C(2,2,1,1)*F(2,2)^2*Fbar(3,1)*Fbar(1,1)+C(3,3,3,1)*F(2,3)^2*Fbar(3,3)*Fbar(1,1)+C(1,1,1,2)*F(2,1)^2*Fbar(3,1)*Fbar(1,2)+C(1,1,3,3)*F(2,1)^2*Fbar(3,3)*Fbar(1,3)+C(2,2,1,3)*F(2,2)^2*Fbar(3,1)*Fbar(1,3)+C(2,2,3,3)*F(2,2)^2*Fbar(3,3)*Fbar(1,3)+C(2,1,3,1)*F(2,2)*F(2,1)*Fbar(3,3)*Fbar(1,1)+C(1,2,3,3)*F(2,1)*F(2,2)*Fbar(3,3)*Fbar(1,3)+C(3,2,1,3)*F(2,3)*F(2,2)*Fbar(3,1)*Fbar(1,3)+C(1,2,1,1)*F(2,1)*F(2,2)*Fbar(3,1)*Fbar(1,1)+C(2,1,3,2)*F(2,2)*F(2,1)*Fbar(3,3)*Fbar(1,2)+C(1,3,1,1)*F(2,1)*F(2,3)*Fbar(3,1)*Fbar(1,1)+C(2,3,1,3)*F(2,2)*F(2,3)*Fbar(3,1)*Fbar(1,3)+C(3,2,2,3)*F(2,3)*F(2,2)*Fbar(3,2)*Fbar(1,3)+C(2,3,2,3)*F(2,2)*F(2,3)*Fbar(3,2)*Fbar(1,3)+C(3,2,3,1)*F(2,3)*F(2,2)*Fbar(3,3)*Fbar(1,1)+C(2,3,2,1)*F(2,2)*F(2,3)*Fbar(3,2)*Fbar(1,1)+C(3,1,1,1)*F(2,3)*F(2,1)*Fbar(3,1)*Fbar(1,1)+C(3,1,3,3)*F(2,3)*F(2,1)*Fbar(3,3)*Fbar(1,3)+C(3,2,2,1)*F(2,3)*F(2,2)*Fbar(3,2)*Fbar(1,1)+C(2,1,1,3)*F(2,2)*F(2,1)*Fbar(3,1)*Fbar(1,3)+C(3,1,2,3)*F(2,3)*F(2,1)*Fbar(3,2)*Fbar(1,3)+C(3,2,3,2)*F(2,3)*F(2,2)*Fbar(3,3)*Fbar(1,2)+C(1,3,3,3)*F(2,1)*F(2,3)*Fbar(3,3)*Fbar(1,3)+C(2,3,1,2)*F(2,2)*F(2,3)*Fbar(3,1)*Fbar(1,2)+C(3,1,2,1)*F(2,3)*F(2,1)*Fbar(3,2)*Fbar(1,1)+C(3,2,3,3)*F(2,3)*F(2,2)*Fbar(3,3)*Fbar(1,3)+C(3,1,1,3)*F(2,3)*F(2,1)*Fbar(3,1)*Fbar(1,3)+C(2,3,1,1)*F(2,2)*F(2,3)*Fbar(3,1)*Fbar(1,1)+C(3,1,3,2)*F(2,3)*F(2,1)*Fbar(3,3)*Fbar(1,2)+C(1,3,2,2)*F(2,1)*F(2,3)*Fbar(3,2)*Fbar(1,2)+C(1,2,2,3)*F(2,1)*F(2,2)*Fbar(3,2)*Fbar(1,3)+C(1,1,2,2)*F(2,1)^2*Fbar(3,2)*Fbar(1,2)+C(3,3,3,3)*F(2,3)^2*Fbar(3,3)*Fbar(1,3)+C(3,3,2,1)*F(2,3)^2*Fbar(3,2)*Fbar(1,1)+C(3,3,3,2)*F(2,3)^2*Fbar(3,3)*Fbar(1,2)+C(1,1,3,2)*F(2,1)^2*Fbar(3,3)*Fbar(1,2)+C(2,1,2,3)*F(2,2)*F(2,1)*Fbar(3,2)*Fbar(1,3)+C(1,2,3,1)*F(2,1)*F(2,2)*Fbar(3,3)*Fbar(1,1)+C(2,3,3,1)*F(2,2)*F(2,3)*Fbar(3,3)*Fbar(1,1)+C(1,3,1,2)*F(2,1)*F(2,3)*Fbar(3,1)*Fbar(1,2)+C(2,3,3,3)*F(2,2)*F(2,3)*Fbar(3,3)*Fbar(1,3)+C(3,1,2,2)*F(2,3)*F(2,1)*Fbar(3,2)*Fbar(1,2)+C(1,3,2,1)*F(2,1)*F(2,3)*Fbar(3,2)*Fbar(1,1)+C(2,3,2,2)*F(2,2)*F(2,3)*Fbar(3,2)*Fbar(1,2)+C(3,3,2,2)*F(2,3)^2*Fbar(3,2)*Fbar(1,2)+C(3,3,1,3)*F(2,3)^2*Fbar(3,1)*Fbar(1,3)+C(2,2,3,2)*F(2,2)^2*Fbar(3,3)*Fbar(1,2)+C(3,3,2,3)*F(2,3)^2*Fbar(3,2)*Fbar(1,3)+C(1,1,2,1)*F(2,1)^2*Fbar(3,2)*Fbar(1,1)+C(2,2,2,3)*F(2,2)^2*Fbar(3,2)*Fbar(1,3)+C(2,2,3,1)*F(2,2)^2*Fbar(3,3)*Fbar(1,1)+C(1,1,3,1)*F(2,1)^2*Fbar(3,3)*Fbar(1,1)+C(1,1,1,1)*F(2,1)^2*Fbar(3,1)*Fbar(1,1)+C(1,2,1,2)*F(2,1)*F(2,2)*Fbar(3,1)*Fbar(1,2);
D(2,6)=C(2,1,1,3)*F(2,2)*F(2,1)*Fbar(3,1)*Fbar(2,3)+C(2,1,3,2)*F(2,2)*F(2,1)*Fbar(3,3)*Fbar(2,2)+C(1,2,3,1)*F(2,1)*F(2,2)*Fbar(3,3)*Fbar(2,1)+C(3,2,3,3)*F(2,3)*F(2,2)*Fbar(3,3)*Fbar(2,3)+C(1,2,1,2)*F(2,1)*F(2,2)*Fbar(3,1)*Fbar(2,2)+C(1,2,1,1)*F(2,1)*F(2,2)*Fbar(3,1)*Fbar(2,1)+C(2,1,1,1)*F(2,2)*F(2,1)*Fbar(3,1)*Fbar(2,1)+C(3,1,1,1)*F(2,3)*F(2,1)*Fbar(3,1)*Fbar(2,1)+C(3,2,1,2)*F(2,3)*F(2,2)*Fbar(3,1)*Fbar(2,2)+C(2,1,3,1)*F(2,2)*F(2,1)*Fbar(3,3)*Fbar(2,1)+C(1,3,2,1)*F(2,1)*F(2,3)*Fbar(3,2)*Fbar(2,1)+C(2,3,1,1)*F(2,2)*F(2,3)*Fbar(3,1)*Fbar(2,1)+C(2,2,2,1)*F(2,2)^2*Fbar(3,2)*Fbar(2,1)+C(2,2,1,2)*F(2,2)^2*Fbar(3,1)*Fbar(2,2)+C(2,2,1,3)*F(2,2)^2*Fbar(3,1)*Fbar(2,3)+C(2,2,3,2)*F(2,2)^2*Fbar(3,3)*Fbar(2,2)+C(1,1,3,1)*F(2,1)^2*Fbar(3,3)*Fbar(2,1)+C(1,1,3,3)*F(2,1)^2*Fbar(3,3)*Fbar(2,3)+C(3,3,2,1)*F(2,3)^2*Fbar(3,2)*Fbar(2,1)+C(1,1,2,2)*F(2,1)^2*Fbar(3,2)*Fbar(2,2)+C(1,1,2,3)*F(2,1)^2*Fbar(3,2)*Fbar(2,3)+C(3,3,2,3)*F(2,3)^2*Fbar(3,2)*Fbar(2,3)+C(1,1,1,3)*F(2,1)^2*Fbar(3,1)*Fbar(2,3)+C(1,1,3,2)*F(2,1)^2*Fbar(3,3)*Fbar(2,2)+C(3,3,1,3)*F(2,3)^2*Fbar(3,1)*Fbar(2,3)+C(2,2,3,3)*F(2,2)^2*Fbar(3,3)*Fbar(2,3)+C(2,2,2,3)*F(2,2)^2*Fbar(3,2)*Fbar(2,3)+C(2,2,3,1)*F(2,2)^2*Fbar(3,3)*Fbar(2,1)+C(1,1,1,1)*F(2,1)^2*Fbar(3,1)*Fbar(2,1)+C(2,2,2,2)*F(2,2)^2*Fbar(3,2)*Fbar(2,2)+C(3,3,3,1)*F(2,3)^2*Fbar(3,3)*Fbar(2,1)+C(3,3,1,2)*F(2,3)^2*Fbar(3,1)*Fbar(2,2)+C(3,3,3,2)*F(2,3)^2*Fbar(3,3)*Fbar(2,2)+C(3,3,1,1)*F(2,3)^2*Fbar(3,1)*Fbar(2,1)+C(1,1,1,2)*F(2,1)^2*Fbar(3,1)*Fbar(2,2)+C(3,3,3,3)*F(2,3)^2*Fbar(3,3)*Fbar(2,3)+C(3,3,2,2)*F(2,3)^2*Fbar(3,2)*Fbar(2,2)+C(2,2,1,1)*F(2,2)^2*Fbar(3,1)*Fbar(2,1)+C(1,1,2,1)*F(2,1)^2*Fbar(3,2)*Fbar(2,1)+C(2,1,2,3)*F(2,2)*F(2,1)*Fbar(3,2)*Fbar(2,3)+C(2,1,2,1)*F(2,2)*F(2,1)*Fbar(3,2)*Fbar(2,1)+C(1,2,1,3)*F(2,1)*F(2,2)*Fbar(3,1)*Fbar(2,3)+C(3,1,3,2)*F(2,3)*F(2,1)*Fbar(3,3)*Fbar(2,2)+C(3,2,2,1)*F(2,3)*F(2,2)*Fbar(3,2)*Fbar(2,1)+C(2,3,3,3)*F(2,2)*F(2,3)*Fbar(3,3)*Fbar(2,3)+C(1,2,2,1)*F(2,1)*F(2,2)*Fbar(3,2)*Fbar(2,1)+C(2,1,3,3)*F(2,2)*F(2,1)*Fbar(3,3)*Fbar(2,3)+C(1,3,2,3)*F(2,1)*F(2,3)*Fbar(3,2)*Fbar(2,3)+C(1,2,3,3)*F(2,1)*F(2,2)*Fbar(3,3)*Fbar(2,3)+C(1,3,3,1)*F(2,1)*F(2,3)*Fbar(3,3)*Fbar(2,1)+C(3,1,1,3)*F(2,3)*F(2,1)*Fbar(3,1)*Fbar(2,3)+C(3,2,2,2)*F(2,3)*F(2,2)*Fbar(3,2)*Fbar(2,2)+C(2,3,2,1)*F(2,2)*F(2,3)*Fbar(3,2)*Fbar(2,1)+C(1,3,3,2)*F(2,1)*F(2,3)*Fbar(3,3)*Fbar(2,2)+C(1,3,2,2)*F(2,1)*F(2,3)*Fbar(3,2)*Fbar(2,2)+C(1,3,3,3)*F(2,1)*F(2,3)*Fbar(3,3)*Fbar(2,3)+C(3,1,3,1)*F(2,3)*F(2,1)*Fbar(3,3)*Fbar(2,1)+C(2,1,2,2)*F(2,2)*F(2,1)*Fbar(3,2)*Fbar(2,2)+C(3,1,2,1)*F(2,3)*F(2,1)*Fbar(3,2)*Fbar(2,1)+C(3,2,2,3)*F(2,3)*F(2,2)*Fbar(3,2)*Fbar(2,3)+C(3,1,1,2)*F(2,3)*F(2,1)*Fbar(3,1)*Fbar(2,2)+C(3,2,1,3)*F(2,3)*F(2,2)*Fbar(3,1)*Fbar(2,3)+C(2,3,2,3)*F(2,2)*F(2,3)*Fbar(3,2)*Fbar(2,3)+C(3,1,2,2)*F(2,3)*F(2,1)*Fbar(3,2)*Fbar(2,2)+C(2,3,3,2)*F(2,2)*F(2,3)*Fbar(3,3)*Fbar(2,2)+C(1,2,2,2)*F(2,1)*F(2,2)*Fbar(3,2)*Fbar(2,2)+C(3,2,3,1)*F(2,3)*F(2,2)*Fbar(3,3)*Fbar(2,1)+C(3,1,3,3)*F(2,3)*F(2,1)*Fbar(3,3)*Fbar(2,3)+C(3,1,2,3)*F(2,3)*F(2,1)*Fbar(3,2)*Fbar(2,3)+C(2,1,1,2)*F(2,2)*F(2,1)*Fbar(3,1)*Fbar(2,2)+C(2,3,2,2)*F(2,2)*F(2,3)*Fbar(3,2)*Fbar(2,2)+C(2,3,1,3)*F(2,2)*F(2,3)*Fbar(3,1)*Fbar(2,3)+C(1,3,1,1)*F(2,1)*F(2,3)*Fbar(3,1)*Fbar(2,1)+C(1,3,1,3)*F(2,1)*F(2,3)*Fbar(3,1)*Fbar(2,3)+C(1,2,3,2)*F(2,1)*F(2,2)*Fbar(3,3)*Fbar(2,2)+C(1,3,1,2)*F(2,1)*F(2,3)*Fbar(3,1)*Fbar(2,2)+C(3,2,3,2)*F(2,3)*F(2,2)*Fbar(3,3)*Fbar(2,2)+C(2,3,3,1)*F(2,2)*F(2,3)*Fbar(3,3)*Fbar(2,1)+C(2,3,1,2)*F(2,2)*F(2,3)*Fbar(3,1)*Fbar(2,2)+C(3,2,1,1)*F(2,3)*F(2,2)*Fbar(3,1)*Fbar(2,1)+C(1,2,2,3)*F(2,1)*F(2,2)*Fbar(3,2)*Fbar(2,3);
D(3,1)=C(3,3,3,3)*F(3,3)^2*Fbar(1,3)^2+C(1,2,1,3)*F(3,1)*F(3,2)*Fbar(1,1)*Fbar(1,3)+C(3,3,2,2)*F(3,3)^2*Fbar(1,2)^2+C(3,3,1,1)*F(3,3)^2*Fbar(1,1)^2+C(2,2,2,2)*F(3,2)^2*Fbar(1,2)^2+C(1,1,1,2)*F(3,1)^2*Fbar(1,1)*Fbar(1,2)+C(1,1,1,3)*F(3,1)^2*Fbar(1,1)*Fbar(1,3)+C(1,1,2,1)*F(3,1)^2*Fbar(1,2)*Fbar(1,1)+C(1,3,3,3)*F(3,1)*F(3,3)*Fbar(1,3)^2+C(2,3,3,3)*F(3,2)*F(3,3)*Fbar(1,3)^2+C(3,1,3,3)*F(3,3)*F(3,1)*Fbar(1,3)^2+C(1,1,1,1)*F(3,1)^2*Fbar(1,1)^2+C(1,2,3,3)*F(3,1)*F(3,2)*Fbar(1,3)^2+C(1,1,2,3)*F(3,1)^2*Fbar(1,2)*Fbar(1,3)+C(1,2,2,2)*F(3,1)*F(3,2)*Fbar(1,2)^2+C(2,2,1,2)*F(3,2)^2*Fbar(1,1)*Fbar(1,2)+C(1,2,1,1)*F(3,1)*F(3,2)*Fbar(1,1)^2+C(1,1,3,2)*F(3,1)^2*Fbar(1,3)*Fbar(1,2)+C(1,3,1,1)*F(3,1)*F(3,3)*Fbar(1,1)^2+C(2,3,1,1)*F(3,2)*F(3,3)*Fbar(1,1)^2+C(2,2,1,3)*F(3,2)^2*Fbar(1,1)*Fbar(1,3)+C(1,1,2,2)*F(3,1)^2*Fbar(1,2)^2+C(1,3,2,1)*F(3,1)*F(3,3)*Fbar(1,2)*Fbar(1,1)+C(1,3,2,3)*F(3,1)*F(3,3)*Fbar(1,2)*Fbar(1,3)+C(3,1,3,1)*F(3,3)*F(3,1)*Fbar(1,3)*Fbar(1,1)+C(2,1,1,3)*F(3,2)*F(3,1)*Fbar(1,1)*Fbar(1,3)+C(2,3,2,1)*F(3,2)*F(3,3)*Fbar(1,2)*Fbar(1,1)+C(1,2,1,2)*F(3,1)*F(3,2)*Fbar(1,1)*Fbar(1,2)+C(2,3,3,2)*F(3,2)*F(3,3)*Fbar(1,3)*Fbar(1,2)+C(1,3,1,3)*F(3,1)*F(3,3)*Fbar(1,1)*Fbar(1,3)+C(3,1,3,2)*F(3,3)*F(3,1)*Fbar(1,3)*Fbar(1,2)+C(1,3,3,2)*F(3,1)*F(3,3)*Fbar(1,3)*Fbar(1,2)+C(3,1,2,3)*F(3,3)*F(3,1)*Fbar(1,2)*Fbar(1,3)+C(2,1,2,3)*F(3,2)*F(3,1)*Fbar(1,2)*Fbar(1,3)+C(1,3,3,1)*F(3,1)*F(3,3)*Fbar(1,3)*Fbar(1,1)+C(3,1,1,3)*F(3,3)*F(3,1)*Fbar(1,1)*Fbar(1,3)+C(3,1,2,1)*F(3,3)*F(3,1)*Fbar(1,2)*Fbar(1,1)+C(3,2,3,2)*F(3,3)*F(3,2)*Fbar(1,3)*Fbar(1,2)+C(3,3,1,2)*F(3,3)^2*Fbar(1,1)*Fbar(1,2)+C(2,2,2,3)*F(3,2)^2*Fbar(1,2)*Fbar(1,3)+C(2,2,3,3)*F(3,2)^2*Fbar(1,3)^2+C(2,1,1,1)*F(3,2)*F(3,1)*Fbar(1,1)^2+C(3,1,2,2)*F(3,3)*F(3,1)*Fbar(1,2)^2+C(2,1,3,3)*F(3,2)*F(3,1)*Fbar(1,3)^2+C(1,1,3,1)*F(3,1)^2*Fbar(1,3)*Fbar(1,1)+C(2,1,2,2)*F(3,2)*F(3,1)*Fbar(1,2)^2+C(3,1,1,1)*F(3,3)*F(3,1)*Fbar(1,1)^2+C(3,3,3,2)*F(3,3)^2*Fbar(1,3)*Fbar(1,2)+C(1,3,2,2)*F(3,1)*F(3,3)*Fbar(1,2)^2+C(2,2,3,1)*F(3,2)^2*Fbar(1,3)*Fbar(1,1)+C(3,3,2,3)*F(3,3)^2*Fbar(1,2)*Fbar(1,3)+C(2,1,1,2)*F(3,2)*F(3,1)*Fbar(1,1)*Fbar(1,2)+C(3,2,2,1)*F(3,3)*F(3,2)*Fbar(1,2)*Fbar(1,1)+C(1,2,3,1)*F(3,1)*F(3,2)*Fbar(1,3)*Fbar(1,1)+C(2,3,1,2)*F(3,2)*F(3,3)*Fbar(1,1)*Fbar(1,2)+C(1,2,3,2)*F(3,1)*F(3,2)*Fbar(1,3)*Fbar(1,2)+C(1,2,2,1)*F(3,1)*F(3,2)*Fbar(1,2)*Fbar(1,1)+C(3,2,2,3)*F(3,3)*F(3,2)*Fbar(1,2)*Fbar(1,3)+C(3,2,1,2)*F(3,3)*F(3,2)*Fbar(1,1)*Fbar(1,2)+C(2,1,2,1)*F(3,2)*F(3,1)*Fbar(1,2)*Fbar(1,1)+C(2,3,1,3)*F(3,2)*F(3,3)*Fbar(1,1)*Fbar(1,3)+C(3,3,1,3)*F(3,3)^2*Fbar(1,1)*Fbar(1,3)+C(2,2,2,1)*F(3,2)^2*Fbar(1,2)*Fbar(1,1)+C(2,2,3,2)*F(3,2)^2*Fbar(1,3)*Fbar(1,2)+C(2,3,2,2)*F(3,2)*F(3,3)*Fbar(1,2)^2+C(3,2,2,2)*F(3,3)*F(3,2)*Fbar(1,2)^2+C(3,3,3,1)*F(3,3)^2*Fbar(1,3)*Fbar(1,1)+C(3,2,3,3)*F(3,3)*F(3,2)*Fbar(1,3)^2+C(1,2,2,3)*F(3,1)*F(3,2)*Fbar(1,2)*Fbar(1,3)+C(3,1,1,2)*F(3,3)*F(3,1)*Fbar(1,1)*Fbar(1,2)+C(2,1,3,1)*F(3,2)*F(3,1)*Fbar(1,3)*Fbar(1,1)+C(2,3,3,1)*F(3,2)*F(3,3)*Fbar(1,3)*Fbar(1,1)+C(2,1,3,2)*F(3,2)*F(3,1)*Fbar(1,3)*Fbar(1,2)+C(1,3,1,2)*F(3,1)*F(3,3)*Fbar(1,1)*Fbar(1,2)+C(3,2,3,1)*F(3,3)*F(3,2)*Fbar(1,3)*Fbar(1,1)+C(3,2,1,3)*F(3,3)*F(3,2)*Fbar(1,1)*Fbar(1,3)+C(3,3,2,1)*F(3,3)^2*Fbar(1,2)*Fbar(1,1)+C(3,2,1,1)*F(3,3)*F(3,2)*Fbar(1,1)^2+C(2,2,1,1)*F(3,2)^2*Fbar(1,1)^2+C(1,1,3,3)*F(3,1)^2*Fbar(1,3)^2+C(2,3,2,3)*F(3,2)*F(3,3)*Fbar(1,2)*Fbar(1,3);
D(3,2)=C(3,3,3,3)*F(3,3)^2*Fbar(2,3)^2+C(2,1,2,3)*F(3,2)*F(3,1)*Fbar(2,2)*Fbar(2,3)+C(1,1,3,3)*F(3,1)^2*Fbar(2,3)^2+C(2,3,1,2)*F(3,2)*F(3,3)*Fbar(2,1)*Fbar(2,2)+C(2,3,2,3)*F(3,2)*F(3,3)*Fbar(2,2)*Fbar(2,3)+C(3,2,1,3)*F(3,3)*F(3,2)*Fbar(2,1)*Fbar(2,3)+C(3,2,3,1)*F(3,3)*F(3,2)*Fbar(2,3)*Fbar(2,1)+C(1,2,3,1)*F(3,1)*F(3,2)*Fbar(2,3)*Fbar(2,1)+C(2,1,2,1)*F(3,2)*F(3,1)*Fbar(2,2)*Fbar(2,1)+C(3,1,2,1)*F(3,3)*F(3,1)*Fbar(2,2)*Fbar(2,1)+C(2,3,2,1)*F(3,2)*F(3,3)*Fbar(2,2)*Fbar(2,1)+C(3,2,2,1)*F(3,3)*F(3,2)*Fbar(2,2)*Fbar(2,1)+C(3,1,3,2)*F(3,3)*F(3,1)*Fbar(2,3)*Fbar(2,2)+C(3,1,1,3)*F(3,3)*F(3,1)*Fbar(2,1)*Fbar(2,3)+C(2,3,3,2)*F(3,2)*F(3,3)*Fbar(2,3)*Fbar(2,2)+C(3,2,2,3)*F(3,3)*F(3,2)*Fbar(2,2)*Fbar(2,3)+C(1,2,2,1)*F(3,1)*F(3,2)*Fbar(2,2)*Fbar(2,1)+C(1,3,2,1)*F(3,1)*F(3,3)*Fbar(2,2)*Fbar(2,1)+C(1,3,3,2)*F(3,1)*F(3,3)*Fbar(2,3)*Fbar(2,2)+C(1,3,3,1)*F(3,1)*F(3,3)*Fbar(2,3)*Fbar(2,1)+C(2,1,1,2)*F(3,2)*F(3,1)*Fbar(2,1)*Fbar(2,2)+C(3,1,1,1)*F(3,3)*F(3,1)*Fbar(2,1)^2+C(2,2,2,3)*F(3,2)^2*Fbar(2,2)*Fbar(2,3)+C(1,1,3,2)*F(3,1)^2*Fbar(2,3)*Fbar(2,2)+C(2,2,3,2)*F(3,2)^2*Fbar(2,3)*Fbar(2,2)+C(2,2,1,2)*F(3,2)^2*Fbar(2,1)*Fbar(2,2)+C(3,2,1,1)*F(3,3)*F(3,2)*Fbar(2,1)^2+C(2,1,1,1)*F(3,2)*F(3,1)*Fbar(2,1)^2+C(1,2,1,1)*F(3,1)*F(3,2)*Fbar(2,1)^2+C(2,1,3,2)*F(3,2)*F(3,1)*Fbar(2,3)*Fbar(2,2)+C(2,2,1,3)*F(3,2)^2*Fbar(2,1)*Fbar(2,3)+C(1,3,2,2)*F(3,1)*F(3,3)*Fbar(2,2)^2+C(1,2,3,3)*F(3,1)*F(3,2)*Fbar(2,3)^2+C(1,1,3,1)*F(3,1)^2*Fbar(2,3)*Fbar(2,1)+C(1,2,2,3)*F(3,1)*F(3,2)*Fbar(2,2)*Fbar(2,3)+C(3,3,1,1)*F(3,3)^2*Fbar(2,1)^2+C(1,1,1,1)*F(3,1)^2*Fbar(2,1)^2+C(1,1,2,2)*F(3,1)^2*Fbar(2,2)^2+C(2,2,1,1)*F(3,2)^2*Fbar(2,1)^2+C(1,1,1,2)*F(3,1)^2*Fbar(2,1)*Fbar(2,2)+C(3,3,3,1)*F(3,3)^2*Fbar(2,3)*Fbar(2,1)+C(1,2,2,2)*F(3,1)*F(3,2)*Fbar(2,2)^2+C(1,1,1,3)*F(3,1)^2*Fbar(2,1)*Fbar(2,3)+C(2,1,2,2)*F(3,2)*F(3,1)*Fbar(2,2)^2+C(1,1,2,1)*F(3,1)^2*Fbar(2,2)*Fbar(2,1)+C(3,1,3,3)*F(3,3)*F(3,1)*Fbar(2,3)^2+C(2,3,2,2)*F(3,2)*F(3,3)*Fbar(2,2)^2+C(3,3,2,1)*F(3,3)^2*Fbar(2,2)*Fbar(2,1)+C(3,2,2,2)*F(3,3)*F(3,2)*Fbar(2,2)^2+C(3,3,2,3)*F(3,3)^2*Fbar(2,2)*Fbar(2,3)+C(2,3,3,3)*F(3,2)*F(3,3)*Fbar(2,3)^2+C(2,1,3,3)*F(3,2)*F(3,1)*Fbar(2,3)^2+C(3,3,3,2)*F(3,3)^2*Fbar(2,3)*Fbar(2,2)+C(1,1,2,3)*F(3,1)^2*Fbar(2,2)*Fbar(2,3)+C(3,2,3,3)*F(3,3)*F(3,2)*Fbar(2,3)^2+C(2,3,1,1)*F(3,2)*F(3,3)*Fbar(2,1)^2+C(1,3,1,1)*F(3,1)*F(3,3)*Fbar(2,1)^2+C(3,3,1,3)*F(3,3)^2*Fbar(2,1)*Fbar(2,3)+C(2,2,3,1)*F(3,2)^2*Fbar(2,3)*Fbar(2,1)+C(3,1,2,2)*F(3,3)*F(3,1)*Fbar(2,2)^2+C(1,3,3,3)*F(3,1)*F(3,3)*Fbar(2,3)^2+C(2,2,2,1)*F(3,2)^2*Fbar(2,2)*Fbar(2,1)+C(3,3,1,2)*F(3,3)^2*Fbar(2,1)*Fbar(2,2)+C(3,2,3,2)*F(3,3)*F(3,2)*Fbar(2,3)*Fbar(2,2)+C(1,2,1,3)*F(3,1)*F(3,2)*Fbar(2,1)*Fbar(2,3)+C(1,3,2,3)*F(3,1)*F(3,3)*Fbar(2,2)*Fbar(2,3)+C(1,3,1,3)*F(3,1)*F(3,3)*Fbar(2,1)*Fbar(2,3)+C(2,1,3,1)*F(3,2)*F(3,1)*Fbar(2,3)*Fbar(2,1)+C(3,1,2,3)*F(3,3)*F(3,1)*Fbar(2,2)*Fbar(2,3)+C(1,2,3,2)*F(3,1)*F(3,2)*Fbar(2,3)*Fbar(2,2)+C(1,2,1,2)*F(3,1)*F(3,2)*Fbar(2,1)*Fbar(2,2)+C(2,3,1,3)*F(3,2)*F(3,3)*Fbar(2,1)*Fbar(2,3)+C(1,3,1,2)*F(3,1)*F(3,3)*Fbar(2,1)*Fbar(2,2)+C(3,1,3,1)*F(3,3)*F(3,1)*Fbar(2,3)*Fbar(2,1)+C(2,1,1,3)*F(3,2)*F(3,1)*Fbar(2,1)*Fbar(2,3)+C(3,1,1,2)*F(3,3)*F(3,1)*Fbar(2,1)*Fbar(2,2)+C(3,2,1,2)*F(3,3)*F(3,2)*Fbar(2,1)*Fbar(2,2)+C(2,3,3,1)*F(3,2)*F(3,3)*Fbar(2,3)*Fbar(2,1)+C(2,2,2,2)*F(3,2)^2*Fbar(2,2)^2+C(2,2,3,3)*F(3,2)^2*Fbar(2,3)^2+C(3,3,2,2)*F(3,3)^2*Fbar(2,2)^2;
D(3,3)=C(3,1,2,1)*F(3,3)*F(3,1)*Fbar(3,2)*Fbar(3,1)+C(3,3,1,2)*F(3,3)^2*Fbar(3,1)*Fbar(3,2)+C(3,2,3,2)*F(3,3)*F(3,2)*Fbar(3,3)*Fbar(3,2)+C(3,3,1,3)*F(3,3)^2*Fbar(3,1)*Fbar(3,3)+C(3,3,2,3)*F(3,3)^2*Fbar(3,2)*Fbar(3,3)+C(3,2,1,1)*F(3,3)*F(3,2)*Fbar(3,1)^2+C(3,3,3,2)*F(3,3)^2*Fbar(3,3)*Fbar(3,2)+C(3,2,2,1)*F(3,3)*F(3,2)*Fbar(3,2)*Fbar(3,1)+C(3,2,1,2)*F(3,3)*F(3,2)*Fbar(3,1)*Fbar(3,2)+C(3,2,3,3)*F(3,3)*F(3,2)*Fbar(3,3)^2+C(3,2,2,2)*F(3,3)*F(3,2)*Fbar(3,2)^2+C(1,2,2,1)*F(3,1)*F(3,2)*Fbar(3,2)*Fbar(3,1)+C(1,2,2,2)*F(3,1)*F(3,2)*Fbar(3,2)^2+C(3,3,3,1)*F(3,3)^2*Fbar(3,3)*Fbar(3,1)+C(1,3,3,1)*F(3,1)*F(3,3)*Fbar(3,3)*Fbar(3,1)+C(3,3,2,1)*F(3,3)^2*Fbar(3,2)*Fbar(3,1)+C(2,3,2,1)*F(3,2)*F(3,3)*Fbar(3,2)*Fbar(3,1)+C(1,2,2,3)*F(3,1)*F(3,2)*Fbar(3,2)*Fbar(3,3)+C(3,1,1,2)*F(3,3)*F(3,1)*Fbar(3,1)*Fbar(3,2)+C(1,1,2,1)*F(3,1)^2*Fbar(3,2)*Fbar(3,1)+C(1,2,1,1)*F(3,1)*F(3,2)*Fbar(3,1)^2+C(3,3,3,3)*F(3,3)^2*Fbar(3,3)^2+C(2,2,1,1)*F(3,2)^2*Fbar(3,1)^2+C(1,2,1,3)*F(3,1)*F(3,2)*Fbar(3,1)*Fbar(3,3)+C(2,1,2,3)*F(3,2)*F(3,1)*Fbar(3,2)*Fbar(3,3)+C(3,1,3,2)*F(3,3)*F(3,1)*Fbar(3,3)*Fbar(3,2)+C(2,3,1,3)*F(3,2)*F(3,3)*Fbar(3,1)*Fbar(3,3)+C(2,3,1,2)*F(3,2)*F(3,3)*Fbar(3,1)*Fbar(3,2)+C(2,1,2,2)*F(3,2)*F(3,1)*Fbar(3,2)^2+C(2,2,3,2)*F(3,2)^2*Fbar(3,3)*Fbar(3,2)+C(2,2,2,3)*F(3,2)^2*Fbar(3,2)*Fbar(3,3)+C(3,1,2,3)*F(3,3)*F(3,1)*Fbar(3,2)*Fbar(3,3)+C(3,2,3,1)*F(3,3)*F(3,2)*Fbar(3,3)*Fbar(3,1)+C(1,3,3,2)*F(3,1)*F(3,3)*Fbar(3,3)*Fbar(3,2)+C(1,1,3,3)*F(3,1)^2*Fbar(3,3)^2+C(3,1,1,1)*F(3,3)*F(3,1)*Fbar(3,1)^2+C(1,2,1,2)*F(3,1)*F(3,2)*Fbar(3,1)*Fbar(3,2)+C(2,2,3,1)*F(3,2)^2*Fbar(3,3)*Fbar(3,1)+C(2,1,1,3)*F(3,2)*F(3,1)*Fbar(3,1)*Fbar(3,3)+C(1,3,1,1)*F(3,1)*F(3,3)*Fbar(3,1)^2+C(2,3,1,1)*F(3,2)*F(3,3)*Fbar(3,1)^2+C(2,3,3,1)*F(3,2)*F(3,3)*Fbar(3,3)*Fbar(3,1)+C(1,2,3,1)*F(3,1)*F(3,2)*Fbar(3,3)*Fbar(3,1)+C(1,3,1,3)*F(3,1)*F(3,3)*Fbar(3,1)*Fbar(3,3)+C(3,3,1,1)*F(3,3)^2*Fbar(3,1)^2+C(3,1,3,3)*F(3,3)*F(3,1)*Fbar(3,3)^2+C(1,1,2,2)*F(3,1)^2*Fbar(3,2)^2+C(1,1,2,3)*F(3,1)^2*Fbar(3,2)*Fbar(3,3)+C(2,2,1,2)*F(3,2)^2*Fbar(3,1)*Fbar(3,2)+C(1,1,3,1)*F(3,1)^2*Fbar(3,3)*Fbar(3,1)+C(1,1,3,2)*F(3,1)^2*Fbar(3,3)*Fbar(3,2)+C(1,2,3,2)*F(3,1)*F(3,2)*Fbar(3,3)*Fbar(3,2)+C(1,2,3,3)*F(3,1)*F(3,2)*Fbar(3,3)^2+C(2,1,2,1)*F(3,2)*F(3,1)*Fbar(3,2)*Fbar(3,1)+C(2,2,1,3)*F(3,2)^2*Fbar(3,1)*Fbar(3,3)+C(2,2,3,3)*F(3,2)^2*Fbar(3,3)^2+C(3,1,1,3)*F(3,3)*F(3,1)*Fbar(3,1)*Fbar(3,3)+C(2,1,3,3)*F(3,2)*F(3,1)*Fbar(3,3)^2+C(2,3,2,2)*F(3,2)*F(3,3)*Fbar(3,2)^2+C(1,3,1,2)*F(3,1)*F(3,3)*Fbar(3,1)*Fbar(3,2)+C(3,1,3,1)*F(3,3)*F(3,1)*Fbar(3,3)*Fbar(3,1)+C(2,1,3,1)*F(3,2)*F(3,1)*Fbar(3,3)*Fbar(3,1)+C(2,3,2,3)*F(3,2)*F(3,3)*Fbar(3,2)*Fbar(3,3)+C(2,2,2,1)*F(3,2)^2*Fbar(3,2)*Fbar(3,1)+C(3,2,2,3)*F(3,3)*F(3,2)*Fbar(3,2)*Fbar(3,3)+C(2,3,3,2)*F(3,2)*F(3,3)*Fbar(3,3)*Fbar(3,2)+C(2,1,3,2)*F(3,2)*F(3,1)*Fbar(3,3)*Fbar(3,2)+C(2,2,2,2)*F(3,2)^2*Fbar(3,2)^2+C(3,2,1,3)*F(3,3)*F(3,2)*Fbar(3,1)*Fbar(3,3)+C(1,3,2,2)*F(3,1)*F(3,3)*Fbar(3,2)^2+C(3,1,2,2)*F(3,3)*F(3,1)*Fbar(3,2)^2+C(1,3,3,3)*F(3,1)*F(3,3)*Fbar(3,3)^2+C(3,3,2,2)*F(3,3)^2*Fbar(3,2)^2+C(2,1,1,1)*F(3,2)*F(3,1)*Fbar(3,1)^2+C(1,3,2,1)*F(3,1)*F(3,3)*Fbar(3,2)*Fbar(3,1)+C(2,1,1,2)*F(3,2)*F(3,1)*Fbar(3,1)*Fbar(3,2)+C(1,1,1,1)*F(3,1)^2*Fbar(3,1)^2+C(1,1,1,2)*F(3,1)^2*Fbar(3,1)*Fbar(3,2)+C(1,1,1,3)*F(3,1)^2*Fbar(3,1)*Fbar(3,3)+C(1,3,2,3)*F(3,1)*F(3,3)*Fbar(3,2)*Fbar(3,3)+C(2,3,3,3)*F(3,2)*F(3,3)*Fbar(3,3)^2;
D(3,4)=C(1,1,2,1)*F(3,1)^2*Fbar(2,2)*Fbar(1,1)+C(2,2,3,3)*F(3,2)^2*Fbar(1,3)*Fbar(2,3)+C(1,1,2,3)*F(3,1)^2*Fbar(2,2)*Fbar(1,3)+C(2,2,2,3)*F(3,2)^2*Fbar(2,2)*Fbar(1,3)+C(2,2,1,3)*F(3,2)^2*Fbar(2,1)*Fbar(1,3)+C(3,3,1,3)*F(3,3)^2*Fbar(2,1)*Fbar(1,3)+C(1,1,2,2)*F(3,1)^2*Fbar(1,2)*Fbar(2,2)+C(2,2,3,2)*F(3,2)^2*Fbar(2,3)*Fbar(1,2)+C(3,3,3,1)*F(3,3)^2*Fbar(2,3)*Fbar(1,1)+C(2,1,3,1)*F(3,2)*F(3,1)*Fbar(2,3)*Fbar(1,1)+C(1,3,2,2)*F(3,1)*F(3,3)*Fbar(1,2)*Fbar(2,2)+C(3,1,1,2)*F(3,3)*F(3,1)*Fbar(2,1)*Fbar(1,2)+C(1,3,1,2)*F(3,1)*F(3,3)*Fbar(2,1)*Fbar(1,2)+C(2,3,2,1)*F(3,2)*F(3,3)*Fbar(2,2)*Fbar(1,1)+C(1,2,2,1)*F(3,1)*F(3,2)*Fbar(2,2)*Fbar(1,1)+C(1,3,2,1)*F(3,1)*F(3,3)*Fbar(2,2)*Fbar(1,1)+C(1,3,1,1)*F(3,1)*F(3,3)*Fbar(1,1)*Fbar(2,1)+C(2,1,1,2)*F(3,2)*F(3,1)*Fbar(2,1)*Fbar(1,2)+C(1,2,1,2)*F(3,1)*F(3,2)*Fbar(2,1)*Fbar(1,2)+C(1,3,3,1)*F(3,1)*F(3,3)*Fbar(2,3)*Fbar(1,1)+C(3,1,3,2)*F(3,3)*F(3,1)*Fbar(2,3)*Fbar(1,2)+C(1,1,1,1)*F(3,1)^2*Fbar(1,1)*Fbar(2,1)+C(1,1,3,2)*F(3,1)^2*Fbar(2,3)*Fbar(1,2)+C(3,2,2,1)*F(3,3)*F(3,2)*Fbar(2,2)*Fbar(1,1)+C(3,1,1,1)*F(3,3)*F(3,1)*Fbar(1,1)*Fbar(2,1)+C(2,3,1,1)*F(3,2)*F(3,3)*Fbar(1,1)*Fbar(2,1)+C(2,3,3,2)*F(3,2)*F(3,3)*Fbar(2,3)*Fbar(1,2)+C(2,1,2,2)*F(3,2)*F(3,1)*Fbar(1,2)*Fbar(2,2)+C(3,1,1,3)*F(3,3)*F(3,1)*Fbar(2,1)*Fbar(1,3)+C(3,2,1,1)*F(3,3)*F(3,2)*Fbar(1,1)*Fbar(2,1)+C(3,2,2,2)*F(3,3)*F(3,2)*Fbar(2,2)*Fbar(1,2)+C(3,2,1,2)*F(3,3)*F(3,2)*Fbar(2,1)*Fbar(1,2)+C(2,3,3,1)*F(3,2)*F(3,3)*Fbar(2,3)*Fbar(1,1)+C(1,1,1,3)*F(3,1)^2*Fbar(2,1)*Fbar(1,3)+C(2,2,1,1)*F(3,2)^2*Fbar(1,1)*Fbar(2,1)+C(1,3,3,3)*F(3,1)*F(3,3)*Fbar(1,3)*Fbar(2,3)+C(2,3,1,2)*F(3,2)*F(3,3)*Fbar(2,1)*Fbar(1,2)+C(1,3,1,3)*F(3,1)*F(3,3)*Fbar(2,1)*Fbar(1,3)+C(3,1,2,1)*F(3,3)*F(3,1)*Fbar(2,2)*Fbar(1,1)+C(2,3,2,3)*F(3,2)*F(3,3)*Fbar(2,2)*Fbar(1,3)+C(3,1,2,3)*F(3,3)*F(3,1)*Fbar(2,2)*Fbar(1,3)+C(1,3,2,3)*F(3,1)*F(3,3)*Fbar(2,2)*Fbar(1,3)+C(1,2,1,3)*F(3,1)*F(3,2)*Fbar(2,1)*Fbar(1,3)+C(1,2,2,2)*F(3,1)*F(3,2)*Fbar(1,2)*Fbar(2,2)+C(1,2,3,3)*F(3,1)*F(3,2)*Fbar(1,3)*Fbar(2,3)+C(1,3,3,2)*F(3,1)*F(3,3)*Fbar(2,3)*Fbar(1,2)+C(2,1,3,2)*F(3,2)*F(3,1)*Fbar(2,3)*Fbar(1,2)+C(2,3,1,3)*F(3,2)*F(3,3)*Fbar(2,1)*Fbar(1,3)+C(1,2,2,3)*F(3,1)*F(3,2)*Fbar(2,2)*Fbar(1,3)+C(1,1,3,1)*F(3,1)^2*Fbar(2,3)*Fbar(1,1)+C(2,1,2,3)*F(3,2)*F(3,1)*Fbar(2,2)*Fbar(1,3)+C(2,1,1,3)*F(3,2)*F(3,1)*Fbar(2,1)*Fbar(1,3)+C(2,1,3,3)*F(3,2)*F(3,1)*Fbar(1,3)*Fbar(2,3)+C(3,2,1,3)*F(3,3)*F(3,2)*Fbar(2,1)*Fbar(1,3)+C(1,1,1,2)*F(3,1)^2*Fbar(2,1)*Fbar(1,2)+C(2,2,3,1)*F(3,2)^2*Fbar(2,3)*Fbar(1,1)+C(3,3,2,1)*F(3,3)^2*Fbar(2,2)*Fbar(1,1)+C(3,3,3,2)*F(3,3)^2*Fbar(2,3)*Fbar(1,2)+C(3,3,1,1)*F(3,3)^2*Fbar(2,1)*Fbar(1,1)+C(1,1,3,3)*F(3,1)^2*Fbar(1,3)*Fbar(2,3)+C(3,3,3,3)*F(3,3)^2*Fbar(2,3)*Fbar(1,3)+C(3,3,2,2)*F(3,3)^2*Fbar(2,2)*Fbar(1,2)+C(2,2,2,2)*F(3,2)^2*Fbar(1,2)*Fbar(2,2)+C(3,3,1,2)*F(3,3)^2*Fbar(2,1)*Fbar(1,2)+C(2,2,2,1)*F(3,2)^2*Fbar(2,2)*Fbar(1,1)+C(2,2,1,2)*F(3,2)^2*Fbar(2,1)*Fbar(1,2)+C(2,3,2,2)*F(3,2)*F(3,3)*Fbar(1,2)*Fbar(2,2)+C(1,2,3,2)*F(3,1)*F(3,2)*Fbar(2,3)*Fbar(1,2)+C(3,2,2,3)*F(3,3)*F(3,2)*Fbar(2,2)*Fbar(1,3)+C(3,1,2,2)*F(3,3)*F(3,1)*Fbar(1,2)*Fbar(2,2)+C(2,1,2,1)*F(3,2)*F(3,1)*Fbar(2,2)*Fbar(1,1)+C(2,1,1,1)*F(3,2)*F(3,1)*Fbar(1,1)*Fbar(2,1)+C(3,1,3,1)*F(3,3)*F(3,1)*Fbar(2,3)*Fbar(1,1)+C(1,2,1,1)*F(3,1)*F(3,2)*Fbar(1,1)*Fbar(2,1)+C(3,2,3,2)*F(3,3)*F(3,2)*Fbar(2,3)*Fbar(1,2)+C(1,2,3,1)*F(3,1)*F(3,2)*Fbar(2,3)*Fbar(1,1)+C(2,3,3,3)*F(3,2)*F(3,3)*Fbar(1,3)*Fbar(2,3)+C(3,1,3,3)*F(3,3)*F(3,1)*Fbar(1,3)*Fbar(2,3)+C(3,2,3,1)*F(3,3)*F(3,2)*Fbar(2,3)*Fbar(1,1)+C(3,2,3,3)*F(3,3)*F(3,2)*Fbar(2,3)*Fbar(1,3)+C(3,3,2,3)*F(3,3)^2*Fbar(2,2)*Fbar(1,3);
D(3,5)=C(1,1,1,2)*F(3,1)^2*Fbar(3,1)*Fbar(1,2)+C(2,2,3,2)*F(3,2)^2*Fbar(3,3)*Fbar(1,2)+C(3,3,1,2)*F(3,3)^2*Fbar(3,1)*Fbar(1,2)+C(1,1,1,1)*F(3,1)^2*Fbar(3,1)*Fbar(1,1)+C(2,2,1,3)*F(3,2)^2*Fbar(3,1)*Fbar(1,3)+C(2,2,1,2)*F(3,2)^2*Fbar(3,1)*Fbar(1,2)+C(2,2,2,3)*F(3,2)^2*Fbar(3,2)*Fbar(1,3)+C(3,3,3,2)*F(3,3)^2*Fbar(3,3)*Fbar(1,2)+C(2,1,1,3)*F(3,2)*F(3,1)*Fbar(3,1)*Fbar(1,3)+C(3,1,3,1)*F(3,3)*F(3,1)*Fbar(3,3)*Fbar(1,1)+C(3,2,2,2)*F(3,3)*F(3,2)*Fbar(3,2)*Fbar(1,2)+C(1,2,1,3)*F(3,1)*F(3,2)*Fbar(3,1)*Fbar(1,3)+C(1,2,3,2)*F(3,1)*F(3,2)*Fbar(3,3)*Fbar(1,2)+C(3,2,3,2)*F(3,3)*F(3,2)*Fbar(3,3)*Fbar(1,2)+C(1,3,3,1)*F(3,1)*F(3,3)*Fbar(3,3)*Fbar(1,1)+C(1,2,2,1)*F(3,1)*F(3,2)*Fbar(3,2)*Fbar(1,1)+C(1,2,2,3)*F(3,1)*F(3,2)*Fbar(3,2)*Fbar(1,3)+C(1,3,2,1)*F(3,1)*F(3,3)*Fbar(3,2)*Fbar(1,1)+C(1,2,1,2)*F(3,1)*F(3,2)*Fbar(3,1)*Fbar(1,2)+C(1,3,1,3)*F(3,1)*F(3,3)*Fbar(3,1)*Fbar(1,3)+C(2,3,3,1)*F(3,2)*F(3,3)*Fbar(3,3)*Fbar(1,1)+C(3,2,1,3)*F(3,3)*F(3,2)*Fbar(3,1)*Fbar(1,3)+C(3,2,3,3)*F(3,3)*F(3,2)*Fbar(3,3)*Fbar(1,3)+C(2,3,3,3)*F(3,2)*F(3,3)*Fbar(3,3)*Fbar(1,3)+C(2,1,3,2)*F(3,2)*F(3,1)*Fbar(3,3)*Fbar(1,2)+C(2,1,3,3)*F(3,2)*F(3,1)*Fbar(3,3)*Fbar(1,3)+C(3,1,1,1)*F(3,3)*F(3,1)*Fbar(3,1)*Fbar(1,1)+C(2,3,2,3)*F(3,2)*F(3,3)*Fbar(3,2)*Fbar(1,3)+C(3,1,1,3)*F(3,3)*F(3,1)*Fbar(3,1)*Fbar(1,3)+C(1,2,1,1)*F(3,1)*F(3,2)*Fbar(3,1)*Fbar(1,1)+C(1,3,1,1)*F(3,1)*F(3,3)*Fbar(3,1)*Fbar(1,1)+C(2,1,1,2)*F(3,2)*F(3,1)*Fbar(3,1)*Fbar(1,2)+C(3,2,2,1)*F(3,3)*F(3,2)*Fbar(3,2)*Fbar(1,1)+C(1,3,3,2)*F(3,1)*F(3,3)*Fbar(3,3)*Fbar(1,2)+C(3,3,1,1)*F(3,3)^2*Fbar(3,1)*Fbar(1,1)+C(3,3,2,1)*F(3,3)^2*Fbar(3,2)*Fbar(1,1)+C(3,3,3,1)*F(3,3)^2*Fbar(3,3)*Fbar(1,1)+C(1,1,2,1)*F(3,1)^2*Fbar(3,2)*Fbar(1,1)+C(1,1,2,2)*F(3,1)^2*Fbar(3,2)*Fbar(1,2)+C(1,1,2,3)*F(3,1)^2*Fbar(3,2)*Fbar(1,3)+C(2,2,1,1)*F(3,2)^2*Fbar(3,1)*Fbar(1,1)+C(1,1,3,2)*F(3,1)^2*Fbar(3,3)*Fbar(1,2)+C(3,3,2,2)*F(3,3)^2*Fbar(3,2)*Fbar(1,2)+C(1,1,3,1)*F(3,1)^2*Fbar(3,3)*Fbar(1,1)+C(1,1,3,3)*F(3,1)^2*Fbar(3,3)*Fbar(1,3)+C(1,1,1,3)*F(3,1)^2*Fbar(3,1)*Fbar(1,3)+C(3,3,3,3)*F(3,3)^2*Fbar(3,3)*Fbar(1,3)+C(3,3,1,3)*F(3,3)^2*Fbar(3,1)*Fbar(1,3)+C(2,2,3,3)*F(3,2)^2*Fbar(3,3)*Fbar(1,3)+C(3,3,2,3)*F(3,3)^2*Fbar(3,2)*Fbar(1,3)+C(3,1,1,2)*F(3,3)*F(3,1)*Fbar(3,1)*Fbar(1,2)+C(2,3,1,2)*F(3,2)*F(3,3)*Fbar(3,1)*Fbar(1,2)+C(2,3,1,1)*F(3,2)*F(3,3)*Fbar(3,1)*Fbar(1,1)+C(2,1,2,1)*F(3,2)*F(3,1)*Fbar(3,2)*Fbar(1,1)+C(2,1,2,2)*F(3,2)*F(3,1)*Fbar(3,2)*Fbar(1,2)+C(2,1,3,1)*F(3,2)*F(3,1)*Fbar(3,3)*Fbar(1,1)+C(2,3,2,1)*F(3,2)*F(3,3)*Fbar(3,2)*Fbar(1,1)+C(3,1,2,2)*F(3,3)*F(3,1)*Fbar(3,2)*Fbar(1,2)+C(3,1,3,2)*F(3,3)*F(3,1)*Fbar(3,3)*Fbar(1,2)+C(3,1,2,3)*F(3,3)*F(3,1)*Fbar(3,2)*Fbar(1,3)+C(2,3,3,2)*F(3,2)*F(3,3)*Fbar(3,3)*Fbar(1,2)+C(2,3,2,2)*F(3,2)*F(3,3)*Fbar(3,2)*Fbar(1,2)+C(2,1,2,3)*F(3,2)*F(3,1)*Fbar(3,2)*Fbar(1,3)+C(2,3,1,3)*F(3,2)*F(3,3)*Fbar(3,1)*Fbar(1,3)+C(3,2,1,2)*F(3,3)*F(3,2)*Fbar(3,1)*Fbar(1,2)+C(1,2,2,2)*F(3,1)*F(3,2)*Fbar(3,2)*Fbar(1,2)+C(2,2,3,1)*F(3,2)^2*Fbar(3,3)*Fbar(1,1)+C(2,2,2,1)*F(3,2)^2*Fbar(3,2)*Fbar(1,1)+C(2,2,2,2)*F(3,2)^2*Fbar(3,2)*Fbar(1,2)+C(1,2,3,3)*F(3,1)*F(3,2)*Fbar(3,3)*Fbar(1,3)+C(3,2,2,3)*F(3,3)*F(3,2)*Fbar(3,2)*Fbar(1,3)+C(3,2,1,1)*F(3,3)*F(3,2)*Fbar(3,1)*Fbar(1,1)+C(2,1,1,1)*F(3,2)*F(3,1)*Fbar(3,1)*Fbar(1,1)+C(1,3,1,2)*F(3,1)*F(3,3)*Fbar(3,1)*Fbar(1,2)+C(1,2,3,1)*F(3,1)*F(3,2)*Fbar(3,3)*Fbar(1,1)+C(1,3,2,2)*F(3,1)*F(3,3)*Fbar(3,2)*Fbar(1,2)+C(3,1,3,3)*F(3,3)*F(3,1)*Fbar(3,3)*Fbar(1,3)+C(1,3,2,3)*F(3,1)*F(3,3)*Fbar(3,2)*Fbar(1,3)+C(3,2,3,1)*F(3,3)*F(3,2)*Fbar(3,3)*Fbar(1,1)+C(3,1,2,1)*F(3,3)*F(3,1)*Fbar(3,2)*Fbar(1,1)+C(1,3,3,3)*F(3,1)*F(3,3)*Fbar(3,3)*Fbar(1,3);
D(3,6)=C(3,1,1,1)*F(3,3)*F(3,1)*Fbar(3,1)*Fbar(2,1)+C(3,1,2,3)*F(3,3)*F(3,1)*Fbar(3,2)*Fbar(2,3)+C(3,2,2,2)*F(3,3)*F(3,2)*Fbar(3,2)*Fbar(2,2)+C(3,1,1,2)*F(3,3)*F(3,1)*Fbar(3,1)*Fbar(2,2)+C(1,3,3,1)*F(3,1)*F(3,3)*Fbar(3,3)*Fbar(2,1)+C(1,3,3,3)*F(3,1)*F(3,3)*Fbar(3,3)*Fbar(2,3)+C(1,3,1,3)*F(3,1)*F(3,3)*Fbar(3,1)*Fbar(2,3)+C(1,2,2,2)*F(3,1)*F(3,2)*Fbar(3,2)*Fbar(2,2)+C(3,1,2,1)*F(3,3)*F(3,1)*Fbar(3,2)*Fbar(2,1)+C(1,3,2,1)*F(3,1)*F(3,3)*Fbar(3,2)*Fbar(2,1)+C(1,2,2,3)*F(3,1)*F(3,2)*Fbar(3,2)*Fbar(2,3)+C(3,1,3,2)*F(3,3)*F(3,1)*Fbar(3,3)*Fbar(2,2)+C(2,1,1,3)*F(3,2)*F(3,1)*Fbar(3,1)*Fbar(2,3)+C(2,1,2,3)*F(3,2)*F(3,1)*Fbar(3,2)*Fbar(2,3)+C(2,2,3,3)*F(3,2)^2*Fbar(3,3)*Fbar(2,3)+C(1,2,1,3)*F(3,1)*F(3,2)*Fbar(3,1)*Fbar(2,3)+C(1,2,1,2)*F(3,1)*F(3,2)*Fbar(3,1)*Fbar(2,2)+C(1,1,2,2)*F(3,1)^2*Fbar(3,2)*Fbar(2,2)+C(1,2,1,1)*F(3,1)*F(3,2)*Fbar(3,1)*Fbar(2,1)+C(1,3,1,2)*F(3,1)*F(3,3)*Fbar(3,1)*Fbar(2,2)+C(2,3,2,2)*F(3,2)*F(3,3)*Fbar(3,2)*Fbar(2,2)+C(2,1,2,1)*F(3,2)*F(3,1)*Fbar(3,2)*Fbar(2,1)+C(2,1,3,1)*F(3,2)*F(3,1)*Fbar(3,3)*Fbar(2,1)+C(2,1,2,2)*F(3,2)*F(3,1)*Fbar(3,2)*Fbar(2,2)+C(3,1,2,2)*F(3,3)*F(3,1)*Fbar(3,2)*Fbar(2,2)+C(1,3,1,1)*F(3,1)*F(3,3)*Fbar(3,1)*Fbar(2,1)+C(3,2,3,2)*F(3,3)*F(3,2)*Fbar(3,3)*Fbar(2,2)+C(2,1,3,3)*F(3,2)*F(3,1)*Fbar(3,3)*Fbar(2,3)+C(2,3,2,1)*F(3,2)*F(3,3)*Fbar(3,2)*Fbar(2,1)+C(1,3,2,3)*F(3,1)*F(3,3)*Fbar(3,2)*Fbar(2,3)+C(3,2,2,1)*F(3,3)*F(3,2)*Fbar(3,2)*Fbar(2,1)+C(3,3,3,3)*F(3,3)^2*Fbar(3,3)*Fbar(2,3)+C(1,2,3,1)*F(3,1)*F(3,2)*Fbar(3,3)*Fbar(2,1)+C(2,1,1,1)*F(3,2)*F(3,1)*Fbar(3,1)*Fbar(2,1)+C(2,3,3,3)*F(3,2)*F(3,3)*Fbar(3,3)*Fbar(2,3)+C(3,1,3,1)*F(3,3)*F(3,1)*Fbar(3,3)*Fbar(2,1)+C(2,3,1,2)*F(3,2)*F(3,3)*Fbar(3,1)*Fbar(2,2)+C(3,2,1,2)*F(3,3)*F(3,2)*Fbar(3,1)*Fbar(2,2)+C(3,2,3,1)*F(3,3)*F(3,2)*Fbar(3,3)*Fbar(2,1)+C(1,3,2,2)*F(3,1)*F(3,3)*Fbar(3,2)*Fbar(2,2)+C(3,2,1,3)*F(3,3)*F(3,2)*Fbar(3,1)*Fbar(2,3)+C(2,3,1,1)*F(3,2)*F(3,3)*Fbar(3,1)*Fbar(2,1)+C(3,1,1,3)*F(3,3)*F(3,1)*Fbar(3,1)*Fbar(2,3)+C(3,2,1,1)*F(3,3)*F(3,2)*Fbar(3,1)*Fbar(2,1)+C(2,3,3,1)*F(3,2)*F(3,3)*Fbar(3,3)*Fbar(2,1)+C(2,1,1,2)*F(3,2)*F(3,1)*Fbar(3,1)*Fbar(2,2)+C(1,2,3,3)*F(3,1)*F(3,2)*Fbar(3,3)*Fbar(2,3)+C(2,1,3,2)*F(3,2)*F(3,1)*Fbar(3,3)*Fbar(2,2)+C(2,3,1,3)*F(3,2)*F(3,3)*Fbar(3,1)*Fbar(2,3)+C(2,3,2,3)*F(3,2)*F(3,3)*Fbar(3,2)*Fbar(2,3)+C(3,2,2,3)*F(3,3)*F(3,2)*Fbar(3,2)*Fbar(2,3)+C(1,2,3,2)*F(3,1)*F(3,2)*Fbar(3,3)*Fbar(2,2)+C(2,3,3,2)*F(3,2)*F(3,3)*Fbar(3,3)*Fbar(2,2)+C(3,1,3,3)*F(3,3)*F(3,1)*Fbar(3,3)*Fbar(2,3)+C(3,2,3,3)*F(3,3)*F(3,2)*Fbar(3,3)*Fbar(2,3)+C(1,2,2,1)*F(3,1)*F(3,2)*Fbar(3,2)*Fbar(2,1)+C(1,3,3,2)*F(3,1)*F(3,3)*Fbar(3,3)*Fbar(2,2)+C(2,2,1,3)*F(3,2)^2*Fbar(3,1)*Fbar(2,3)+C(2,2,1,1)*F(3,2)^2*Fbar(3,1)*Fbar(2,1)+C(1,1,3,3)*F(3,1)^2*Fbar(3,3)*Fbar(2,3)+C(1,1,2,3)*F(3,1)^2*Fbar(3,2)*Fbar(2,3)+C(3,3,1,1)*F(3,3)^2*Fbar(3,1)*Fbar(2,1)+C(2,2,1,2)*F(3,2)^2*Fbar(3,1)*Fbar(2,2)+C(2,2,3,1)*F(3,2)^2*Fbar(3,3)*Fbar(2,1)+C(3,3,2,1)*F(3,3)^2*Fbar(3,2)*Fbar(2,1)+C(1,1,3,2)*F(3,1)^2*Fbar(3,3)*Fbar(2,2)+C(1,1,1,3)*F(3,1)^2*Fbar(3,1)*Fbar(2,3)+C(1,1,3,1)*F(3,1)^2*Fbar(3,3)*Fbar(2,1)+C(3,3,1,2)*F(3,3)^2*Fbar(3,1)*Fbar(2,2)+C(1,1,2,1)*F(3,1)^2*Fbar(3,2)*Fbar(2,1)+C(3,3,1,3)*F(3,3)^2*Fbar(3,1)*Fbar(2,3)+C(3,3,3,1)*F(3,3)^2*Fbar(3,3)*Fbar(2,1)+C(2,2,2,2)*F(3,2)^2*Fbar(3,2)*Fbar(2,2)+C(3,3,2,2)*F(3,3)^2*Fbar(3,2)*Fbar(2,2)+C(2,2,3,2)*F(3,2)^2*Fbar(3,3)*Fbar(2,2)+C(3,3,3,2)*F(3,3)^2*Fbar(3,3)*Fbar(2,2)+C(2,2,2,3)*F(3,2)^2*Fbar(3,2)*Fbar(2,3)+C(3,3,2,3)*F(3,3)^2*Fbar(3,2)*Fbar(2,3)+C(1,1,1,2)*F(3,1)^2*Fbar(3,1)*Fbar(2,2)+C(1,1,1,1)*F(3,1)^2*Fbar(3,1)*Fbar(2,1)+C(2,2,2,1)*F(3,2)^2*Fbar(3,2)*Fbar(2,1);
D(4,1)=C(1,2,3,2)*F(2,1)*F(1,2)*Fbar(1,3)*Fbar(1,2)+C(1,1,3,3)*F(2,1)*F(1,1)*Fbar(1,3)^2+C(2,3,3,3)*F(2,2)*F(1,3)*Fbar(1,3)^2+C(2,2,1,1)*F(2,2)*F(1,2)*Fbar(1,1)^2+C(1,1,2,2)*F(2,1)*F(1,1)*Fbar(1,2)^2+C(3,2,1,1)*F(2,3)*F(1,2)*Fbar(1,1)^2+C(2,2,2,2)*F(2,2)*F(1,2)*Fbar(1,2)^2+C(3,2,2,2)*F(2,3)*F(1,2)*Fbar(1,2)^2+C(2,1,3,3)*F(2,2)*F(1,1)*Fbar(1,3)^2+C(1,2,3,3)*F(2,1)*F(1,2)*Fbar(1,3)^2+C(1,2,1,1)*F(2,1)*F(1,2)*Fbar(1,1)^2+C(3,1,1,1)*F(2,3)*F(1,1)*Fbar(1,1)^2+C(2,3,2,2)*F(2,2)*F(1,3)*Fbar(1,2)^2+C(1,3,2,2)*F(2,1)*F(1,3)*Fbar(1,2)^2+C(3,3,2,2)*F(2,3)*F(1,3)*Fbar(1,2)^2+C(2,3,2,3)*F(2,2)*F(1,3)*Fbar(1,2)*Fbar(1,3)+C(3,1,3,2)*F(2,3)*F(1,1)*Fbar(1,3)*Fbar(1,2)+C(3,2,1,3)*F(2,3)*F(1,2)*Fbar(1,1)*Fbar(1,3)+C(1,2,2,1)*F(2,1)*F(1,2)*Fbar(1,2)*Fbar(1,1)+C(1,3,3,2)*F(2,1)*F(1,3)*Fbar(1,3)*Fbar(1,2)+C(2,3,1,3)*F(2,2)*F(1,3)*Fbar(1,1)*Fbar(1,3)+C(3,3,3,2)*F(2,3)*F(1,3)*Fbar(1,3)*Fbar(1,2)+C(3,2,3,2)*F(2,3)*F(1,2)*Fbar(1,3)*Fbar(1,2)+C(3,3,2,3)*F(2,3)*F(1,3)*Fbar(1,2)*Fbar(1,3)+C(2,1,1,3)*F(2,2)*F(1,1)*Fbar(1,1)*Fbar(1,3)+C(3,2,3,1)*F(2,3)*F(1,2)*Fbar(1,3)*Fbar(1,1)+C(1,1,2,1)*F(2,1)*F(1,1)*Fbar(1,2)*Fbar(1,1)+C(3,2,2,1)*F(2,3)*F(1,2)*Fbar(1,2)*Fbar(1,1)+C(3,1,2,3)*F(2,3)*F(1,1)*Fbar(1,2)*Fbar(1,3)+C(2,2,1,2)*F(2,2)*F(1,2)*Fbar(1,1)*Fbar(1,2)+C(2,1,2,3)*F(2,2)*F(1,1)*Fbar(1,2)*Fbar(1,3)+C(2,2,2,1)*F(2,2)*F(1,2)*Fbar(1,2)*Fbar(1,1)+C(1,2,2,2)*F(2,1)*F(1,2)*Fbar(1,2)^2+C(2,3,1,1)*F(2,2)*F(1,3)*Fbar(1,1)^2+C(3,3,3,3)*F(2,3)*F(1,3)*Fbar(1,3)^2+C(2,1,2,2)*F(2,2)*F(1,1)*Fbar(1,2)^2+C(2,2,3,3)*F(2,2)*F(1,2)*Fbar(1,3)^2+C(3,1,1,2)*F(2,3)*F(1,1)*Fbar(1,1)*Fbar(1,2)+C(3,3,1,2)*F(2,3)*F(1,3)*Fbar(1,1)*Fbar(1,2)+C(1,2,2,3)*F(2,1)*F(1,2)*Fbar(1,2)*Fbar(1,3)+C(3,3,2,1)*F(2,3)*F(1,3)*Fbar(1,2)*Fbar(1,1)+C(3,3,1,3)*F(2,3)*F(1,3)*Fbar(1,1)*Fbar(1,3)+C(2,3,2,1)*F(2,2)*F(1,3)*Fbar(1,2)*Fbar(1,1)+C(1,3,1,2)*F(2,1)*F(1,3)*Fbar(1,1)*Fbar(1,2)+C(2,2,3,2)*F(2,2)*F(1,2)*Fbar(1,3)*Fbar(1,2)+C(3,2,1,2)*F(2,3)*F(1,2)*Fbar(1,1)*Fbar(1,2)+C(3,1,2,1)*F(2,3)*F(1,1)*Fbar(1,2)*Fbar(1,1)+C(2,1,2,1)*F(2,2)*F(1,1)*Fbar(1,2)*Fbar(1,1)+C(2,2,3,1)*F(2,2)*F(1,2)*Fbar(1,3)*Fbar(1,1)+C(2,1,1,2)*F(2,2)*F(1,1)*Fbar(1,1)*Fbar(1,2)+C(1,3,3,3)*F(2,1)*F(1,3)*Fbar(1,3)^2+C(3,1,3,3)*F(2,3)*F(1,1)*Fbar(1,3)^2+C(2,1,1,1)*F(2,2)*F(1,1)*Fbar(1,1)^2+C(3,1,2,2)*F(2,3)*F(1,1)*Fbar(1,2)^2+C(1,1,1,1)*F(2,1)*F(1,1)*Fbar(1,1)^2+C(3,3,1,1)*F(2,3)*F(1,3)*Fbar(1,1)^2+C(2,1,3,2)*F(2,2)*F(1,1)*Fbar(1,3)*Fbar(1,2)+C(2,2,2,3)*F(2,2)*F(1,2)*Fbar(1,2)*Fbar(1,3)+C(1,2,3,1)*F(2,1)*F(1,2)*Fbar(1,3)*Fbar(1,1)+C(1,1,3,1)*F(2,1)*F(1,1)*Fbar(1,3)*Fbar(1,1)+C(3,1,3,1)*F(2,3)*F(1,1)*Fbar(1,3)*Fbar(1,1)+C(2,3,3,1)*F(2,2)*F(1,3)*Fbar(1,3)*Fbar(1,1)+C(1,1,2,3)*F(2,1)*F(1,1)*Fbar(1,2)*Fbar(1,3)+C(2,1,3,1)*F(2,2)*F(1,1)*Fbar(1,3)*Fbar(1,1)+C(3,2,2,3)*F(2,3)*F(1,2)*Fbar(1,2)*Fbar(1,3)+C(3,1,1,3)*F(2,3)*F(1,1)*Fbar(1,1)*Fbar(1,3)+C(1,3,1,1)*F(2,1)*F(1,3)*Fbar(1,1)^2+C(3,2,3,3)*F(2,3)*F(1,2)*Fbar(1,3)^2+C(1,1,1,2)*F(2,1)*F(1,1)*Fbar(1,1)*Fbar(1,2)+C(2,2,1,3)*F(2,2)*F(1,2)*Fbar(1,1)*Fbar(1,3)+C(1,1,1,3)*F(2,1)*F(1,1)*Fbar(1,1)*Fbar(1,3)+C(1,3,1,3)*F(2,1)*F(1,3)*Fbar(1,1)*Fbar(1,3)+C(2,3,3,2)*F(2,2)*F(1,3)*Fbar(1,3)*Fbar(1,2)+C(3,3,3,1)*F(2,3)*F(1,3)*Fbar(1,3)*Fbar(1,1)+C(2,3,1,2)*F(2,2)*F(1,3)*Fbar(1,1)*Fbar(1,2)+C(1,2,1,2)*F(2,1)*F(1,2)*Fbar(1,1)*Fbar(1,2)+C(1,1,3,2)*F(2,1)*F(1,1)*Fbar(1,3)*Fbar(1,2)+C(1,2,1,3)*F(2,1)*F(1,2)*Fbar(1,1)*Fbar(1,3)+C(1,3,3,1)*F(2,1)*F(1,3)*Fbar(1,3)*Fbar(1,1)+C(1,3,2,3)*F(2,1)*F(1,3)*Fbar(1,2)*Fbar(1,3)+C(1,3,2,1)*F(2,1)*F(1,3)*Fbar(1,2)*Fbar(1,1);
D(4,2)=C(3,1,3,1)*F(2,3)*F(1,1)*Fbar(2,3)*Fbar(2,1)+C(3,2,1,3)*F(2,3)*F(1,2)*Fbar(2,1)*Fbar(2,3)+C(2,1,2,3)*F(2,2)*F(1,1)*Fbar(2,2)*Fbar(2,3)+C(3,3,1,3)*F(2,3)*F(1,3)*Fbar(2,1)*Fbar(2,3)+C(3,1,1,2)*F(2,3)*F(1,1)*Fbar(2,1)*Fbar(2,2)+C(2,3,3,1)*F(2,2)*F(1,3)*Fbar(2,3)*Fbar(2,1)+C(3,2,2,1)*F(2,3)*F(1,2)*Fbar(2,2)*Fbar(2,1)+C(2,2,3,1)*F(2,2)*F(1,2)*Fbar(2,3)*Fbar(2,1)+C(3,1,2,3)*F(2,3)*F(1,1)*Fbar(2,2)*Fbar(2,3)+C(1,2,2,3)*F(2,1)*F(1,2)*Fbar(2,2)*Fbar(2,3)+C(3,2,3,2)*F(2,3)*F(1,2)*Fbar(2,3)*Fbar(2,2)+C(2,2,1,3)*F(2,2)*F(1,2)*Fbar(2,1)*Fbar(2,3)+C(3,2,1,2)*F(2,3)*F(1,2)*Fbar(2,1)*Fbar(2,2)+C(1,2,1,2)*F(2,1)*F(1,2)*Fbar(2,1)*Fbar(2,2)+C(2,1,2,1)*F(2,2)*F(1,1)*Fbar(2,2)*Fbar(2,1)+C(3,3,1,2)*F(2,3)*F(1,3)*Fbar(2,1)*Fbar(2,2)+C(1,1,1,2)*F(2,1)*F(1,1)*Fbar(2,1)*Fbar(2,2)+C(1,2,3,2)*F(2,1)*F(1,2)*Fbar(2,3)*Fbar(2,2)+C(3,1,3,3)*F(2,3)*F(1,1)*Fbar(2,3)^2+C(1,3,3,3)*F(2,1)*F(1,3)*Fbar(2,3)^2+C(3,3,1,1)*F(2,3)*F(1,3)*Fbar(2,1)^2+C(3,2,1,1)*F(2,3)*F(1,2)*Fbar(2,1)^2+C(2,3,2,2)*F(2,2)*F(1,3)*Fbar(2,2)^2+C(2,1,3,3)*F(2,2)*F(1,1)*Fbar(2,3)^2+C(1,1,2,3)*F(2,1)*F(1,1)*Fbar(2,2)*Fbar(2,3)+C(1,2,1,3)*F(2,1)*F(1,2)*Fbar(2,1)*Fbar(2,3)+C(3,3,2,2)*F(2,3)*F(1,3)*Fbar(2,2)^2+C(1,2,3,3)*F(2,1)*F(1,2)*Fbar(2,3)^2+C(1,1,2,2)*F(2,1)*F(1,1)*Fbar(2,2)^2+C(2,1,1,2)*F(2,2)*F(1,1)*Fbar(2,1)*Fbar(2,2)+C(1,3,2,3)*F(2,1)*F(1,3)*Fbar(2,2)*Fbar(2,3)+C(1,1,3,1)*F(2,1)*F(1,1)*Fbar(2,3)*Fbar(2,1)+C(2,2,1,2)*F(2,2)*F(1,2)*Fbar(2,1)*Fbar(2,2)+C(1,1,3,2)*F(2,1)*F(1,1)*Fbar(2,3)*Fbar(2,2)+C(3,1,2,1)*F(2,3)*F(1,1)*Fbar(2,2)*Fbar(2,1)+C(3,3,2,1)*F(2,3)*F(1,3)*Fbar(2,2)*Fbar(2,1)+C(2,3,1,2)*F(2,2)*F(1,3)*Fbar(2,1)*Fbar(2,2)+C(2,1,3,1)*F(2,2)*F(1,1)*Fbar(2,3)*Fbar(2,1)+C(3,2,3,1)*F(2,3)*F(1,2)*Fbar(2,3)*Fbar(2,1)+C(1,3,1,2)*F(2,1)*F(1,3)*Fbar(2,1)*Fbar(2,2)+C(1,3,3,1)*F(2,1)*F(1,3)*Fbar(2,3)*Fbar(2,1)+C(3,2,2,2)*F(2,3)*F(1,2)*Fbar(2,2)^2+C(1,2,2,2)*F(2,1)*F(1,2)*Fbar(2,2)^2+C(2,1,1,1)*F(2,2)*F(1,1)*Fbar(2,1)^2+C(2,2,2,2)*F(2,2)*F(1,2)*Fbar(2,2)^2+C(1,2,1,1)*F(2,1)*F(1,2)*Fbar(2,1)^2+C(1,3,1,1)*F(2,1)*F(1,3)*Fbar(2,1)^2+C(2,3,3,3)*F(2,2)*F(1,3)*Fbar(2,3)^2+C(1,1,3,3)*F(2,1)*F(1,1)*Fbar(2,3)^2+C(3,2,3,3)*F(2,3)*F(1,2)*Fbar(2,3)^2+C(2,2,1,1)*F(2,2)*F(1,2)*Fbar(2,1)^2+C(1,3,2,2)*F(2,1)*F(1,3)*Fbar(2,2)^2+C(2,2,3,3)*F(2,2)*F(1,2)*Fbar(2,3)^2+C(3,1,1,1)*F(2,3)*F(1,1)*Fbar(2,1)^2+C(1,1,1,1)*F(2,1)*F(1,1)*Fbar(2,1)^2+C(2,3,1,1)*F(2,2)*F(1,3)*Fbar(2,1)^2+C(3,1,2,2)*F(2,3)*F(1,1)*Fbar(2,2)^2+C(2,1,2,2)*F(2,2)*F(1,1)*Fbar(2,2)^2+C(3,3,3,3)*F(2,3)*F(1,3)*Fbar(2,3)^2+C(1,1,1,3)*F(2,1)*F(1,1)*Fbar(2,1)*Fbar(2,3)+C(1,3,1,3)*F(2,1)*F(1,3)*Fbar(2,1)*Fbar(2,3)+C(2,1,3,2)*F(2,2)*F(1,1)*Fbar(2,3)*Fbar(2,2)+C(3,3,3,1)*F(2,3)*F(1,3)*Fbar(2,3)*Fbar(2,1)+C(2,3,3,2)*F(2,2)*F(1,3)*Fbar(2,3)*Fbar(2,2)+C(3,3,3,2)*F(2,3)*F(1,3)*Fbar(2,3)*Fbar(2,2)+C(1,2,2,1)*F(2,1)*F(1,2)*Fbar(2,2)*Fbar(2,1)+C(2,3,2,3)*F(2,2)*F(1,3)*Fbar(2,2)*Fbar(2,3)+C(2,2,2,3)*F(2,2)*F(1,2)*Fbar(2,2)*Fbar(2,3)+C(3,3,2,3)*F(2,3)*F(1,3)*Fbar(2,2)*Fbar(2,3)+C(2,2,2,1)*F(2,2)*F(1,2)*Fbar(2,2)*Fbar(2,1)+C(2,1,1,3)*F(2,2)*F(1,1)*Fbar(2,1)*Fbar(2,3)+C(1,2,3,1)*F(2,1)*F(1,2)*Fbar(2,3)*Fbar(2,1)+C(3,1,3,2)*F(2,3)*F(1,1)*Fbar(2,3)*Fbar(2,2)+C(3,1,1,3)*F(2,3)*F(1,1)*Fbar(2,1)*Fbar(2,3)+C(1,1,2,1)*F(2,1)*F(1,1)*Fbar(2,2)*Fbar(2,1)+C(3,2,2,3)*F(2,3)*F(1,2)*Fbar(2,2)*Fbar(2,3)+C(1,3,2,1)*F(2,1)*F(1,3)*Fbar(2,2)*Fbar(2,1)+C(2,3,1,3)*F(2,2)*F(1,3)*Fbar(2,1)*Fbar(2,3)+C(2,3,2,1)*F(2,2)*F(1,3)*Fbar(2,2)*Fbar(2,1)+C(2,2,3,2)*F(2,2)*F(1,2)*Fbar(2,3)*Fbar(2,2)+C(1,3,3,2)*F(2,1)*F(1,3)*Fbar(2,3)*Fbar(2,2);
D(4,3)=C(3,2,3,1)*F(2,3)*F(1,2)*Fbar(3,3)*Fbar(3,1)+C(2,1,3,2)*F(2,2)*F(1,1)*Fbar(3,3)*Fbar(3,2)+C(1,1,3,2)*F(2,1)*F(1,1)*Fbar(3,3)*Fbar(3,2)+C(1,2,3,1)*F(2,1)*F(1,2)*Fbar(3,3)*Fbar(3,1)+C(1,2,3,2)*F(2,1)*F(1,2)*Fbar(3,3)*Fbar(3,2)+C(3,1,3,2)*F(2,3)*F(1,1)*Fbar(3,3)*Fbar(3,2)+C(2,3,3,2)*F(2,2)*F(1,3)*Fbar(3,3)*Fbar(3,2)+C(3,1,1,3)*F(2,3)*F(1,1)*Fbar(3,1)*Fbar(3,3)+C(3,2,2,3)*F(2,3)*F(1,2)*Fbar(3,2)*Fbar(3,3)+C(3,1,2,1)*F(2,3)*F(1,1)*Fbar(3,2)*Fbar(3,1)+C(3,2,1,3)*F(2,3)*F(1,2)*Fbar(3,1)*Fbar(3,3)+C(3,3,1,3)*F(2,3)*F(1,3)*Fbar(3,1)*Fbar(3,3)+C(3,1,2,3)*F(2,3)*F(1,1)*Fbar(3,2)*Fbar(3,3)+C(3,1,1,2)*F(2,3)*F(1,1)*Fbar(3,1)*Fbar(3,2)+C(2,3,3,1)*F(2,2)*F(1,3)*Fbar(3,3)*Fbar(3,1)+C(2,2,1,2)*F(2,2)*F(1,2)*Fbar(3,1)*Fbar(3,2)+C(2,3,2,3)*F(2,2)*F(1,3)*Fbar(3,2)*Fbar(3,3)+C(1,3,2,3)*F(2,1)*F(1,3)*Fbar(3,2)*Fbar(3,3)+C(2,3,1,3)*F(2,2)*F(1,3)*Fbar(3,1)*Fbar(3,3)+C(2,1,2,3)*F(2,2)*F(1,1)*Fbar(3,2)*Fbar(3,3)+C(2,3,1,2)*F(2,2)*F(1,3)*Fbar(3,1)*Fbar(3,2)+C(3,3,1,2)*F(2,3)*F(1,3)*Fbar(3,1)*Fbar(3,2)+C(3,2,1,2)*F(2,3)*F(1,2)*Fbar(3,1)*Fbar(3,2)+C(2,1,3,1)*F(2,2)*F(1,1)*Fbar(3,3)*Fbar(3,1)+C(1,2,1,2)*F(2,1)*F(1,2)*Fbar(3,1)*Fbar(3,2)+C(1,3,3,1)*F(2,1)*F(1,3)*Fbar(3,3)*Fbar(3,1)+C(2,2,3,2)*F(2,2)*F(1,2)*Fbar(3,3)*Fbar(3,2)+C(1,3,3,2)*F(2,1)*F(1,3)*Fbar(3,3)*Fbar(3,2)+C(1,1,1,3)*F(2,1)*F(1,1)*Fbar(3,1)*Fbar(3,3)+C(2,2,2,1)*F(2,2)*F(1,2)*Fbar(3,2)*Fbar(3,1)+C(2,1,1,2)*F(2,2)*F(1,1)*Fbar(3,1)*Fbar(3,2)+C(1,2,2,1)*F(2,1)*F(1,2)*Fbar(3,2)*Fbar(3,1)+C(2,1,1,3)*F(2,2)*F(1,1)*Fbar(3,1)*Fbar(3,3)+C(3,3,3,2)*F(2,3)*F(1,3)*Fbar(3,3)*Fbar(3,2)+C(2,2,3,1)*F(2,2)*F(1,2)*Fbar(3,3)*Fbar(3,1)+C(1,3,2,1)*F(2,1)*F(1,3)*Fbar(3,2)*Fbar(3,1)+C(1,2,1,3)*F(2,1)*F(1,2)*Fbar(3,1)*Fbar(3,3)+C(2,2,2,3)*F(2,2)*F(1,2)*Fbar(3,2)*Fbar(3,3)+C(1,1,2,1)*F(2,1)*F(1,1)*Fbar(3,2)*Fbar(3,1)+C(1,3,1,2)*F(2,1)*F(1,3)*Fbar(3,1)*Fbar(3,2)+C(2,2,1,3)*F(2,2)*F(1,2)*Fbar(3,1)*Fbar(3,3)+C(1,1,1,2)*F(2,1)*F(1,1)*Fbar(3,1)*Fbar(3,2)+C(3,3,3,1)*F(2,3)*F(1,3)*Fbar(3,3)*Fbar(3,1)+C(2,1,2,1)*F(2,2)*F(1,1)*Fbar(3,2)*Fbar(3,1)+C(1,3,1,3)*F(2,1)*F(1,3)*Fbar(3,1)*Fbar(3,3)+C(3,3,2,1)*F(2,3)*F(1,3)*Fbar(3,2)*Fbar(3,1)+C(1,1,3,1)*F(2,1)*F(1,1)*Fbar(3,3)*Fbar(3,1)+C(1,2,2,3)*F(2,1)*F(1,2)*Fbar(3,2)*Fbar(3,3)+C(2,3,2,1)*F(2,2)*F(1,3)*Fbar(3,2)*Fbar(3,1)+C(3,1,3,1)*F(2,3)*F(1,1)*Fbar(3,3)*Fbar(3,1)+C(3,3,2,3)*F(2,3)*F(1,3)*Fbar(3,2)*Fbar(3,3)+C(3,2,3,2)*F(2,3)*F(1,2)*Fbar(3,3)*Fbar(3,2)+C(1,1,2,3)*F(2,1)*F(1,1)*Fbar(3,2)*Fbar(3,3)+C(3,2,2,1)*F(2,3)*F(1,2)*Fbar(3,2)*Fbar(3,1)+C(1,3,1,1)*F(2,1)*F(1,3)*Fbar(3,1)^2+C(3,3,3,3)*F(2,3)*F(1,3)*Fbar(3,3)^2+C(2,1,2,2)*F(2,2)*F(1,1)*Fbar(3,2)^2+C(2,2,3,3)*F(2,2)*F(1,2)*Fbar(3,3)^2+C(1,2,1,1)*F(2,1)*F(1,2)*Fbar(3,1)^2+C(3,1,1,1)*F(2,3)*F(1,1)*Fbar(3,1)^2+C(2,3,3,3)*F(2,2)*F(1,3)*Fbar(3,3)^2+C(3,1,2,2)*F(2,3)*F(1,1)*Fbar(3,2)^2+C(3,2,3,3)*F(2,3)*F(1,2)*Fbar(3,3)^2+C(3,2,1,1)*F(2,3)*F(1,2)*Fbar(3,1)^2+C(3,1,3,3)*F(2,3)*F(1,1)*Fbar(3,3)^2+C(1,2,2,2)*F(2,1)*F(1,2)*Fbar(3,2)^2+C(2,3,2,2)*F(2,2)*F(1,3)*Fbar(3,2)^2+C(1,1,3,3)*F(2,1)*F(1,1)*Fbar(3,3)^2+C(1,2,3,3)*F(2,1)*F(1,2)*Fbar(3,3)^2+C(2,2,2,2)*F(2,2)*F(1,2)*Fbar(3,2)^2+C(3,3,1,1)*F(2,3)*F(1,3)*Fbar(3,1)^2+C(2,1,1,1)*F(2,2)*F(1,1)*Fbar(3,1)^2+C(3,2,2,2)*F(2,3)*F(1,2)*Fbar(3,2)^2+C(1,3,2,2)*F(2,1)*F(1,3)*Fbar(3,2)^2+C(2,3,1,1)*F(2,2)*F(1,3)*Fbar(3,1)^2+C(1,1,1,1)*F(2,1)*F(1,1)*Fbar(3,1)^2+C(1,3,3,3)*F(2,1)*F(1,3)*Fbar(3,3)^2+C(3,3,2,2)*F(2,3)*F(1,3)*Fbar(3,2)^2+C(2,1,3,3)*F(2,2)*F(1,1)*Fbar(3,3)^2+C(1,1,2,2)*F(2,1)*F(1,1)*Fbar(3,2)^2+C(2,2,1,1)*F(2,2)*F(1,2)*Fbar(3,1)^2;
D(4,4)=C(3,3,2,2)*F(2,3)*F(1,3)*Fbar(2,2)*Fbar(1,2)+C(3,2,1,1)*F(2,3)*F(1,2)*Fbar(1,1)*Fbar(2,1)+C(3,3,1,2)*F(2,3)*F(1,3)*Fbar(2,1)*Fbar(1,2)+C(1,2,1,3)*F(2,1)*F(1,2)*Fbar(2,1)*Fbar(1,3)+C(2,3,2,2)*F(2,2)*F(1,3)*Fbar(1,2)*Fbar(2,2)+C(1,3,2,1)*F(2,1)*F(1,3)*Fbar(2,2)*Fbar(1,1)+C(3,2,2,2)*F(2,3)*F(1,2)*Fbar(1,2)*Fbar(2,2)+C(2,1,3,2)*F(2,2)*F(1,1)*Fbar(2,3)*Fbar(1,2)+C(3,3,3,3)*F(2,3)*F(1,3)*Fbar(2,3)*Fbar(1,3)+C(3,2,1,2)*F(2,3)*F(1,2)*Fbar(2,1)*Fbar(1,2)+C(3,3,1,1)*F(2,3)*F(1,3)*Fbar(2,1)*Fbar(1,1)+C(3,1,2,2)*F(2,3)*F(1,1)*Fbar(1,2)*Fbar(2,2)+C(3,3,3,2)*F(2,3)*F(1,3)*Fbar(2,3)*Fbar(1,2)+C(3,2,3,3)*F(2,3)*F(1,2)*Fbar(2,3)*Fbar(1,3)+C(3,3,3,1)*F(2,3)*F(1,3)*Fbar(2,3)*Fbar(1,1)+C(3,3,1,3)*F(2,3)*F(1,3)*Fbar(2,1)*Fbar(1,3)+C(1,3,3,3)*F(2,1)*F(1,3)*Fbar(1,3)*Fbar(2,3)+C(2,3,3,3)*F(2,2)*F(1,3)*Fbar(1,3)*Fbar(2,3)+C(1,1,2,3)*F(2,1)*F(1,1)*Fbar(2,2)*Fbar(1,3)+C(1,3,2,3)*F(2,1)*F(1,3)*Fbar(2,2)*Fbar(1,3)+C(3,2,3,1)*F(2,3)*F(1,2)*Fbar(2,3)*Fbar(1,1)+C(2,2,1,1)*F(2,2)*F(1,2)*Fbar(1,1)*Fbar(2,1)+C(1,2,2,1)*F(2,1)*F(1,2)*Fbar(2,2)*Fbar(1,1)+C(2,1,2,1)*F(2,2)*F(1,1)*Fbar(2,2)*Fbar(1,1)+C(1,1,2,2)*F(2,1)*F(1,1)*Fbar(1,2)*Fbar(2,2)+C(3,2,2,3)*F(2,3)*F(1,2)*Fbar(2,2)*Fbar(1,3)+C(2,2,2,3)*F(2,2)*F(1,2)*Fbar(2,2)*Fbar(1,3)+C(1,1,1,1)*F(2,1)*F(1,1)*Fbar(1,1)*Fbar(2,1)+C(1,2,3,1)*F(2,1)*F(1,2)*Fbar(2,3)*Fbar(1,1)+C(2,3,1,2)*F(2,2)*F(1,3)*Fbar(2,1)*Fbar(1,2)+C(2,1,2,3)*F(2,2)*F(1,1)*Fbar(2,2)*Fbar(1,3)+C(1,1,2,1)*F(2,1)*F(1,1)*Fbar(2,2)*Fbar(1,1)+C(3,2,1,3)*F(2,3)*F(1,2)*Fbar(2,1)*Fbar(1,3)+C(1,3,2,2)*F(2,1)*F(1,3)*Fbar(1,2)*Fbar(2,2)+C(1,2,1,1)*F(2,1)*F(1,2)*Fbar(1,1)*Fbar(2,1)+C(2,2,2,1)*F(2,2)*F(1,2)*Fbar(2,2)*Fbar(1,1)+C(3,1,2,3)*F(2,3)*F(1,1)*Fbar(2,2)*Fbar(1,3)+C(2,2,2,2)*F(2,2)*F(1,2)*Fbar(1,2)*Fbar(2,2)+C(1,3,1,2)*F(2,1)*F(1,3)*Fbar(2,1)*Fbar(1,2)+C(1,2,3,2)*F(2,1)*F(1,2)*Fbar(2,3)*Fbar(1,2)+C(2,1,1,2)*F(2,2)*F(1,1)*Fbar(2,1)*Fbar(1,2)+C(3,2,2,1)*F(2,3)*F(1,2)*Fbar(2,2)*Fbar(1,1)+C(3,1,1,1)*F(2,3)*F(1,1)*Fbar(1,1)*Fbar(2,1)+C(2,2,3,2)*F(2,2)*F(1,2)*Fbar(2,3)*Fbar(1,2)+C(2,3,2,1)*F(2,2)*F(1,3)*Fbar(2,2)*Fbar(1,1)+C(1,1,1,2)*F(2,1)*F(1,1)*Fbar(2,1)*Fbar(1,2)+C(1,1,1,3)*F(2,1)*F(1,1)*Fbar(2,1)*Fbar(1,3)+C(1,1,3,2)*F(2,1)*F(1,1)*Fbar(2,3)*Fbar(1,2)+C(1,3,1,1)*F(2,1)*F(1,3)*Fbar(1,1)*Fbar(2,1)+C(3,3,2,1)*F(2,3)*F(1,3)*Fbar(2,2)*Fbar(1,1)+C(3,1,3,3)*F(2,3)*F(1,1)*Fbar(1,3)*Fbar(2,3)+C(1,3,1,3)*F(2,1)*F(1,3)*Fbar(2,1)*Fbar(1,3)+C(3,1,3,1)*F(2,3)*F(1,1)*Fbar(2,3)*Fbar(1,1)+C(2,3,1,1)*F(2,2)*F(1,3)*Fbar(1,1)*Fbar(2,1)+C(1,1,3,1)*F(2,1)*F(1,1)*Fbar(2,3)*Fbar(1,1)+C(2,1,1,3)*F(2,2)*F(1,1)*Fbar(2,1)*Fbar(1,3)+C(2,1,1,1)*F(2,2)*F(1,1)*Fbar(1,1)*Fbar(2,1)+C(2,1,2,2)*F(2,2)*F(1,1)*Fbar(1,2)*Fbar(2,2)+C(1,2,3,3)*F(2,1)*F(1,2)*Fbar(1,3)*Fbar(2,3)+C(2,2,1,3)*F(2,2)*F(1,2)*Fbar(2,1)*Fbar(1,3)+C(2,3,3,1)*F(2,2)*F(1,3)*Fbar(2,3)*Fbar(1,1)+C(1,2,2,2)*F(2,1)*F(1,2)*Fbar(1,2)*Fbar(2,2)+C(2,2,1,2)*F(2,2)*F(1,2)*Fbar(2,1)*Fbar(1,2)+C(1,2,2,3)*F(2,1)*F(1,2)*Fbar(2,2)*Fbar(1,3)+C(2,1,3,1)*F(2,2)*F(1,1)*Fbar(2,3)*Fbar(1,1)+C(3,1,2,1)*F(2,3)*F(1,1)*Fbar(2,2)*Fbar(1,1)+C(2,3,1,3)*F(2,2)*F(1,3)*Fbar(2,1)*Fbar(1,3)+C(2,2,3,1)*F(2,2)*F(1,2)*Fbar(2,3)*Fbar(1,1)+C(3,1,1,3)*F(2,3)*F(1,1)*Fbar(2,1)*Fbar(1,3)+C(2,1,3,3)*F(2,2)*F(1,1)*Fbar(1,3)*Fbar(2,3)+C(1,3,3,2)*F(2,1)*F(1,3)*Fbar(2,3)*Fbar(1,2)+C(3,1,1,2)*F(2,3)*F(1,1)*Fbar(2,1)*Fbar(1,2)+C(2,3,2,3)*F(2,2)*F(1,3)*Fbar(2,2)*Fbar(1,3)+C(1,3,3,1)*F(2,1)*F(1,3)*Fbar(2,3)*Fbar(1,1)+C(1,2,1,2)*F(2,1)*F(1,2)*Fbar(2,1)*Fbar(1,2)+C(3,3,2,3)*F(2,3)*F(1,3)*Fbar(2,2)*Fbar(1,3)+C(3,2,3,2)*F(2,3)*F(1,2)*Fbar(2,3)*Fbar(1,2)+C(2,3,3,2)*F(2,2)*F(1,3)*Fbar(2,3)*Fbar(1,2)+C(3,1,3,2)*F(2,3)*F(1,1)*Fbar(2,3)*Fbar(1,2)+C(2,2,3,3)*F(2,2)*F(1,2)*Fbar(1,3)*Fbar(2,3)+C(1,1,3,3)*F(2,1)*F(1,1)*Fbar(1,3)*Fbar(2,3);
D(4,5)=C(1,1,2,3)*F(2,1)*F(1,1)*Fbar(3,2)*Fbar(1,3)+C(2,3,3,2)*F(2,2)*F(1,3)*Fbar(3,3)*Fbar(1,2)+C(1,1,1,2)*F(2,1)*F(1,1)*Fbar(3,1)*Fbar(1,2)+C(1,1,1,3)*F(2,1)*F(1,1)*Fbar(3,1)*Fbar(1,3)+C(1,2,2,3)*F(2,1)*F(1,2)*Fbar(3,2)*Fbar(1,3)+C(3,2,2,3)*F(2,3)*F(1,2)*Fbar(3,2)*Fbar(1,3)+C(3,3,2,3)*F(2,3)*F(1,3)*Fbar(3,2)*Fbar(1,3)+C(3,1,1,2)*F(2,3)*F(1,1)*Fbar(3,1)*Fbar(1,2)+C(3,2,1,3)*F(2,3)*F(1,2)*Fbar(3,1)*Fbar(1,3)+C(1,2,2,2)*F(2,1)*F(1,2)*Fbar(3,2)*Fbar(1,2)+C(1,2,1,2)*F(2,1)*F(1,2)*Fbar(3,1)*Fbar(1,2)+C(2,3,3,1)*F(2,2)*F(1,3)*Fbar(3,3)*Fbar(1,1)+C(2,2,1,3)*F(2,2)*F(1,2)*Fbar(3,1)*Fbar(1,3)+C(1,2,3,1)*F(2,1)*F(1,2)*Fbar(3,3)*Fbar(1,1)+C(2,1,3,2)*F(2,2)*F(1,1)*Fbar(3,3)*Fbar(1,2)+C(2,1,2,3)*F(2,2)*F(1,1)*Fbar(3,2)*Fbar(1,3)+C(1,2,3,2)*F(2,1)*F(1,2)*Fbar(3,3)*Fbar(1,2)+C(1,1,2,1)*F(2,1)*F(1,1)*Fbar(3,2)*Fbar(1,1)+C(3,2,3,2)*F(2,3)*F(1,2)*Fbar(3,3)*Fbar(1,2)+C(1,3,1,3)*F(2,1)*F(1,3)*Fbar(3,1)*Fbar(1,3)+C(3,1,2,1)*F(2,3)*F(1,1)*Fbar(3,2)*Fbar(1,1)+C(1,1,1,1)*F(2,1)*F(1,1)*Fbar(3,1)*Fbar(1,1)+C(3,2,3,1)*F(2,3)*F(1,2)*Fbar(3,3)*Fbar(1,1)+C(3,1,3,1)*F(2,3)*F(1,1)*Fbar(3,3)*Fbar(1,1)+C(1,3,2,1)*F(2,1)*F(1,3)*Fbar(3,2)*Fbar(1,1)+C(2,1,3,3)*F(2,2)*F(1,1)*Fbar(3,3)*Fbar(1,3)+C(1,1,3,1)*F(2,1)*F(1,1)*Fbar(3,3)*Fbar(1,1)+C(2,1,1,3)*F(2,2)*F(1,1)*Fbar(3,1)*Fbar(1,3)+C(1,2,1,1)*F(2,1)*F(1,2)*Fbar(3,1)*Fbar(1,1)+C(3,3,1,3)*F(2,3)*F(1,3)*Fbar(3,1)*Fbar(1,3)+C(1,1,2,2)*F(2,1)*F(1,1)*Fbar(3,2)*Fbar(1,2)+C(3,1,3,2)*F(2,3)*F(1,1)*Fbar(3,3)*Fbar(1,2)+C(2,1,2,1)*F(2,2)*F(1,1)*Fbar(3,2)*Fbar(1,1)+C(2,3,2,1)*F(2,2)*F(1,3)*Fbar(3,2)*Fbar(1,1)+C(2,2,2,1)*F(2,2)*F(1,2)*Fbar(3,2)*Fbar(1,1)+C(2,2,1,1)*F(2,2)*F(1,2)*Fbar(3,1)*Fbar(1,1)+C(3,1,2,3)*F(2,3)*F(1,1)*Fbar(3,2)*Fbar(1,3)+C(2,3,2,3)*F(2,2)*F(1,3)*Fbar(3,2)*Fbar(1,3)+C(2,1,1,2)*F(2,2)*F(1,1)*Fbar(3,1)*Fbar(1,2)+C(3,1,1,1)*F(2,3)*F(1,1)*Fbar(3,1)*Fbar(1,1)+C(1,3,3,2)*F(2,1)*F(1,3)*Fbar(3,3)*Fbar(1,2)+C(2,3,1,2)*F(2,2)*F(1,3)*Fbar(3,1)*Fbar(1,2)+C(3,1,2,2)*F(2,3)*F(1,1)*Fbar(3,2)*Fbar(1,2)+C(3,3,2,2)*F(2,3)*F(1,3)*Fbar(3,2)*Fbar(1,2)+C(1,2,2,1)*F(2,1)*F(1,2)*Fbar(3,2)*Fbar(1,1)+C(2,3,2,2)*F(2,2)*F(1,3)*Fbar(3,2)*Fbar(1,2)+C(2,3,3,3)*F(2,2)*F(1,3)*Fbar(3,3)*Fbar(1,3)+C(3,3,1,2)*F(2,3)*F(1,3)*Fbar(3,1)*Fbar(1,2)+C(1,2,1,3)*F(2,1)*F(1,2)*Fbar(3,1)*Fbar(1,3)+C(2,2,2,3)*F(2,2)*F(1,2)*Fbar(3,2)*Fbar(1,3)+C(1,1,3,2)*F(2,1)*F(1,1)*Fbar(3,3)*Fbar(1,2)+C(3,3,2,1)*F(2,3)*F(1,3)*Fbar(3,2)*Fbar(1,1)+C(1,3,2,2)*F(2,1)*F(1,3)*Fbar(3,2)*Fbar(1,2)+C(1,2,3,3)*F(2,1)*F(1,2)*Fbar(3,3)*Fbar(1,3)+C(3,3,1,1)*F(2,3)*F(1,3)*Fbar(3,1)*Fbar(1,1)+C(3,2,1,1)*F(2,3)*F(1,2)*Fbar(3,1)*Fbar(1,1)+C(1,3,1,2)*F(2,1)*F(1,3)*Fbar(3,1)*Fbar(1,2)+C(1,3,2,3)*F(2,1)*F(1,3)*Fbar(3,2)*Fbar(1,3)+C(1,3,3,3)*F(2,1)*F(1,3)*Fbar(3,3)*Fbar(1,3)+C(2,2,2,2)*F(2,2)*F(1,2)*Fbar(3,2)*Fbar(1,2)+C(2,1,1,1)*F(2,2)*F(1,1)*Fbar(3,1)*Fbar(1,1)+C(2,2,1,2)*F(2,2)*F(1,2)*Fbar(3,1)*Fbar(1,2)+C(2,2,3,1)*F(2,2)*F(1,2)*Fbar(3,3)*Fbar(1,1)+C(2,1,2,2)*F(2,2)*F(1,1)*Fbar(3,2)*Fbar(1,2)+C(2,1,3,1)*F(2,2)*F(1,1)*Fbar(3,3)*Fbar(1,1)+C(1,1,3,3)*F(2,1)*F(1,1)*Fbar(3,3)*Fbar(1,3)+C(1,3,3,1)*F(2,1)*F(1,3)*Fbar(3,3)*Fbar(1,1)+C(3,2,2,2)*F(2,3)*F(1,2)*Fbar(3,2)*Fbar(1,2)+C(3,3,3,1)*F(2,3)*F(1,3)*Fbar(3,3)*Fbar(1,1)+C(3,2,1,2)*F(2,3)*F(1,2)*Fbar(3,1)*Fbar(1,2)+C(1,3,1,1)*F(2,1)*F(1,3)*Fbar(3,1)*Fbar(1,1)+C(3,1,1,3)*F(2,3)*F(1,1)*Fbar(3,1)*Fbar(1,3)+C(3,2,3,3)*F(2,3)*F(1,2)*Fbar(3,3)*Fbar(1,3)+C(2,3,1,1)*F(2,2)*F(1,3)*Fbar(3,1)*Fbar(1,1)+C(2,3,1,3)*F(2,2)*F(1,3)*Fbar(3,1)*Fbar(1,3)+C(2,2,3,2)*F(2,2)*F(1,2)*Fbar(3,3)*Fbar(1,2)+C(3,3,3,3)*F(2,3)*F(1,3)*Fbar(3,3)*Fbar(1,3)+C(3,2,2,1)*F(2,3)*F(1,2)*Fbar(3,2)*Fbar(1,1)+C(2,2,3,3)*F(2,2)*F(1,2)*Fbar(3,3)*Fbar(1,3)+C(3,3,3,2)*F(2,3)*F(1,3)*Fbar(3,3)*Fbar(1,2)+C(3,1,3,3)*F(2,3)*F(1,1)*Fbar(3,3)*Fbar(1,3);
D(4,6)=C(3,2,3,2)*F(2,3)*F(1,2)*Fbar(3,3)*Fbar(2,2)+C(2,2,3,3)*F(2,2)*F(1,2)*Fbar(3,3)*Fbar(2,3)+C(1,1,1,2)*F(2,1)*F(1,1)*Fbar(3,1)*Fbar(2,2)+C(3,1,2,1)*F(2,3)*F(1,1)*Fbar(3,2)*Fbar(2,1)+C(1,1,1,3)*F(2,1)*F(1,1)*Fbar(3,1)*Fbar(2,3)+C(3,3,3,1)*F(2,3)*F(1,3)*Fbar(3,3)*Fbar(2,1)+C(2,1,1,2)*F(2,2)*F(1,1)*Fbar(3,1)*Fbar(2,2)+C(2,1,3,1)*F(2,2)*F(1,1)*Fbar(3,3)*Fbar(2,1)+C(2,2,2,2)*F(2,2)*F(1,2)*Fbar(3,2)*Fbar(2,2)+C(1,3,1,2)*F(2,1)*F(1,3)*Fbar(3,1)*Fbar(2,2)+C(2,3,1,2)*F(2,2)*F(1,3)*Fbar(3,1)*Fbar(2,2)+C(1,1,2,2)*F(2,1)*F(1,1)*Fbar(3,2)*Fbar(2,2)+C(2,3,1,1)*F(2,2)*F(1,3)*Fbar(3,1)*Fbar(2,1)+C(3,3,2,1)*F(2,3)*F(1,3)*Fbar(3,2)*Fbar(2,1)+C(3,3,2,3)*F(2,3)*F(1,3)*Fbar(3,2)*Fbar(2,3)+C(1,2,3,2)*F(2,1)*F(1,2)*Fbar(3,3)*Fbar(2,2)+C(3,3,3,2)*F(2,3)*F(1,3)*Fbar(3,3)*Fbar(2,2)+C(1,3,1,1)*F(2,1)*F(1,3)*Fbar(3,1)*Fbar(2,1)+C(1,1,1,1)*F(2,1)*F(1,1)*Fbar(3,1)*Fbar(2,1)+C(2,3,1,3)*F(2,2)*F(1,3)*Fbar(3,1)*Fbar(2,3)+C(3,2,2,2)*F(2,3)*F(1,2)*Fbar(3,2)*Fbar(2,2)+C(3,1,1,2)*F(2,3)*F(1,1)*Fbar(3,1)*Fbar(2,2)+C(2,3,2,2)*F(2,2)*F(1,3)*Fbar(3,2)*Fbar(2,2)+C(1,1,3,1)*F(2,1)*F(1,1)*Fbar(3,3)*Fbar(2,1)+C(3,3,2,2)*F(2,3)*F(1,3)*Fbar(3,2)*Fbar(2,2)+C(1,3,3,1)*F(2,1)*F(1,3)*Fbar(3,3)*Fbar(2,1)+C(2,2,3,1)*F(2,2)*F(1,2)*Fbar(3,3)*Fbar(2,1)+C(3,3,1,2)*F(2,3)*F(1,3)*Fbar(3,1)*Fbar(2,2)+C(3,1,2,2)*F(2,3)*F(1,1)*Fbar(3,2)*Fbar(2,2)+C(3,3,3,3)*F(2,3)*F(1,3)*Fbar(3,3)*Fbar(2,3)+C(2,2,1,2)*F(2,2)*F(1,2)*Fbar(3,1)*Fbar(2,2)+C(3,2,1,3)*F(2,3)*F(1,2)*Fbar(3,1)*Fbar(2,3)+C(3,2,2,3)*F(2,3)*F(1,2)*Fbar(3,2)*Fbar(2,3)+C(2,3,2,3)*F(2,2)*F(1,3)*Fbar(3,2)*Fbar(2,3)+C(2,2,3,2)*F(2,2)*F(1,2)*Fbar(3,3)*Fbar(2,2)+C(1,3,3,3)*F(2,1)*F(1,3)*Fbar(3,3)*Fbar(2,3)+C(2,2,2,3)*F(2,2)*F(1,2)*Fbar(3,2)*Fbar(2,3)+C(3,1,3,3)*F(2,3)*F(1,1)*Fbar(3,3)*Fbar(2,3)+C(1,2,2,2)*F(2,1)*F(1,2)*Fbar(3,2)*Fbar(2,2)+C(1,3,3,2)*F(2,1)*F(1,3)*Fbar(3,3)*Fbar(2,2)+C(3,2,3,1)*F(2,3)*F(1,2)*Fbar(3,3)*Fbar(2,1)+C(1,2,2,3)*F(2,1)*F(1,2)*Fbar(3,2)*Fbar(2,3)+C(1,1,3,2)*F(2,1)*F(1,1)*Fbar(3,3)*Fbar(2,2)+C(2,3,3,1)*F(2,2)*F(1,3)*Fbar(3,3)*Fbar(2,1)+C(1,2,1,1)*F(2,1)*F(1,2)*Fbar(3,1)*Fbar(2,1)+C(3,2,1,1)*F(2,3)*F(1,2)*Fbar(3,1)*Fbar(2,1)+C(2,1,3,3)*F(2,2)*F(1,1)*Fbar(3,3)*Fbar(2,3)+C(1,2,1,2)*F(2,1)*F(1,2)*Fbar(3,1)*Fbar(2,2)+C(3,1,1,1)*F(2,3)*F(1,1)*Fbar(3,1)*Fbar(2,1)+C(1,2,2,1)*F(2,1)*F(1,2)*Fbar(3,2)*Fbar(2,1)+C(1,2,1,3)*F(2,1)*F(1,2)*Fbar(3,1)*Fbar(2,3)+C(1,2,3,3)*F(2,1)*F(1,2)*Fbar(3,3)*Fbar(2,3)+C(2,2,2,1)*F(2,2)*F(1,2)*Fbar(3,2)*Fbar(2,1)+C(3,1,3,1)*F(2,3)*F(1,1)*Fbar(3,3)*Fbar(2,1)+C(3,3,1,1)*F(2,3)*F(1,3)*Fbar(3,1)*Fbar(2,1)+C(2,1,1,1)*F(2,2)*F(1,1)*Fbar(3,1)*Fbar(2,1)+C(2,2,1,3)*F(2,2)*F(1,2)*Fbar(3,1)*Fbar(2,3)+C(3,2,2,1)*F(2,3)*F(1,2)*Fbar(3,2)*Fbar(2,1)+C(2,1,2,3)*F(2,2)*F(1,1)*Fbar(3,2)*Fbar(2,3)+C(3,1,1,3)*F(2,3)*F(1,1)*Fbar(3,1)*Fbar(2,3)+C(1,2,3,1)*F(2,1)*F(1,2)*Fbar(3,3)*Fbar(2,1)+C(2,1,3,2)*F(2,2)*F(1,1)*Fbar(3,3)*Fbar(2,2)+C(1,1,3,3)*F(2,1)*F(1,1)*Fbar(3,3)*Fbar(2,3)+C(3,3,1,3)*F(2,3)*F(1,3)*Fbar(3,1)*Fbar(2,3)+C(1,1,2,3)*F(2,1)*F(1,1)*Fbar(3,2)*Fbar(2,3)+C(3,1,3,2)*F(2,3)*F(1,1)*Fbar(3,3)*Fbar(2,2)+C(2,1,1,3)*F(2,2)*F(1,1)*Fbar(3,1)*Fbar(2,3)+C(1,3,2,1)*F(2,1)*F(1,3)*Fbar(3,2)*Fbar(2,1)+C(2,3,3,2)*F(2,2)*F(1,3)*Fbar(3,3)*Fbar(2,2)+C(3,2,1,2)*F(2,3)*F(1,2)*Fbar(3,1)*Fbar(2,2)+C(3,1,2,3)*F(2,3)*F(1,1)*Fbar(3,2)*Fbar(2,3)+C(1,3,2,3)*F(2,1)*F(1,3)*Fbar(3,2)*Fbar(2,3)+C(2,1,2,1)*F(2,2)*F(1,1)*Fbar(3,2)*Fbar(2,1)+C(1,3,2,2)*F(2,1)*F(1,3)*Fbar(3,2)*Fbar(2,2)+C(2,2,1,1)*F(2,2)*F(1,2)*Fbar(3,1)*Fbar(2,1)+C(2,3,2,1)*F(2,2)*F(1,3)*Fbar(3,2)*Fbar(2,1)+C(2,3,3,3)*F(2,2)*F(1,3)*Fbar(3,3)*Fbar(2,3)+C(3,2,3,3)*F(2,3)*F(1,2)*Fbar(3,3)*Fbar(2,3)+C(2,1,2,2)*F(2,2)*F(1,1)*Fbar(3,2)*Fbar(2,2)+C(1,1,2,1)*F(2,1)*F(1,1)*Fbar(3,2)*Fbar(2,1)+C(1,3,1,3)*F(2,1)*F(1,3)*Fbar(3,1)*Fbar(2,3);
D(5,1)=C(1,2,2,1)*F(3,1)*F(1,2)*Fbar(1,2)*Fbar(1,1)+C(1,2,3,1)*F(3,1)*F(1,2)*Fbar(1,3)*Fbar(1,1)+C(3,1,2,1)*F(3,3)*F(1,1)*Fbar(1,2)*Fbar(1,1)+C(3,3,3,2)*F(3,3)*F(1,3)*Fbar(1,3)*Fbar(1,2)+C(2,3,1,2)*F(3,2)*F(1,3)*Fbar(1,1)*Fbar(1,2)+C(3,3,2,1)*F(3,3)*F(1,3)*Fbar(1,2)*Fbar(1,1)+C(3,3,2,2)*F(3,3)*F(1,3)*Fbar(1,2)^2+C(1,2,3,3)*F(3,1)*F(1,2)*Fbar(1,3)^2+C(2,1,3,3)*F(3,2)*F(1,1)*Fbar(1,3)^2+C(2,2,2,2)*F(3,2)*F(1,2)*Fbar(1,2)^2+C(1,1,1,1)*F(3,1)*F(1,1)*Fbar(1,1)^2+C(3,1,1,1)*F(3,3)*F(1,1)*Fbar(1,1)^2+C(3,2,2,2)*F(3,3)*F(1,2)*Fbar(1,2)^2+C(1,2,2,2)*F(3,1)*F(1,2)*Fbar(1,2)^2+C(1,1,3,3)*F(3,1)*F(1,1)*Fbar(1,3)^2+C(1,1,1,2)*F(3,1)*F(1,1)*Fbar(1,1)*Fbar(1,2)+C(2,1,3,2)*F(3,2)*F(1,1)*Fbar(1,3)*Fbar(1,2)+C(2,2,1,2)*F(3,2)*F(1,2)*Fbar(1,1)*Fbar(1,2)+C(1,1,1,3)*F(3,1)*F(1,1)*Fbar(1,1)*Fbar(1,3)+C(2,1,1,3)*F(3,2)*F(1,1)*Fbar(1,1)*Fbar(1,3)+C(1,1,2,3)*F(3,1)*F(1,1)*Fbar(1,2)*Fbar(1,3)+C(2,1,1,1)*F(3,2)*F(1,1)*Fbar(1,1)^2+C(2,1,3,1)*F(3,2)*F(1,1)*Fbar(1,3)*Fbar(1,1)+C(2,2,2,1)*F(3,2)*F(1,2)*Fbar(1,2)*Fbar(1,1)+C(2,3,3,1)*F(3,2)*F(1,3)*Fbar(1,3)*Fbar(1,1)+C(2,3,1,1)*F(3,2)*F(1,3)*Fbar(1,1)^2+C(3,3,3,3)*F(3,3)*F(1,3)*Fbar(1,3)^2+C(1,2,1,1)*F(3,1)*F(1,2)*Fbar(1,1)^2+C(1,3,2,2)*F(3,1)*F(1,3)*Fbar(1,2)^2+C(2,3,2,2)*F(3,2)*F(1,3)*Fbar(1,2)^2+C(2,2,3,1)*F(3,2)*F(1,2)*Fbar(1,3)*Fbar(1,1)+C(1,3,2,3)*F(3,1)*F(1,3)*Fbar(1,2)*Fbar(1,3)+C(3,1,3,1)*F(3,3)*F(1,1)*Fbar(1,3)*Fbar(1,1)+C(1,3,1,1)*F(3,1)*F(1,3)*Fbar(1,1)^2+C(2,3,3,3)*F(3,2)*F(1,3)*Fbar(1,3)^2+C(2,3,3,2)*F(3,2)*F(1,3)*Fbar(1,3)*Fbar(1,2)+C(1,1,2,1)*F(3,1)*F(1,1)*Fbar(1,2)*Fbar(1,1)+C(1,2,1,2)*F(3,1)*F(1,2)*Fbar(1,1)*Fbar(1,2)+C(1,3,1,3)*F(3,1)*F(1,3)*Fbar(1,1)*Fbar(1,3)+C(3,1,1,2)*F(3,3)*F(1,1)*Fbar(1,1)*Fbar(1,2)+C(3,2,1,3)*F(3,3)*F(1,2)*Fbar(1,1)*Fbar(1,3)+C(3,2,1,1)*F(3,3)*F(1,2)*Fbar(1,1)^2+C(3,3,1,1)*F(3,3)*F(1,3)*Fbar(1,1)^2+C(3,2,3,3)*F(3,3)*F(1,2)*Fbar(1,3)^2+C(3,1,3,3)*F(3,3)*F(1,1)*Fbar(1,3)^2+C(1,3,3,1)*F(3,1)*F(1,3)*Fbar(1,3)*Fbar(1,1)+C(3,2,1,2)*F(3,3)*F(1,2)*Fbar(1,1)*Fbar(1,2)+C(2,2,3,3)*F(3,2)*F(1,2)*Fbar(1,3)^2+C(1,1,2,2)*F(3,1)*F(1,1)*Fbar(1,2)^2+C(1,3,3,3)*F(3,1)*F(1,3)*Fbar(1,3)^2+C(3,1,2,2)*F(3,3)*F(1,1)*Fbar(1,2)^2+C(2,2,1,1)*F(3,2)*F(1,2)*Fbar(1,1)^2+C(2,1,2,2)*F(3,2)*F(1,1)*Fbar(1,2)^2+C(3,2,3,1)*F(3,3)*F(1,2)*Fbar(1,3)*Fbar(1,1)+C(3,3,2,3)*F(3,3)*F(1,3)*Fbar(1,2)*Fbar(1,3)+C(3,3,3,1)*F(3,3)*F(1,3)*Fbar(1,3)*Fbar(1,1)+C(3,1,3,2)*F(3,3)*F(1,1)*Fbar(1,3)*Fbar(1,2)+C(3,1,2,3)*F(3,3)*F(1,1)*Fbar(1,2)*Fbar(1,3)+C(2,2,1,3)*F(3,2)*F(1,2)*Fbar(1,1)*Fbar(1,3)+C(3,2,3,2)*F(3,3)*F(1,2)*Fbar(1,3)*Fbar(1,2)+C(3,2,2,3)*F(3,3)*F(1,2)*Fbar(1,2)*Fbar(1,3)+C(3,2,2,1)*F(3,3)*F(1,2)*Fbar(1,2)*Fbar(1,1)+C(1,2,3,2)*F(3,1)*F(1,2)*Fbar(1,3)*Fbar(1,2)+C(3,3,1,3)*F(3,3)*F(1,3)*Fbar(1,1)*Fbar(1,3)+C(2,1,2,3)*F(3,2)*F(1,1)*Fbar(1,2)*Fbar(1,3)+C(2,3,2,1)*F(3,2)*F(1,3)*Fbar(1,2)*Fbar(1,1)+C(3,3,1,2)*F(3,3)*F(1,3)*Fbar(1,1)*Fbar(1,2)+C(1,1,3,1)*F(3,1)*F(1,1)*Fbar(1,3)*Fbar(1,1)+C(1,1,3,2)*F(3,1)*F(1,1)*Fbar(1,3)*Fbar(1,2)+C(2,1,1,2)*F(3,2)*F(1,1)*Fbar(1,1)*Fbar(1,2)+C(3,1,1,3)*F(3,3)*F(1,1)*Fbar(1,1)*Fbar(1,3)+C(1,3,2,1)*F(3,1)*F(1,3)*Fbar(1,2)*Fbar(1,1)+C(1,3,3,2)*F(3,1)*F(1,3)*Fbar(1,3)*Fbar(1,2)+C(1,3,1,2)*F(3,1)*F(1,3)*Fbar(1,1)*Fbar(1,2)+C(1,2,2,3)*F(3,1)*F(1,2)*Fbar(1,2)*Fbar(1,3)+C(2,3,2,3)*F(3,2)*F(1,3)*Fbar(1,2)*Fbar(1,3)+C(2,3,1,3)*F(3,2)*F(1,3)*Fbar(1,1)*Fbar(1,3)+C(1,2,1,3)*F(3,1)*F(1,2)*Fbar(1,1)*Fbar(1,3)+C(2,2,2,3)*F(3,2)*F(1,2)*Fbar(1,2)*Fbar(1,3)+C(2,1,2,1)*F(3,2)*F(1,1)*Fbar(1,2)*Fbar(1,1)+C(2,2,3,2)*F(3,2)*F(1,2)*Fbar(1,3)*Fbar(1,2);
D(5,2)=C(2,2,3,1)*F(3,2)*F(1,2)*Fbar(2,3)*Fbar(2,1)+C(1,3,1,1)*F(3,1)*F(1,3)*Fbar(2,1)^2+C(2,2,1,1)*F(3,2)*F(1,2)*Fbar(2,1)^2+C(3,3,3,3)*F(3,3)*F(1,3)*Fbar(2,3)^2+C(3,3,1,1)*F(3,3)*F(1,3)*Fbar(2,1)^2+C(2,3,1,1)*F(3,2)*F(1,3)*Fbar(2,1)^2+C(2,2,2,2)*F(3,2)*F(1,2)*Fbar(2,2)^2+C(2,1,3,3)*F(3,2)*F(1,1)*Fbar(2,3)^2+C(1,3,3,3)*F(3,1)*F(1,3)*Fbar(2,3)^2+C(3,1,2,2)*F(3,3)*F(1,1)*Fbar(2,2)^2+C(1,1,3,3)*F(3,1)*F(1,1)*Fbar(2,3)^2+C(1,3,2,2)*F(3,1)*F(1,3)*Fbar(2,2)^2+C(2,3,3,3)*F(3,2)*F(1,3)*Fbar(2,3)^2+C(3,2,3,3)*F(3,3)*F(1,2)*Fbar(2,3)^2+C(2,2,2,3)*F(3,2)*F(1,2)*Fbar(2,2)*Fbar(2,3)+C(2,2,1,3)*F(3,2)*F(1,2)*Fbar(2,1)*Fbar(2,3)+C(3,3,3,1)*F(3,3)*F(1,3)*Fbar(2,3)*Fbar(2,1)+C(3,1,1,2)*F(3,3)*F(1,1)*Fbar(2,1)*Fbar(2,2)+C(3,3,2,1)*F(3,3)*F(1,3)*Fbar(2,2)*Fbar(2,1)+C(2,1,2,3)*F(3,2)*F(1,1)*Fbar(2,2)*Fbar(2,3)+C(1,2,1,3)*F(3,1)*F(1,2)*Fbar(2,1)*Fbar(2,3)+C(3,2,3,1)*F(3,3)*F(1,2)*Fbar(2,3)*Fbar(2,1)+C(3,2,1,2)*F(3,3)*F(1,2)*Fbar(2,1)*Fbar(2,2)+C(2,3,3,2)*F(3,2)*F(1,3)*Fbar(2,3)*Fbar(2,2)+C(2,3,2,1)*F(3,2)*F(1,3)*Fbar(2,2)*Fbar(2,1)+C(3,2,2,1)*F(3,3)*F(1,2)*Fbar(2,2)*Fbar(2,1)+C(2,3,1,3)*F(3,2)*F(1,3)*Fbar(2,1)*Fbar(2,3)+C(2,3,1,2)*F(3,2)*F(1,3)*Fbar(2,1)*Fbar(2,2)+C(1,2,2,3)*F(3,1)*F(1,2)*Fbar(2,2)*Fbar(2,3)+C(3,1,3,1)*F(3,3)*F(1,1)*Fbar(2,3)*Fbar(2,1)+C(3,1,3,2)*F(3,3)*F(1,1)*Fbar(2,3)*Fbar(2,2)+C(1,1,3,2)*F(3,1)*F(1,1)*Fbar(2,3)*Fbar(2,2)+C(2,1,1,2)*F(3,2)*F(1,1)*Fbar(2,1)*Fbar(2,2)+C(1,3,3,2)*F(3,1)*F(1,3)*Fbar(2,3)*Fbar(2,2)+C(3,3,2,3)*F(3,3)*F(1,3)*Fbar(2,2)*Fbar(2,3)+C(3,2,2,3)*F(3,3)*F(1,2)*Fbar(2,2)*Fbar(2,3)+C(1,2,3,2)*F(3,1)*F(1,2)*Fbar(2,3)*Fbar(2,2)+C(3,2,3,2)*F(3,3)*F(1,2)*Fbar(2,3)*Fbar(2,2)+C(3,3,1,3)*F(3,3)*F(1,3)*Fbar(2,1)*Fbar(2,3)+C(1,2,1,1)*F(3,1)*F(1,2)*Fbar(2,1)^2+C(2,3,2,2)*F(3,2)*F(1,3)*Fbar(2,2)^2+C(1,1,1,1)*F(3,1)*F(1,1)*Fbar(2,1)^2+C(1,1,2,2)*F(3,1)*F(1,1)*Fbar(2,2)^2+C(1,2,3,3)*F(3,1)*F(1,2)*Fbar(2,3)^2+C(2,2,3,3)*F(3,2)*F(1,2)*Fbar(2,3)^2+C(3,2,1,1)*F(3,3)*F(1,2)*Fbar(2,1)^2+C(2,1,2,2)*F(3,2)*F(1,1)*Fbar(2,2)^2+C(2,1,1,1)*F(3,2)*F(1,1)*Fbar(2,1)^2+C(3,3,2,2)*F(3,3)*F(1,3)*Fbar(2,2)^2+C(1,2,2,2)*F(3,1)*F(1,2)*Fbar(2,2)^2+C(3,1,3,3)*F(3,3)*F(1,1)*Fbar(2,3)^2+C(1,1,2,3)*F(3,1)*F(1,1)*Fbar(2,2)*Fbar(2,3)+C(2,1,2,1)*F(3,2)*F(1,1)*Fbar(2,2)*Fbar(2,1)+C(1,2,1,2)*F(3,1)*F(1,2)*Fbar(2,1)*Fbar(2,2)+C(3,3,3,2)*F(3,3)*F(1,3)*Fbar(2,3)*Fbar(2,2)+C(1,3,3,1)*F(3,1)*F(1,3)*Fbar(2,3)*Fbar(2,1)+C(3,2,1,3)*F(3,3)*F(1,2)*Fbar(2,1)*Fbar(2,3)+C(1,3,1,2)*F(3,1)*F(1,3)*Fbar(2,1)*Fbar(2,2)+C(2,2,3,2)*F(3,2)*F(1,2)*Fbar(2,3)*Fbar(2,2)+C(2,2,1,2)*F(3,2)*F(1,2)*Fbar(2,1)*Fbar(2,2)+C(1,3,1,3)*F(3,1)*F(1,3)*Fbar(2,1)*Fbar(2,3)+C(2,3,3,1)*F(3,2)*F(1,3)*Fbar(2,3)*Fbar(2,1)+C(1,3,2,1)*F(3,1)*F(1,3)*Fbar(2,2)*Fbar(2,1)+C(3,2,2,2)*F(3,3)*F(1,2)*Fbar(2,2)^2+C(1,1,1,2)*F(3,1)*F(1,1)*Fbar(2,1)*Fbar(2,2)+C(1,1,1,3)*F(3,1)*F(1,1)*Fbar(2,1)*Fbar(2,3)+C(1,1,3,1)*F(3,1)*F(1,1)*Fbar(2,3)*Fbar(2,1)+C(2,1,3,2)*F(3,2)*F(1,1)*Fbar(2,3)*Fbar(2,2)+C(2,1,1,3)*F(3,2)*F(1,1)*Fbar(2,1)*Fbar(2,3)+C(3,3,1,2)*F(3,3)*F(1,3)*Fbar(2,1)*Fbar(2,2)+C(2,2,2,1)*F(3,2)*F(1,2)*Fbar(2,2)*Fbar(2,1)+C(1,1,2,1)*F(3,1)*F(1,1)*Fbar(2,2)*Fbar(2,1)+C(3,1,2,3)*F(3,3)*F(1,1)*Fbar(2,2)*Fbar(2,3)+C(3,1,2,1)*F(3,3)*F(1,1)*Fbar(2,2)*Fbar(2,1)+C(1,2,2,1)*F(3,1)*F(1,2)*Fbar(2,2)*Fbar(2,1)+C(3,1,1,3)*F(3,3)*F(1,1)*Fbar(2,1)*Fbar(2,3)+C(2,1,3,1)*F(3,2)*F(1,1)*Fbar(2,3)*Fbar(2,1)+C(1,2,3,1)*F(3,1)*F(1,2)*Fbar(2,3)*Fbar(2,1)+C(1,3,2,3)*F(3,1)*F(1,3)*Fbar(2,2)*Fbar(2,3)+C(2,3,2,3)*F(3,2)*F(1,3)*Fbar(2,2)*Fbar(2,3)+C(3,1,1,1)*F(3,3)*F(1,1)*Fbar(2,1)^2;
D(5,3)=C(1,2,3,2)*F(3,1)*F(1,2)*Fbar(3,3)*Fbar(3,2)+C(1,3,3,1)*F(3,1)*F(1,3)*Fbar(3,3)*Fbar(3,1)+C(2,3,2,1)*F(3,2)*F(1,3)*Fbar(3,2)*Fbar(3,1)+C(2,3,1,2)*F(3,2)*F(1,3)*Fbar(3,1)*Fbar(3,2)+C(1,1,2,1)*F(3,1)*F(1,1)*Fbar(3,2)*Fbar(3,1)+C(3,2,1,2)*F(3,3)*F(1,2)*Fbar(3,1)*Fbar(3,2)+C(2,3,2,3)*F(3,2)*F(1,3)*Fbar(3,2)*Fbar(3,3)+C(3,1,1,2)*F(3,3)*F(1,1)*Fbar(3,1)*Fbar(3,2)+C(3,3,1,3)*F(3,3)*F(1,3)*Fbar(3,1)*Fbar(3,3)+C(3,3,3,1)*F(3,3)*F(1,3)*Fbar(3,3)*Fbar(3,1)+C(3,2,3,1)*F(3,3)*F(1,2)*Fbar(3,3)*Fbar(3,1)+C(2,1,2,1)*F(3,2)*F(1,1)*Fbar(3,2)*Fbar(3,1)+C(3,3,2,1)*F(3,3)*F(1,3)*Fbar(3,2)*Fbar(3,1)+C(2,1,1,2)*F(3,2)*F(1,1)*Fbar(3,1)*Fbar(3,2)+C(3,2,1,3)*F(3,3)*F(1,2)*Fbar(3,1)*Fbar(3,3)+C(1,2,3,1)*F(3,1)*F(1,2)*Fbar(3,3)*Fbar(3,1)+C(1,1,3,1)*F(3,1)*F(1,1)*Fbar(3,3)*Fbar(3,1)+C(1,1,3,2)*F(3,1)*F(1,1)*Fbar(3,3)*Fbar(3,2)+C(3,3,3,2)*F(3,3)*F(1,3)*Fbar(3,3)*Fbar(3,2)+C(2,2,2,3)*F(3,2)*F(1,2)*Fbar(3,2)*Fbar(3,3)+C(2,2,3,1)*F(3,2)*F(1,2)*Fbar(3,3)*Fbar(3,1)+C(3,2,2,1)*F(3,3)*F(1,2)*Fbar(3,2)*Fbar(3,1)+C(3,3,1,2)*F(3,3)*F(1,3)*Fbar(3,1)*Fbar(3,2)+C(2,1,2,3)*F(3,2)*F(1,1)*Fbar(3,2)*Fbar(3,3)+C(1,1,2,3)*F(3,1)*F(1,1)*Fbar(3,2)*Fbar(3,3)+C(1,2,1,2)*F(3,1)*F(1,2)*Fbar(3,1)*Fbar(3,2)+C(2,2,2,1)*F(3,2)*F(1,2)*Fbar(3,2)*Fbar(3,1)+C(3,1,1,3)*F(3,3)*F(1,1)*Fbar(3,1)*Fbar(3,3)+C(2,3,3,2)*F(3,2)*F(1,3)*Fbar(3,3)*Fbar(3,2)+C(1,3,2,3)*F(3,1)*F(1,3)*Fbar(3,2)*Fbar(3,3)+C(2,2,1,3)*F(3,2)*F(1,2)*Fbar(3,1)*Fbar(3,3)+C(2,1,1,3)*F(3,2)*F(1,1)*Fbar(3,1)*Fbar(3,3)+C(1,2,2,3)*F(3,1)*F(1,2)*Fbar(3,2)*Fbar(3,3)+C(1,2,2,1)*F(3,1)*F(1,2)*Fbar(3,2)*Fbar(3,1)+C(1,3,3,2)*F(3,1)*F(1,3)*Fbar(3,3)*Fbar(3,2)+C(2,3,3,1)*F(3,2)*F(1,3)*Fbar(3,3)*Fbar(3,1)+C(1,3,1,2)*F(3,1)*F(1,3)*Fbar(3,1)*Fbar(3,2)+C(1,3,2,1)*F(3,1)*F(1,3)*Fbar(3,2)*Fbar(3,1)+C(1,3,1,3)*F(3,1)*F(1,3)*Fbar(3,1)*Fbar(3,3)+C(2,2,3,2)*F(3,2)*F(1,2)*Fbar(3,3)*Fbar(3,2)+C(3,1,2,1)*F(3,3)*F(1,1)*Fbar(3,2)*Fbar(3,1)+C(3,2,3,2)*F(3,3)*F(1,2)*Fbar(3,3)*Fbar(3,2)+C(2,1,3,2)*F(3,2)*F(1,1)*Fbar(3,3)*Fbar(3,2)+C(3,1,2,3)*F(3,3)*F(1,1)*Fbar(3,2)*Fbar(3,3)+C(1,1,1,1)*F(3,1)*F(1,1)*Fbar(3,1)^2+C(2,3,3,3)*F(3,2)*F(1,3)*Fbar(3,3)^2+C(2,1,2,2)*F(3,2)*F(1,1)*Fbar(3,2)^2+C(2,3,1,1)*F(3,2)*F(1,3)*Fbar(3,1)^2+C(2,2,3,3)*F(3,2)*F(1,2)*Fbar(3,3)^2+C(1,2,2,2)*F(3,1)*F(1,2)*Fbar(3,2)^2+C(1,1,2,2)*F(3,1)*F(1,1)*Fbar(3,2)^2+C(1,3,2,2)*F(3,1)*F(1,3)*Fbar(3,2)^2+C(2,2,2,2)*F(3,2)*F(1,2)*Fbar(3,2)^2+C(1,3,1,1)*F(3,1)*F(1,3)*Fbar(3,1)^2+C(3,2,2,2)*F(3,3)*F(1,2)*Fbar(3,2)^2+C(3,1,3,3)*F(3,3)*F(1,1)*Fbar(3,3)^2+C(3,1,2,2)*F(3,3)*F(1,1)*Fbar(3,2)^2+C(2,1,1,1)*F(3,2)*F(1,1)*Fbar(3,1)^2+C(1,3,3,3)*F(3,1)*F(1,3)*Fbar(3,3)^2+C(2,3,2,2)*F(3,2)*F(1,3)*Fbar(3,2)^2+C(1,1,3,3)*F(3,1)*F(1,1)*Fbar(3,3)^2+C(3,2,3,3)*F(3,3)*F(1,2)*Fbar(3,3)^2+C(2,1,3,3)*F(3,2)*F(1,1)*Fbar(3,3)^2+C(3,3,1,1)*F(3,3)*F(1,3)*Fbar(3,1)^2+C(1,2,1,1)*F(3,1)*F(1,2)*Fbar(3,1)^2+C(3,1,1,1)*F(3,3)*F(1,1)*Fbar(3,1)^2+C(2,2,1,1)*F(3,2)*F(1,2)*Fbar(3,1)^2+C(3,3,2,2)*F(3,3)*F(1,3)*Fbar(3,2)^2+C(3,3,3,3)*F(3,3)*F(1,3)*Fbar(3,3)^2+C(1,2,3,3)*F(3,1)*F(1,2)*Fbar(3,3)^2+C(3,2,1,1)*F(3,3)*F(1,2)*Fbar(3,1)^2+C(2,3,1,3)*F(3,2)*F(1,3)*Fbar(3,1)*Fbar(3,3)+C(1,2,1,3)*F(3,1)*F(1,2)*Fbar(3,1)*Fbar(3,3)+C(3,1,3,2)*F(3,3)*F(1,1)*Fbar(3,3)*Fbar(3,2)+C(2,1,3,1)*F(3,2)*F(1,1)*Fbar(3,3)*Fbar(3,1)+C(3,2,2,3)*F(3,3)*F(1,2)*Fbar(3,2)*Fbar(3,3)+C(1,1,1,2)*F(3,1)*F(1,1)*Fbar(3,1)*Fbar(3,2)+C(3,1,3,1)*F(3,3)*F(1,1)*Fbar(3,3)*Fbar(3,1)+C(1,1,1,3)*F(3,1)*F(1,1)*Fbar(3,1)*Fbar(3,3)+C(3,3,2,3)*F(3,3)*F(1,3)*Fbar(3,2)*Fbar(3,3)+C(2,2,1,2)*F(3,2)*F(1,2)*Fbar(3,1)*Fbar(3,2);
D(5,4)=C(3,2,2,2)*F(3,3)*F(1,2)*Fbar(2,2)*Fbar(1,2)+C(1,1,1,2)*F(3,1)*F(1,1)*Fbar(2,1)*Fbar(1,2)+C(2,2,2,3)*F(3,2)*F(1,2)*Fbar(2,2)*Fbar(1,3)+C(1,3,2,1)*F(3,1)*F(1,3)*Fbar(2,2)*Fbar(1,1)+C(3,2,3,3)*F(3,3)*F(1,2)*Fbar(2,3)*Fbar(1,3)+C(3,3,1,3)*F(3,3)*F(1,3)*Fbar(2,1)*Fbar(1,3)+C(1,3,2,2)*F(3,1)*F(1,3)*Fbar(2,2)*Fbar(1,2)+C(1,3,3,1)*F(3,1)*F(1,3)*Fbar(2,3)*Fbar(1,1)+C(2,1,2,2)*F(3,2)*F(1,1)*Fbar(2,2)*Fbar(1,2)+C(1,1,2,3)*F(3,1)*F(1,1)*Fbar(2,2)*Fbar(1,3)+C(3,2,1,3)*F(3,3)*F(1,2)*Fbar(2,1)*Fbar(1,3)+C(1,2,2,3)*F(3,1)*F(1,2)*Fbar(2,2)*Fbar(1,3)+C(1,1,2,1)*F(3,1)*F(1,1)*Fbar(2,2)*Fbar(1,1)+C(3,1,3,2)*F(3,3)*F(1,1)*Fbar(2,3)*Fbar(1,2)+C(2,1,2,1)*F(3,2)*F(1,1)*Fbar(2,2)*Fbar(1,1)+C(3,3,2,3)*F(3,3)*F(1,3)*Fbar(2,2)*Fbar(1,3)+C(2,2,1,1)*F(3,2)*F(1,2)*Fbar(2,1)*Fbar(1,1)+C(1,1,3,2)*F(3,1)*F(1,1)*Fbar(2,3)*Fbar(1,2)+C(3,2,1,1)*F(3,3)*F(1,2)*Fbar(2,1)*Fbar(1,1)+C(3,2,3,1)*F(3,3)*F(1,2)*Fbar(2,3)*Fbar(1,1)+C(2,3,1,1)*F(3,2)*F(1,3)*Fbar(2,1)*Fbar(1,1)+C(3,1,2,3)*F(3,3)*F(1,1)*Fbar(2,2)*Fbar(1,3)+C(2,2,2,1)*F(3,2)*F(1,2)*Fbar(2,2)*Fbar(1,1)+C(1,3,1,2)*F(3,1)*F(1,3)*Fbar(2,1)*Fbar(1,2)+C(1,2,2,2)*F(3,1)*F(1,2)*Fbar(2,2)*Fbar(1,2)+C(1,3,2,3)*F(3,1)*F(1,3)*Fbar(2,2)*Fbar(1,3)+C(2,3,1,2)*F(3,2)*F(1,3)*Fbar(2,1)*Fbar(1,2)+C(3,1,1,1)*F(3,3)*F(1,1)*Fbar(2,1)*Fbar(1,1)+C(3,3,2,2)*F(3,3)*F(1,3)*Fbar(2,2)*Fbar(1,2)+C(1,1,2,2)*F(3,1)*F(1,1)*Fbar(2,2)*Fbar(1,2)+C(3,1,2,2)*F(3,3)*F(1,1)*Fbar(2,2)*Fbar(1,2)+C(1,2,1,1)*F(3,1)*F(1,2)*Fbar(2,1)*Fbar(1,1)+C(1,3,3,2)*F(3,1)*F(1,3)*Fbar(2,3)*Fbar(1,2)+C(2,1,1,2)*F(3,2)*F(1,1)*Fbar(2,1)*Fbar(1,2)+C(2,1,1,3)*F(3,2)*F(1,1)*Fbar(2,1)*Fbar(1,3)+C(1,2,3,3)*F(3,1)*F(1,2)*Fbar(2,3)*Fbar(1,3)+C(1,2,2,1)*F(3,1)*F(1,2)*Fbar(2,2)*Fbar(1,1)+C(3,1,2,1)*F(3,3)*F(1,1)*Fbar(2,2)*Fbar(1,1)+C(2,2,2,2)*F(3,2)*F(1,2)*Fbar(2,2)*Fbar(1,2)+C(2,1,3,3)*F(3,2)*F(1,1)*Fbar(2,3)*Fbar(1,3)+C(2,1,3,1)*F(3,2)*F(1,1)*Fbar(2,3)*Fbar(1,1)+C(2,2,1,2)*F(3,2)*F(1,2)*Fbar(2,1)*Fbar(1,2)+C(3,3,3,3)*F(3,3)*F(1,3)*Fbar(2,3)*Fbar(1,3)+C(2,3,1,3)*F(3,2)*F(1,3)*Fbar(2,1)*Fbar(1,3)+C(3,3,1,2)*F(3,3)*F(1,3)*Fbar(2,1)*Fbar(1,2)+C(3,3,1,1)*F(3,3)*F(1,3)*Fbar(2,1)*Fbar(1,1)+C(2,3,2,3)*F(3,2)*F(1,3)*Fbar(2,2)*Fbar(1,3)+C(1,1,3,1)*F(3,1)*F(1,1)*Fbar(2,3)*Fbar(1,1)+C(3,2,2,1)*F(3,3)*F(1,2)*Fbar(2,2)*Fbar(1,1)+C(3,3,3,1)*F(3,3)*F(1,3)*Fbar(2,3)*Fbar(1,1)+C(2,2,3,1)*F(3,2)*F(1,2)*Fbar(2,3)*Fbar(1,1)+C(3,1,1,2)*F(3,3)*F(1,1)*Fbar(2,1)*Fbar(1,2)+C(1,2,1,2)*F(3,1)*F(1,2)*Fbar(2,1)*Fbar(1,2)+C(3,2,3,2)*F(3,3)*F(1,2)*Fbar(2,3)*Fbar(1,2)+C(2,3,3,1)*F(3,2)*F(1,3)*Fbar(2,3)*Fbar(1,1)+C(3,1,3,1)*F(3,3)*F(1,1)*Fbar(2,3)*Fbar(1,1)+C(2,3,2,1)*F(3,2)*F(1,3)*Fbar(2,2)*Fbar(1,1)+C(2,3,3,3)*F(3,2)*F(1,3)*Fbar(2,3)*Fbar(1,3)+C(2,2,3,3)*F(3,2)*F(1,2)*Fbar(2,3)*Fbar(1,3)+C(3,3,3,2)*F(3,3)*F(1,3)*Fbar(2,3)*Fbar(1,2)+C(2,3,3,2)*F(3,2)*F(1,3)*Fbar(2,3)*Fbar(1,2)+C(3,1,3,3)*F(3,3)*F(1,1)*Fbar(2,3)*Fbar(1,3)+C(3,3,2,1)*F(3,3)*F(1,3)*Fbar(2,2)*Fbar(1,1)+C(2,1,1,1)*F(3,2)*F(1,1)*Fbar(2,1)*Fbar(1,1)+C(1,2,3,1)*F(3,1)*F(1,2)*Fbar(2,3)*Fbar(1,1)+C(1,3,1,1)*F(3,1)*F(1,3)*Fbar(2,1)*Fbar(1,1)+C(2,2,1,3)*F(3,2)*F(1,2)*Fbar(2,1)*Fbar(1,3)+C(3,2,1,2)*F(3,3)*F(1,2)*Fbar(2,1)*Fbar(1,2)+C(2,1,2,3)*F(3,2)*F(1,1)*Fbar(2,2)*Fbar(1,3)+C(2,3,2,2)*F(3,2)*F(1,3)*Fbar(2,2)*Fbar(1,2)+C(1,3,1,3)*F(3,1)*F(1,3)*Fbar(2,1)*Fbar(1,3)+C(1,1,3,3)*F(3,1)*F(1,1)*Fbar(2,3)*Fbar(1,3)+C(2,2,3,2)*F(3,2)*F(1,2)*Fbar(2,3)*Fbar(1,2)+C(1,2,3,2)*F(3,1)*F(1,2)*Fbar(2,3)*Fbar(1,2)+C(1,3,3,3)*F(3,1)*F(1,3)*Fbar(2,3)*Fbar(1,3)+C(1,1,1,1)*F(3,1)*F(1,1)*Fbar(2,1)*Fbar(1,1)+C(1,1,1,3)*F(3,1)*F(1,1)*Fbar(2,1)*Fbar(1,3)+C(1,2,1,3)*F(3,1)*F(1,2)*Fbar(2,1)*Fbar(1,3)+C(3,2,2,3)*F(3,3)*F(1,2)*Fbar(2,2)*Fbar(1,3)+C(3,1,1,3)*F(3,3)*F(1,1)*Fbar(2,1)*Fbar(1,3)+C(2,1,3,2)*F(3,2)*F(1,1)*Fbar(2,3)*Fbar(1,2);
D(5,5)=C(3,1,2,3)*F(3,3)*F(1,1)*Fbar(3,2)*Fbar(1,3)+C(3,2,2,3)*F(3,3)*F(1,2)*Fbar(3,2)*Fbar(1,3)+C(2,1,3,2)*F(3,2)*F(1,1)*Fbar(3,3)*Fbar(1,2)+C(2,3,2,3)*F(3,2)*F(1,3)*Fbar(3,2)*Fbar(1,3)+C(3,1,1,3)*F(3,3)*F(1,1)*Fbar(3,1)*Fbar(1,3)+C(1,3,2,2)*F(3,1)*F(1,3)*Fbar(3,2)*Fbar(1,2)+C(1,3,1,3)*F(3,1)*F(1,3)*Fbar(3,1)*Fbar(1,3)+C(1,1,3,3)*F(3,1)*F(1,1)*Fbar(3,3)*Fbar(1,3)+C(2,3,3,1)*F(3,2)*F(1,3)*Fbar(3,3)*Fbar(1,1)+C(3,2,2,1)*F(3,3)*F(1,2)*Fbar(3,2)*Fbar(1,1)+C(1,3,2,1)*F(3,1)*F(1,3)*Fbar(3,2)*Fbar(1,1)+C(2,1,2,3)*F(3,2)*F(1,1)*Fbar(3,2)*Fbar(1,3)+C(3,3,3,2)*F(3,3)*F(1,3)*Fbar(3,3)*Fbar(1,2)+C(1,2,2,1)*F(3,1)*F(1,2)*Fbar(3,2)*Fbar(1,1)+C(2,1,2,2)*F(3,2)*F(1,1)*Fbar(3,2)*Fbar(1,2)+C(2,2,2,2)*F(3,2)*F(1,2)*Fbar(3,2)*Fbar(1,2)+C(2,3,1,1)*F(3,2)*F(1,3)*Fbar(3,1)*Fbar(1,1)+C(1,2,3,1)*F(3,1)*F(1,2)*Fbar(3,3)*Fbar(1,1)+C(2,3,2,1)*F(3,2)*F(1,3)*Fbar(3,2)*Fbar(1,1)+C(2,2,3,1)*F(3,2)*F(1,2)*Fbar(3,3)*Fbar(1,1)+C(2,2,2,1)*F(3,2)*F(1,2)*Fbar(3,2)*Fbar(1,1)+C(2,2,3,3)*F(3,2)*F(1,2)*Fbar(3,3)*Fbar(1,3)+C(2,2,3,2)*F(3,2)*F(1,2)*Fbar(3,3)*Fbar(1,2)+C(1,1,1,1)*F(3,1)*F(1,1)*Fbar(3,1)*Fbar(1,1)+C(2,3,1,2)*F(3,2)*F(1,3)*Fbar(3,1)*Fbar(1,2)+C(3,1,3,2)*F(3,3)*F(1,1)*Fbar(3,3)*Fbar(1,2)+C(2,1,1,3)*F(3,2)*F(1,1)*Fbar(3,1)*Fbar(1,3)+C(1,3,1,2)*F(3,1)*F(1,3)*Fbar(3,1)*Fbar(1,2)+C(1,3,3,1)*F(3,1)*F(1,3)*Fbar(3,3)*Fbar(1,1)+C(3,3,1,1)*F(3,3)*F(1,3)*Fbar(3,1)*Fbar(1,1)+C(2,1,1,2)*F(3,2)*F(1,1)*Fbar(3,1)*Fbar(1,2)+C(3,1,2,1)*F(3,3)*F(1,1)*Fbar(3,2)*Fbar(1,1)+C(1,1,1,3)*F(3,1)*F(1,1)*Fbar(3,1)*Fbar(1,3)+C(1,2,3,3)*F(3,1)*F(1,2)*Fbar(3,3)*Fbar(1,3)+C(1,1,2,3)*F(3,1)*F(1,1)*Fbar(3,2)*Fbar(1,3)+C(3,2,3,1)*F(3,3)*F(1,2)*Fbar(3,3)*Fbar(1,1)+C(1,3,3,3)*F(3,1)*F(1,3)*Fbar(3,3)*Fbar(1,3)+C(2,3,2,2)*F(3,2)*F(1,3)*Fbar(3,2)*Fbar(1,2)+C(2,1,2,1)*F(3,2)*F(1,1)*Fbar(3,2)*Fbar(1,1)+C(1,1,3,1)*F(3,1)*F(1,1)*Fbar(3,3)*Fbar(1,1)+C(3,2,1,3)*F(3,3)*F(1,2)*Fbar(3,1)*Fbar(1,3)+C(2,1,3,1)*F(3,2)*F(1,1)*Fbar(3,3)*Fbar(1,1)+C(1,3,1,1)*F(3,1)*F(1,3)*Fbar(3,1)*Fbar(1,1)+C(3,1,1,2)*F(3,3)*F(1,1)*Fbar(3,1)*Fbar(1,2)+C(3,1,1,1)*F(3,3)*F(1,1)*Fbar(3,1)*Fbar(1,1)+C(1,1,3,2)*F(3,1)*F(1,1)*Fbar(3,3)*Fbar(1,2)+C(3,3,3,3)*F(3,3)*F(1,3)*Fbar(3,3)*Fbar(1,3)+C(1,2,2,3)*F(3,1)*F(1,2)*Fbar(3,2)*Fbar(1,3)+C(1,2,1,2)*F(3,1)*F(1,2)*Fbar(3,1)*Fbar(1,2)+C(1,2,2,2)*F(3,1)*F(1,2)*Fbar(3,2)*Fbar(1,2)+C(2,1,1,1)*F(3,2)*F(1,1)*Fbar(3,1)*Fbar(1,1)+C(1,1,2,1)*F(3,1)*F(1,1)*Fbar(3,2)*Fbar(1,1)+C(2,3,3,2)*F(3,2)*F(1,3)*Fbar(3,3)*Fbar(1,2)+C(3,3,1,3)*F(3,3)*F(1,3)*Fbar(3,1)*Fbar(1,3)+C(3,1,3,1)*F(3,3)*F(1,1)*Fbar(3,3)*Fbar(1,1)+C(2,3,3,3)*F(3,2)*F(1,3)*Fbar(3,3)*Fbar(1,3)+C(3,2,3,3)*F(3,3)*F(1,2)*Fbar(3,3)*Fbar(1,3)+C(2,2,1,2)*F(3,2)*F(1,2)*Fbar(3,1)*Fbar(1,2)+C(1,2,1,3)*F(3,1)*F(1,2)*Fbar(3,1)*Fbar(1,3)+C(3,1,3,3)*F(3,3)*F(1,1)*Fbar(3,3)*Fbar(1,3)+C(2,2,2,3)*F(3,2)*F(1,2)*Fbar(3,2)*Fbar(1,3)+C(3,3,3,1)*F(3,3)*F(1,3)*Fbar(3,3)*Fbar(1,1)+C(2,1,3,3)*F(3,2)*F(1,1)*Fbar(3,3)*Fbar(1,3)+C(2,2,1,3)*F(3,2)*F(1,2)*Fbar(3,1)*Fbar(1,3)+C(1,1,2,2)*F(3,1)*F(1,1)*Fbar(3,2)*Fbar(1,2)+C(2,3,1,3)*F(3,2)*F(1,3)*Fbar(3,1)*Fbar(1,3)+C(1,3,3,2)*F(3,1)*F(1,3)*Fbar(3,3)*Fbar(1,2)+C(3,3,2,1)*F(3,3)*F(1,3)*Fbar(3,2)*Fbar(1,1)+C(2,2,1,1)*F(3,2)*F(1,2)*Fbar(3,1)*Fbar(1,1)+C(1,3,2,3)*F(3,1)*F(1,3)*Fbar(3,2)*Fbar(1,3)+C(3,2,1,2)*F(3,3)*F(1,2)*Fbar(3,1)*Fbar(1,2)+C(1,2,3,2)*F(3,1)*F(1,2)*Fbar(3,3)*Fbar(1,2)+C(1,1,1,2)*F(3,1)*F(1,1)*Fbar(3,1)*Fbar(1,2)+C(3,3,2,2)*F(3,3)*F(1,3)*Fbar(3,2)*Fbar(1,2)+C(3,2,3,2)*F(3,3)*F(1,2)*Fbar(3,3)*Fbar(1,2)+C(3,1,2,2)*F(3,3)*F(1,1)*Fbar(3,2)*Fbar(1,2)+C(3,2,2,2)*F(3,3)*F(1,2)*Fbar(3,2)*Fbar(1,2)+C(3,3,1,2)*F(3,3)*F(1,3)*Fbar(3,1)*Fbar(1,2)+C(3,3,2,3)*F(3,3)*F(1,3)*Fbar(3,2)*Fbar(1,3)+C(1,2,1,1)*F(3,1)*F(1,2)*Fbar(3,1)*Fbar(1,1)+C(3,2,1,1)*F(3,3)*F(1,2)*Fbar(3,1)*Fbar(1,1);
D(5,6)=C(2,1,3,1)*F(3,2)*F(1,1)*Fbar(3,3)*Fbar(2,1)+C(1,2,1,1)*F(3,1)*F(1,2)*Fbar(2,1)*Fbar(3,1)+C(1,1,3,1)*F(3,1)*F(1,1)*Fbar(3,3)*Fbar(2,1)+C(1,3,1,2)*F(3,1)*F(1,3)*Fbar(3,1)*Fbar(2,2)+C(3,2,3,1)*F(3,3)*F(1,2)*Fbar(3,3)*Fbar(2,1)+C(3,2,2,3)*F(3,3)*F(1,2)*Fbar(3,2)*Fbar(2,3)+C(3,3,1,2)*F(3,3)*F(1,3)*Fbar(3,1)*Fbar(2,2)+C(3,2,2,1)*F(3,3)*F(1,2)*Fbar(3,2)*Fbar(2,1)+C(3,1,3,3)*F(3,3)*F(1,1)*Fbar(3,3)*Fbar(2,3)+C(3,2,1,2)*F(3,3)*F(1,2)*Fbar(3,1)*Fbar(2,2)+C(3,1,2,1)*F(3,3)*F(1,1)*Fbar(3,2)*Fbar(2,1)+C(2,3,3,3)*F(3,2)*F(1,3)*Fbar(2,3)*Fbar(3,3)+C(2,3,1,1)*F(3,2)*F(1,3)*Fbar(2,1)*Fbar(3,1)+C(3,1,2,3)*F(3,3)*F(1,1)*Fbar(3,2)*Fbar(2,3)+C(2,2,2,2)*F(3,2)*F(1,2)*Fbar(2,2)*Fbar(3,2)+C(2,2,1,1)*F(3,2)*F(1,2)*Fbar(2,1)*Fbar(3,1)+C(3,3,1,3)*F(3,3)*F(1,3)*Fbar(3,1)*Fbar(2,3)+C(3,2,1,1)*F(3,3)*F(1,2)*Fbar(3,1)*Fbar(2,1)+C(3,1,1,3)*F(3,3)*F(1,1)*Fbar(3,1)*Fbar(2,3)+C(2,2,3,3)*F(3,2)*F(1,2)*Fbar(2,3)*Fbar(3,3)+C(3,1,2,2)*F(3,3)*F(1,1)*Fbar(3,2)*Fbar(2,2)+C(1,3,1,1)*F(3,1)*F(1,3)*Fbar(2,1)*Fbar(3,1)+C(3,2,3,2)*F(3,3)*F(1,2)*Fbar(3,3)*Fbar(2,2)+C(1,1,1,2)*F(3,1)*F(1,1)*Fbar(3,1)*Fbar(2,2)+C(3,3,2,3)*F(3,3)*F(1,3)*Fbar(3,2)*Fbar(2,3)+C(1,3,2,3)*F(3,1)*F(1,3)*Fbar(3,2)*Fbar(2,3)+C(1,2,3,3)*F(3,1)*F(1,2)*Fbar(2,3)*Fbar(3,3)+C(1,1,3,3)*F(3,1)*F(1,1)*Fbar(2,3)*Fbar(3,3)+C(1,1,2,2)*F(3,1)*F(1,1)*Fbar(2,2)*Fbar(3,2)+C(2,3,2,2)*F(3,2)*F(1,3)*Fbar(2,2)*Fbar(3,2)+C(2,1,3,2)*F(3,2)*F(1,1)*Fbar(3,3)*Fbar(2,2)+C(2,2,1,2)*F(3,2)*F(1,2)*Fbar(3,1)*Fbar(2,2)+C(2,1,2,2)*F(3,2)*F(1,1)*Fbar(2,2)*Fbar(3,2)+C(3,3,3,1)*F(3,3)*F(1,3)*Fbar(3,3)*Fbar(2,1)+C(2,3,3,2)*F(3,2)*F(1,3)*Fbar(3,3)*Fbar(2,2)+C(1,2,1,2)*F(3,1)*F(1,2)*Fbar(3,1)*Fbar(2,2)+C(3,3,1,1)*F(3,3)*F(1,3)*Fbar(3,1)*Fbar(2,1)+C(2,3,2,3)*F(3,2)*F(1,3)*Fbar(3,2)*Fbar(2,3)+C(2,2,1,3)*F(3,2)*F(1,2)*Fbar(3,1)*Fbar(2,3)+C(1,3,3,3)*F(3,1)*F(1,3)*Fbar(2,3)*Fbar(3,3)+C(2,3,2,1)*F(3,2)*F(1,3)*Fbar(3,2)*Fbar(2,1)+C(1,2,2,3)*F(3,1)*F(1,2)*Fbar(3,2)*Fbar(2,3)+C(3,1,3,2)*F(3,3)*F(1,1)*Fbar(3,3)*Fbar(2,2)+C(2,2,2,1)*F(3,2)*F(1,2)*Fbar(3,2)*Fbar(2,1)+C(1,3,3,2)*F(3,1)*F(1,3)*Fbar(3,3)*Fbar(2,2)+C(1,3,3,1)*F(3,1)*F(1,3)*Fbar(3,3)*Fbar(2,1)+C(1,1,1,3)*F(3,1)*F(1,1)*Fbar(3,1)*Fbar(2,3)+C(1,1,2,1)*F(3,1)*F(1,1)*Fbar(3,2)*Fbar(2,1)+C(1,2,1,3)*F(3,1)*F(1,2)*Fbar(3,1)*Fbar(2,3)+C(3,2,3,3)*F(3,3)*F(1,2)*Fbar(3,3)*Fbar(2,3)+C(2,2,3,1)*F(3,2)*F(1,2)*Fbar(3,3)*Fbar(2,1)+C(3,1,3,1)*F(3,3)*F(1,1)*Fbar(3,3)*Fbar(2,1)+C(3,2,2,2)*F(3,3)*F(1,2)*Fbar(3,2)*Fbar(2,2)+C(3,2,1,3)*F(3,3)*F(1,2)*Fbar(3,1)*Fbar(2,3)+C(1,3,2,1)*F(3,1)*F(1,3)*Fbar(3,2)*Fbar(2,1)+C(3,3,2,1)*F(3,3)*F(1,3)*Fbar(3,2)*Fbar(2,1)+C(1,1,2,3)*F(3,1)*F(1,1)*Fbar(3,2)*Fbar(2,3)+C(1,1,3,2)*F(3,1)*F(1,1)*Fbar(3,3)*Fbar(2,2)+C(1,2,3,1)*F(3,1)*F(1,2)*Fbar(3,3)*Fbar(2,1)+C(2,3,1,3)*F(3,2)*F(1,3)*Fbar(3,1)*Fbar(2,3)+C(2,1,1,1)*F(3,2)*F(1,1)*Fbar(2,1)*Fbar(3,1)+C(2,3,3,1)*F(3,2)*F(1,3)*Fbar(3,3)*Fbar(2,1)+C(3,1,1,1)*F(3,3)*F(1,1)*Fbar(2,1)*Fbar(3,1)+C(2,2,2,3)*F(3,2)*F(1,2)*Fbar(3,2)*Fbar(2,3)+C(1,2,3,2)*F(3,1)*F(1,2)*Fbar(3,3)*Fbar(2,2)+C(2,1,1,3)*F(3,2)*F(1,1)*Fbar(3,1)*Fbar(2,3)+C(2,1,3,3)*F(3,2)*F(1,1)*Fbar(2,3)*Fbar(3,3)+C(3,3,2,2)*F(3,3)*F(1,3)*Fbar(3,2)*Fbar(2,2)+C(1,3,1,3)*F(3,1)*F(1,3)*Fbar(3,1)*Fbar(2,3)+C(3,3,3,3)*F(3,3)*F(1,3)*Fbar(3,3)*Fbar(2,3)+C(2,1,1,2)*F(3,2)*F(1,1)*Fbar(3,1)*Fbar(2,2)+C(1,2,2,2)*F(3,1)*F(1,2)*Fbar(2,2)*Fbar(3,2)+C(2,1,2,1)*F(3,2)*F(1,1)*Fbar(3,2)*Fbar(2,1)+C(2,2,3,2)*F(3,2)*F(1,2)*Fbar(3,3)*Fbar(2,2)+C(1,1,1,1)*F(3,1)*F(1,1)*Fbar(2,1)*Fbar(3,1)+C(1,2,2,1)*F(3,1)*F(1,2)*Fbar(3,2)*Fbar(2,1)+C(1,3,2,2)*F(3,1)*F(1,3)*Fbar(2,2)*Fbar(3,2)+C(2,3,1,2)*F(3,2)*F(1,3)*Fbar(3,1)*Fbar(2,2)+C(2,1,2,3)*F(3,2)*F(1,1)*Fbar(3,2)*Fbar(2,3)+C(3,3,3,2)*F(3,3)*F(1,3)*Fbar(3,3)*Fbar(2,2)+C(3,1,1,2)*F(3,3)*F(1,1)*Fbar(3,1)*Fbar(2,2);
D(6,1)=C(1,3,3,1)*F(3,1)*F(2,3)*Fbar(1,3)*Fbar(1,1)+C(1,3,2,1)*F(3,1)*F(2,3)*Fbar(1,2)*Fbar(1,1)+C(2,2,2,2)*F(3,2)*F(2,2)*Fbar(1,2)^2+C(2,3,3,2)*F(3,2)*F(2,3)*Fbar(1,3)*Fbar(1,2)+C(1,3,1,1)*F(3,1)*F(2,3)*Fbar(1,1)^2+C(1,2,2,2)*F(3,1)*F(2,2)*Fbar(1,2)^2+C(2,1,3,3)*F(3,2)*F(2,1)*Fbar(1,3)^2+C(3,2,3,3)*F(3,3)*F(2,2)*Fbar(1,3)^2+C(1,1,2,2)*F(3,1)*F(2,1)*Fbar(1,2)^2+C(2,2,1,2)*F(3,2)*F(2,2)*Fbar(1,1)*Fbar(1,2)+C(1,2,1,3)*F(3,1)*F(2,2)*Fbar(1,1)*Fbar(1,3)+C(1,3,3,2)*F(3,1)*F(2,3)*Fbar(1,3)*Fbar(1,2)+C(1,1,3,2)*F(3,1)*F(2,1)*Fbar(1,3)*Fbar(1,2)+C(2,2,2,1)*F(3,2)*F(2,2)*Fbar(1,2)*Fbar(1,1)+C(2,1,3,2)*F(3,2)*F(2,1)*Fbar(1,3)*Fbar(1,2)+C(2,2,1,3)*F(3,2)*F(2,2)*Fbar(1,1)*Fbar(1,3)+C(2,3,1,2)*F(3,2)*F(2,3)*Fbar(1,1)*Fbar(1,2)+C(1,2,3,3)*F(3,1)*F(2,2)*Fbar(1,3)^2+C(3,3,3,2)*F(3,3)*F(2,3)*Fbar(1,3)*Fbar(1,2)+C(3,3,2,3)*F(3,3)*F(2,3)*Fbar(1,2)*Fbar(1,3)+C(3,3,3,1)*F(3,3)*F(2,3)*Fbar(1,3)*Fbar(1,1)+C(2,3,2,1)*F(3,2)*F(2,3)*Fbar(1,2)*Fbar(1,1)+C(3,1,1,3)*F(3,3)*F(2,1)*Fbar(1,1)*Fbar(1,3)+C(2,2,3,1)*F(3,2)*F(2,2)*Fbar(1,3)*Fbar(1,1)+C(1,1,1,3)*F(3,1)*F(2,1)*Fbar(1,1)*Fbar(1,3)+C(2,1,2,2)*F(3,2)*F(2,1)*Fbar(1,2)^2+C(1,1,2,1)*F(3,1)*F(2,1)*Fbar(1,2)*Fbar(1,1)+C(2,1,2,3)*F(3,2)*F(2,1)*Fbar(1,2)*Fbar(1,3)+C(3,1,1,2)*F(3,3)*F(2,1)*Fbar(1,1)*Fbar(1,2)+C(1,1,2,3)*F(3,1)*F(2,1)*Fbar(1,2)*Fbar(1,3)+C(1,3,2,3)*F(3,1)*F(2,3)*Fbar(1,2)*Fbar(1,3)+C(3,1,2,1)*F(3,3)*F(2,1)*Fbar(1,2)*Fbar(1,1)+C(3,2,3,1)*F(3,3)*F(2,2)*Fbar(1,3)*Fbar(1,1)+C(3,3,1,1)*F(3,3)*F(2,3)*Fbar(1,1)^2+C(3,2,2,3)*F(3,3)*F(2,2)*Fbar(1,2)*Fbar(1,3)+C(3,2,1,3)*F(3,3)*F(2,2)*Fbar(1,1)*Fbar(1,3)+C(2,1,2,1)*F(3,2)*F(2,1)*Fbar(1,2)*Fbar(1,1)+C(1,2,3,2)*F(3,1)*F(2,2)*Fbar(1,3)*Fbar(1,2)+C(1,2,2,3)*F(3,1)*F(2,2)*Fbar(1,2)*Fbar(1,3)+C(2,2,2,3)*F(3,2)*F(2,2)*Fbar(1,2)*Fbar(1,3)+C(2,1,1,2)*F(3,2)*F(2,1)*Fbar(1,1)*Fbar(1,2)+C(3,1,1,1)*F(3,3)*F(2,1)*Fbar(1,1)^2+C(1,3,1,3)*F(3,1)*F(2,3)*Fbar(1,1)*Fbar(1,3)+C(3,2,1,2)*F(3,3)*F(2,2)*Fbar(1,1)*Fbar(1,2)+C(3,1,2,3)*F(3,3)*F(2,1)*Fbar(1,2)*Fbar(1,3)+C(1,1,3,1)*F(3,1)*F(2,1)*Fbar(1,3)*Fbar(1,1)+C(2,1,1,3)*F(3,2)*F(2,1)*Fbar(1,1)*Fbar(1,3)+C(1,2,1,2)*F(3,1)*F(2,2)*Fbar(1,1)*Fbar(1,2)+C(3,1,3,2)*F(3,3)*F(2,1)*Fbar(1,3)*Fbar(1,2)+C(2,3,2,2)*F(3,2)*F(2,3)*Fbar(1,2)^2+C(2,1,3,1)*F(3,2)*F(2,1)*Fbar(1,3)*Fbar(1,1)+C(3,1,3,1)*F(3,3)*F(2,1)*Fbar(1,3)*Fbar(1,1)+C(2,2,3,2)*F(3,2)*F(2,2)*Fbar(1,3)*Fbar(1,2)+C(3,3,2,1)*F(3,3)*F(2,3)*Fbar(1,2)*Fbar(1,1)+C(2,2,3,3)*F(3,2)*F(2,2)*Fbar(1,3)^2+C(1,1,3,3)*F(3,1)*F(2,1)*Fbar(1,3)^2+C(3,3,2,2)*F(3,3)*F(2,3)*Fbar(1,2)^2+C(3,1,2,2)*F(3,3)*F(2,1)*Fbar(1,2)^2+C(3,3,3,3)*F(3,3)*F(2,3)*Fbar(1,3)^2+C(1,2,2,1)*F(3,1)*F(2,2)*Fbar(1,2)*Fbar(1,1)+C(1,3,1,2)*F(3,1)*F(2,3)*Fbar(1,1)*Fbar(1,2)+C(3,2,3,2)*F(3,3)*F(2,2)*Fbar(1,3)*Fbar(1,2)+C(3,2,2,1)*F(3,3)*F(2,2)*Fbar(1,2)*Fbar(1,1)+C(2,3,1,1)*F(3,2)*F(2,3)*Fbar(1,1)^2+C(3,2,1,1)*F(3,3)*F(2,2)*Fbar(1,1)^2+C(3,1,3,3)*F(3,3)*F(2,1)*Fbar(1,3)^2+C(3,2,2,2)*F(3,3)*F(2,2)*Fbar(1,2)^2+C(1,3,3,3)*F(3,1)*F(2,3)*Fbar(1,3)^2+C(2,3,3,1)*F(3,2)*F(2,3)*Fbar(1,3)*Fbar(1,1)+C(2,3,2,3)*F(3,2)*F(2,3)*Fbar(1,2)*Fbar(1,3)+C(1,1,1,2)*F(3,1)*F(2,1)*Fbar(1,1)*Fbar(1,2)+C(3,3,1,3)*F(3,3)*F(2,3)*Fbar(1,1)*Fbar(1,3)+C(2,3,1,3)*F(3,2)*F(2,3)*Fbar(1,1)*Fbar(1,3)+C(3,3,1,2)*F(3,3)*F(2,3)*Fbar(1,1)*Fbar(1,2)+C(1,2,3,1)*F(3,1)*F(2,2)*Fbar(1,3)*Fbar(1,1)+C(1,3,2,2)*F(3,1)*F(2,3)*Fbar(1,2)^2+C(1,2,1,1)*F(3,1)*F(2,2)*Fbar(1,1)^2+C(2,1,1,1)*F(3,2)*F(2,1)*Fbar(1,1)^2+C(1,1,1,1)*F(3,1)*F(2,1)*Fbar(1,1)^2+C(2,3,3,3)*F(3,2)*F(2,3)*Fbar(1,3)^2+C(2,2,1,1)*F(3,2)*F(2,2)*Fbar(1,1)^2;
D(6,2)=C(1,2,2,1)*F(3,1)*F(2,2)*Fbar(2,2)*Fbar(2,1)+C(1,3,2,3)*F(3,1)*F(2,3)*Fbar(2,2)*Fbar(2,3)+C(2,2,2,3)*F(3,2)*F(2,2)*Fbar(2,2)*Fbar(2,3)+C(1,2,3,2)*F(3,1)*F(2,2)*Fbar(2,3)*Fbar(2,2)+C(1,1,3,2)*F(3,1)*F(2,1)*Fbar(2,3)*Fbar(2,2)+C(1,3,3,1)*F(3,1)*F(2,3)*Fbar(2,3)*Fbar(2,1)+C(1,1,2,1)*F(3,1)*F(2,1)*Fbar(2,2)*Fbar(2,1)+C(1,1,1,2)*F(3,1)*F(2,1)*Fbar(2,1)*Fbar(2,2)+C(2,3,1,3)*F(3,2)*F(2,3)*Fbar(2,1)*Fbar(2,3)+C(2,3,2,1)*F(3,2)*F(2,3)*Fbar(2,2)*Fbar(2,1)+C(2,1,1,2)*F(3,2)*F(2,1)*Fbar(2,1)*Fbar(2,2)+C(3,1,3,1)*F(3,3)*F(2,1)*Fbar(2,3)*Fbar(2,1)+C(3,2,1,3)*F(3,3)*F(2,2)*Fbar(2,1)*Fbar(2,3)+C(1,2,2,2)*F(3,1)*F(2,2)*Fbar(2,2)^2+C(1,1,1,1)*F(3,1)*F(2,1)*Fbar(2,1)^2+C(1,2,3,3)*F(3,1)*F(2,2)*Fbar(2,3)^2+C(2,1,2,2)*F(3,2)*F(2,1)*Fbar(2,2)^2+C(3,1,1,1)*F(3,3)*F(2,1)*Fbar(2,1)^2+C(1,1,3,3)*F(3,1)*F(2,1)*Fbar(2,3)^2+C(1,3,2,2)*F(3,1)*F(2,3)*Fbar(2,2)^2+C(3,3,1,1)*F(3,3)*F(2,3)*Fbar(2,1)^2+C(2,1,3,3)*F(3,2)*F(2,1)*Fbar(2,3)^2+C(1,3,3,3)*F(3,1)*F(2,3)*Fbar(2,3)^2+C(1,2,1,1)*F(3,1)*F(2,2)*Fbar(2,1)^2+C(1,3,3,2)*F(3,1)*F(2,3)*Fbar(2,3)*Fbar(2,2)+C(2,1,2,3)*F(3,2)*F(2,1)*Fbar(2,2)*Fbar(2,3)+C(2,3,3,1)*F(3,2)*F(2,3)*Fbar(2,3)*Fbar(2,1)+C(3,3,2,3)*F(3,3)*F(2,3)*Fbar(2,2)*Fbar(2,3)+C(3,3,1,2)*F(3,3)*F(2,3)*Fbar(2,1)*Fbar(2,2)+C(2,3,3,2)*F(3,2)*F(2,3)*Fbar(2,3)*Fbar(2,2)+C(1,3,2,1)*F(3,1)*F(2,3)*Fbar(2,2)*Fbar(2,1)+C(2,2,1,3)*F(3,2)*F(2,2)*Fbar(2,1)*Fbar(2,3)+C(2,1,3,1)*F(3,2)*F(2,1)*Fbar(2,3)*Fbar(2,1)+C(1,1,3,1)*F(3,1)*F(2,1)*Fbar(2,3)*Fbar(2,1)+C(3,1,1,2)*F(3,3)*F(2,1)*Fbar(2,1)*Fbar(2,2)+C(3,3,2,1)*F(3,3)*F(2,3)*Fbar(2,2)*Fbar(2,1)+C(2,3,2,3)*F(3,2)*F(2,3)*Fbar(2,2)*Fbar(2,3)+C(1,3,1,2)*F(3,1)*F(2,3)*Fbar(2,1)*Fbar(2,2)+C(2,1,1,3)*F(3,2)*F(2,1)*Fbar(2,1)*Fbar(2,3)+C(3,3,3,1)*F(3,3)*F(2,3)*Fbar(2,3)*Fbar(2,1)+C(1,2,1,2)*F(3,1)*F(2,2)*Fbar(2,1)*Fbar(2,2)+C(2,1,1,1)*F(3,2)*F(2,1)*Fbar(2,1)^2+C(3,3,3,3)*F(3,3)*F(2,3)*Fbar(2,3)^2+C(2,1,2,1)*F(3,2)*F(2,1)*Fbar(2,2)*Fbar(2,1)+C(3,2,2,3)*F(3,3)*F(2,2)*Fbar(2,2)*Fbar(2,3)+C(3,2,1,2)*F(3,3)*F(2,2)*Fbar(2,1)*Fbar(2,2)+C(3,2,3,2)*F(3,3)*F(2,2)*Fbar(2,3)*Fbar(2,2)+C(1,1,2,3)*F(3,1)*F(2,1)*Fbar(2,2)*Fbar(2,3)+C(1,3,1,3)*F(3,1)*F(2,3)*Fbar(2,1)*Fbar(2,3)+C(3,1,2,1)*F(3,3)*F(2,1)*Fbar(2,2)*Fbar(2,1)+C(2,2,1,2)*F(3,2)*F(2,2)*Fbar(2,1)*Fbar(2,2)+C(2,2,2,1)*F(3,2)*F(2,2)*Fbar(2,2)*Fbar(2,1)+C(2,2,1,1)*F(3,2)*F(2,2)*Fbar(2,1)^2+C(3,1,1,3)*F(3,3)*F(2,1)*Fbar(2,1)*Fbar(2,3)+C(3,3,3,2)*F(3,3)*F(2,3)*Fbar(2,3)*Fbar(2,2)+C(3,3,1,3)*F(3,3)*F(2,3)*Fbar(2,1)*Fbar(2,3)+C(1,2,2,3)*F(3,1)*F(2,2)*Fbar(2,2)*Fbar(2,3)+C(2,2,3,2)*F(3,2)*F(2,2)*Fbar(2,3)*Fbar(2,2)+C(2,2,3,1)*F(3,2)*F(2,2)*Fbar(2,3)*Fbar(2,1)+C(3,1,2,3)*F(3,3)*F(2,1)*Fbar(2,2)*Fbar(2,3)+C(1,2,3,1)*F(3,1)*F(2,2)*Fbar(2,3)*Fbar(2,1)+C(3,1,3,2)*F(3,3)*F(2,1)*Fbar(2,3)*Fbar(2,2)+C(3,2,3,1)*F(3,3)*F(2,2)*Fbar(2,3)*Fbar(2,1)+C(2,1,3,2)*F(3,2)*F(2,1)*Fbar(2,3)*Fbar(2,2)+C(3,2,2,1)*F(3,3)*F(2,2)*Fbar(2,2)*Fbar(2,1)+C(2,3,3,3)*F(3,2)*F(2,3)*Fbar(2,3)^2+C(3,2,1,1)*F(3,3)*F(2,2)*Fbar(2,1)^2+C(3,1,3,3)*F(3,3)*F(2,1)*Fbar(2,3)^2+C(3,3,2,2)*F(3,3)*F(2,3)*Fbar(2,2)^2+C(2,3,2,2)*F(3,2)*F(2,3)*Fbar(2,2)^2+C(3,2,2,2)*F(3,3)*F(2,2)*Fbar(2,2)^2+C(1,3,1,1)*F(3,1)*F(2,3)*Fbar(2,1)^2+C(2,3,1,1)*F(3,2)*F(2,3)*Fbar(2,1)^2+C(1,1,2,2)*F(3,1)*F(2,1)*Fbar(2,2)^2+C(3,2,3,3)*F(3,3)*F(2,2)*Fbar(2,3)^2+C(2,2,3,3)*F(3,2)*F(2,2)*Fbar(2,3)^2+C(3,1,2,2)*F(3,3)*F(2,1)*Fbar(2,2)^2+C(2,2,2,2)*F(3,2)*F(2,2)*Fbar(2,2)^2+C(2,3,1,2)*F(3,2)*F(2,3)*Fbar(2,1)*Fbar(2,2)+C(1,2,1,3)*F(3,1)*F(2,2)*Fbar(2,1)*Fbar(2,3)+C(1,1,1,3)*F(3,1)*F(2,1)*Fbar(2,1)*Fbar(2,3);
D(6,3)=C(1,1,2,3)*F(3,1)*F(2,1)*Fbar(3,2)*Fbar(3,3)+C(3,1,3,2)*F(3,3)*F(2,1)*Fbar(3,3)*Fbar(3,2)+C(1,2,1,3)*F(3,1)*F(2,2)*Fbar(3,1)*Fbar(3,3)+C(1,3,3,1)*F(3,1)*F(2,3)*Fbar(3,3)*Fbar(3,1)+C(3,3,3,2)*F(3,3)*F(2,3)*Fbar(3,3)*Fbar(3,2)+C(1,3,1,2)*F(3,1)*F(2,3)*Fbar(3,1)*Fbar(3,2)+C(2,2,1,3)*F(3,2)*F(2,2)*Fbar(3,1)*Fbar(3,3)+C(1,3,2,3)*F(3,1)*F(2,3)*Fbar(3,2)*Fbar(3,3)+C(3,1,1,3)*F(3,3)*F(2,1)*Fbar(3,1)*Fbar(3,3)+C(1,3,1,3)*F(3,1)*F(2,3)*Fbar(3,1)*Fbar(3,3)+C(3,1,1,2)*F(3,3)*F(2,1)*Fbar(3,1)*Fbar(3,2)+C(1,1,3,1)*F(3,1)*F(2,1)*Fbar(3,3)*Fbar(3,1)+C(1,2,2,1)*F(3,1)*F(2,2)*Fbar(3,2)*Fbar(3,1)+C(2,2,1,2)*F(3,2)*F(2,2)*Fbar(3,1)*Fbar(3,2)+C(1,1,3,2)*F(3,1)*F(2,1)*Fbar(3,3)*Fbar(3,2)+C(2,1,1,2)*F(3,2)*F(2,1)*Fbar(3,1)*Fbar(3,2)+C(2,1,3,1)*F(3,2)*F(2,1)*Fbar(3,3)*Fbar(3,1)+C(2,2,2,1)*F(3,2)*F(2,2)*Fbar(3,2)*Fbar(3,1)+C(3,1,3,1)*F(3,3)*F(2,1)*Fbar(3,3)*Fbar(3,1)+C(2,1,2,3)*F(3,2)*F(2,1)*Fbar(3,2)*Fbar(3,3)+C(3,3,2,2)*F(3,3)*F(2,3)*Fbar(3,2)^2+C(3,3,3,1)*F(3,3)*F(2,3)*Fbar(3,3)*Fbar(3,1)+C(3,1,2,3)*F(3,3)*F(2,1)*Fbar(3,2)*Fbar(3,3)+C(3,2,1,3)*F(3,3)*F(2,2)*Fbar(3,1)*Fbar(3,3)+C(2,3,3,1)*F(3,2)*F(2,3)*Fbar(3,3)*Fbar(3,1)+C(2,1,2,1)*F(3,2)*F(2,1)*Fbar(3,2)*Fbar(3,1)+C(3,3,1,3)*F(3,3)*F(2,3)*Fbar(3,1)*Fbar(3,3)+C(2,2,3,2)*F(3,2)*F(2,2)*Fbar(3,3)*Fbar(3,2)+C(1,2,3,1)*F(3,1)*F(2,2)*Fbar(3,3)*Fbar(3,1)+C(2,2,2,3)*F(3,2)*F(2,2)*Fbar(3,2)*Fbar(3,3)+C(3,2,1,2)*F(3,3)*F(2,2)*Fbar(3,1)*Fbar(3,2)+C(3,2,3,2)*F(3,3)*F(2,2)*Fbar(3,3)*Fbar(3,2)+C(3,2,2,1)*F(3,3)*F(2,2)*Fbar(3,2)*Fbar(3,1)+C(3,2,2,3)*F(3,3)*F(2,2)*Fbar(3,2)*Fbar(3,3)+C(2,3,2,1)*F(3,2)*F(2,3)*Fbar(3,2)*Fbar(3,1)+C(1,2,3,2)*F(3,1)*F(2,2)*Fbar(3,3)*Fbar(3,2)+C(1,1,1,2)*F(3,1)*F(2,1)*Fbar(3,1)*Fbar(3,2)+C(1,2,1,2)*F(3,1)*F(2,2)*Fbar(3,1)*Fbar(3,2)+C(2,1,3,2)*F(3,2)*F(2,1)*Fbar(3,3)*Fbar(3,2)+C(2,3,1,2)*F(3,2)*F(2,3)*Fbar(3,1)*Fbar(3,2)+C(2,1,1,3)*F(3,2)*F(2,1)*Fbar(3,1)*Fbar(3,3)+C(1,2,2,3)*F(3,1)*F(2,2)*Fbar(3,2)*Fbar(3,3)+C(2,3,3,2)*F(3,2)*F(2,3)*Fbar(3,3)*Fbar(3,2)+C(2,3,2,3)*F(3,2)*F(2,3)*Fbar(3,2)*Fbar(3,3)+C(3,3,2,1)*F(3,3)*F(2,3)*Fbar(3,2)*Fbar(3,1)+C(3,1,2,1)*F(3,3)*F(2,1)*Fbar(3,2)*Fbar(3,1)+C(1,1,1,3)*F(3,1)*F(2,1)*Fbar(3,1)*Fbar(3,3)+C(2,3,1,3)*F(3,2)*F(2,3)*Fbar(3,1)*Fbar(3,3)+C(3,3,1,2)*F(3,3)*F(2,3)*Fbar(3,1)*Fbar(3,2)+C(3,3,2,3)*F(3,3)*F(2,3)*Fbar(3,2)*Fbar(3,3)+C(1,1,2,1)*F(3,1)*F(2,1)*Fbar(3,2)*Fbar(3,1)+C(1,3,2,1)*F(3,1)*F(2,3)*Fbar(3,2)*Fbar(3,1)+C(3,2,3,1)*F(3,3)*F(2,2)*Fbar(3,3)*Fbar(3,1)+C(1,3,3,2)*F(3,1)*F(2,3)*Fbar(3,3)*Fbar(3,2)+C(2,2,3,1)*F(3,2)*F(2,2)*Fbar(3,3)*Fbar(3,1)+C(1,1,1,1)*F(3,1)*F(2,1)*Fbar(3,1)^2+C(2,1,3,3)*F(3,2)*F(2,1)*Fbar(3,3)^2+C(2,1,2,2)*F(3,2)*F(2,1)*Fbar(3,2)^2+C(3,3,1,1)*F(3,3)*F(2,3)*Fbar(3,1)^2+C(2,2,3,3)*F(3,2)*F(2,2)*Fbar(3,3)^2+C(1,3,3,3)*F(3,1)*F(2,3)*Fbar(3,3)^2+C(3,1,2,2)*F(3,3)*F(2,1)*Fbar(3,2)^2+C(1,2,3,3)*F(3,1)*F(2,2)*Fbar(3,3)^2+C(1,1,2,2)*F(3,1)*F(2,1)*Fbar(3,2)^2+C(3,1,3,3)*F(3,3)*F(2,1)*Fbar(3,3)^2+C(1,1,3,3)*F(3,1)*F(2,1)*Fbar(3,3)^2+C(3,3,3,3)*F(3,3)*F(2,3)*Fbar(3,3)^2+C(1,2,2,2)*F(3,1)*F(2,2)*Fbar(3,2)^2+C(2,1,1,1)*F(3,2)*F(2,1)*Fbar(3,1)^2+C(3,2,3,3)*F(3,3)*F(2,2)*Fbar(3,3)^2+C(3,2,2,2)*F(3,3)*F(2,2)*Fbar(3,2)^2+C(3,2,1,1)*F(3,3)*F(2,2)*Fbar(3,1)^2+C(1,3,1,1)*F(3,1)*F(2,3)*Fbar(3,1)^2+C(1,3,2,2)*F(3,1)*F(2,3)*Fbar(3,2)^2+C(2,2,1,1)*F(3,2)*F(2,2)*Fbar(3,1)^2+C(1,2,1,1)*F(3,1)*F(2,2)*Fbar(3,1)^2+C(3,1,1,1)*F(3,3)*F(2,1)*Fbar(3,1)^2+C(2,2,2,2)*F(3,2)*F(2,2)*Fbar(3,2)^2+C(2,3,2,2)*F(3,2)*F(2,3)*Fbar(3,2)^2+C(2,3,3,3)*F(3,2)*F(2,3)*Fbar(3,3)^2+C(2,3,1,1)*F(3,2)*F(2,3)*Fbar(3,1)^2;
D(6,4)=C(2,2,1,3)*F(3,2)*F(2,2)*Fbar(2,1)*Fbar(1,3)+C(3,2,2,1)*F(3,3)*F(2,2)*Fbar(2,2)*Fbar(1,1)+C(2,2,1,1)*F(3,2)*F(2,2)*Fbar(2,1)*Fbar(1,1)+C(1,1,3,3)*F(3,1)*F(2,1)*Fbar(1,3)*Fbar(2,3)+C(2,1,3,1)*F(3,2)*F(2,1)*Fbar(2,3)*Fbar(1,1)+C(1,2,1,1)*F(3,1)*F(2,2)*Fbar(1,1)*Fbar(2,1)+C(2,1,1,1)*F(3,2)*F(2,1)*Fbar(1,1)*Fbar(2,1)+C(3,3,2,1)*F(3,3)*F(2,3)*Fbar(2,2)*Fbar(1,1)+C(1,2,2,2)*F(3,1)*F(2,2)*Fbar(1,2)*Fbar(2,2)+C(2,1,2,2)*F(3,2)*F(2,1)*Fbar(2,2)*Fbar(1,2)+C(3,1,2,3)*F(3,3)*F(2,1)*Fbar(2,2)*Fbar(1,3)+C(1,2,3,3)*F(3,1)*F(2,2)*Fbar(1,3)*Fbar(2,3)+C(2,3,1,2)*F(3,2)*F(2,3)*Fbar(2,1)*Fbar(1,2)+C(2,2,3,3)*F(3,2)*F(2,2)*Fbar(2,3)*Fbar(1,3)+C(1,3,2,2)*F(3,1)*F(2,3)*Fbar(1,2)*Fbar(2,2)+C(2,1,3,2)*F(3,2)*F(2,1)*Fbar(2,3)*Fbar(1,2)+C(2,2,2,2)*F(3,2)*F(2,2)*Fbar(2,2)*Fbar(1,2)+C(2,2,2,1)*F(3,2)*F(2,2)*Fbar(2,2)*Fbar(1,1)+C(2,3,1,1)*F(3,2)*F(2,3)*Fbar(2,1)*Fbar(1,1)+C(2,2,2,3)*F(3,2)*F(2,2)*Fbar(2,2)*Fbar(1,3)+C(1,1,2,2)*F(3,1)*F(2,1)*Fbar(1,2)*Fbar(2,2)+C(2,3,1,3)*F(3,2)*F(2,3)*Fbar(2,1)*Fbar(1,3)+C(1,1,2,1)*F(3,1)*F(2,1)*Fbar(2,2)*Fbar(1,1)+C(1,1,1,2)*F(3,1)*F(2,1)*Fbar(2,1)*Fbar(1,2)+C(3,2,2,2)*F(3,3)*F(2,2)*Fbar(2,2)*Fbar(1,2)+C(1,3,2,1)*F(3,1)*F(2,3)*Fbar(2,2)*Fbar(1,1)+C(2,3,3,1)*F(3,2)*F(2,3)*Fbar(2,3)*Fbar(1,1)+C(2,1,2,3)*F(3,2)*F(2,1)*Fbar(2,2)*Fbar(1,3)+C(1,3,3,3)*F(3,1)*F(2,3)*Fbar(1,3)*Fbar(2,3)+C(2,1,3,3)*F(3,2)*F(2,1)*Fbar(2,3)*Fbar(1,3)+C(3,1,1,2)*F(3,3)*F(2,1)*Fbar(2,1)*Fbar(1,2)+C(2,3,2,1)*F(3,2)*F(2,3)*Fbar(2,2)*Fbar(1,1)+C(1,3,1,1)*F(3,1)*F(2,3)*Fbar(1,1)*Fbar(2,1)+C(3,1,3,1)*F(3,3)*F(2,1)*Fbar(2,3)*Fbar(1,1)+C(3,3,2,2)*F(3,3)*F(2,3)*Fbar(2,2)*Fbar(1,2)+C(1,2,3,1)*F(3,1)*F(2,2)*Fbar(2,3)*Fbar(1,1)+C(1,3,2,3)*F(3,1)*F(2,3)*Fbar(2,2)*Fbar(1,3)+C(3,3,1,1)*F(3,3)*F(2,3)*Fbar(2,1)*Fbar(1,1)+C(3,3,3,3)*F(3,3)*F(2,3)*Fbar(2,3)*Fbar(1,3)+C(1,1,3,1)*F(3,1)*F(2,1)*Fbar(2,3)*Fbar(1,1)+C(3,3,1,2)*F(3,3)*F(2,3)*Fbar(2,1)*Fbar(1,2)+C(2,1,2,1)*F(3,2)*F(2,1)*Fbar(2,2)*Fbar(1,1)+C(3,2,3,1)*F(3,3)*F(2,2)*Fbar(2,3)*Fbar(1,1)+C(3,3,1,3)*F(3,3)*F(2,3)*Fbar(2,1)*Fbar(1,3)+C(3,3,3,2)*F(3,3)*F(2,3)*Fbar(2,3)*Fbar(1,2)+C(2,2,3,2)*F(3,2)*F(2,2)*Fbar(2,3)*Fbar(1,2)+C(1,2,2,3)*F(3,1)*F(2,2)*Fbar(2,2)*Fbar(1,3)+C(3,2,2,3)*F(3,3)*F(2,2)*Fbar(2,2)*Fbar(1,3)+C(1,1,1,1)*F(3,1)*F(2,1)*Fbar(1,1)*Fbar(2,1)+C(2,3,3,3)*F(3,2)*F(2,3)*Fbar(2,3)*Fbar(1,3)+C(3,3,3,1)*F(3,3)*F(2,3)*Fbar(2,3)*Fbar(1,1)+C(2,1,1,3)*F(3,2)*F(2,1)*Fbar(2,1)*Fbar(1,3)+C(1,2,3,2)*F(3,1)*F(2,2)*Fbar(2,3)*Fbar(1,2)+C(3,2,1,2)*F(3,3)*F(2,2)*Fbar(2,1)*Fbar(1,2)+C(3,1,3,3)*F(3,3)*F(2,1)*Fbar(2,3)*Fbar(1,3)+C(3,2,3,2)*F(3,3)*F(2,2)*Fbar(2,3)*Fbar(1,2)+C(1,2,2,1)*F(3,1)*F(2,2)*Fbar(2,2)*Fbar(1,1)+C(3,1,2,2)*F(3,3)*F(2,1)*Fbar(2,2)*Fbar(1,2)+C(1,3,1,3)*F(3,1)*F(2,3)*Fbar(2,1)*Fbar(1,3)+C(3,1,2,1)*F(3,3)*F(2,1)*Fbar(2,2)*Fbar(1,1)+C(2,3,3,2)*F(3,2)*F(2,3)*Fbar(2,3)*Fbar(1,2)+C(1,3,3,1)*F(3,1)*F(2,3)*Fbar(2,3)*Fbar(1,1)+C(1,1,1,3)*F(3,1)*F(2,1)*Fbar(2,1)*Fbar(1,3)+C(1,3,1,2)*F(3,1)*F(2,3)*Fbar(2,1)*Fbar(1,2)+C(2,1,1,2)*F(3,2)*F(2,1)*Fbar(2,1)*Fbar(1,2)+C(3,3,2,3)*F(3,3)*F(2,3)*Fbar(2,2)*Fbar(1,3)+C(1,1,3,2)*F(3,1)*F(2,1)*Fbar(2,3)*Fbar(1,2)+C(1,2,1,3)*F(3,1)*F(2,2)*Fbar(2,1)*Fbar(1,3)+C(2,3,2,3)*F(3,2)*F(2,3)*Fbar(2,2)*Fbar(1,3)+C(2,2,3,1)*F(3,2)*F(2,2)*Fbar(2,3)*Fbar(1,1)+C(3,1,3,2)*F(3,3)*F(2,1)*Fbar(2,3)*Fbar(1,2)+C(3,2,3,3)*F(3,3)*F(2,2)*Fbar(2,3)*Fbar(1,3)+C(3,1,1,3)*F(3,3)*F(2,1)*Fbar(2,1)*Fbar(1,3)+C(1,1,2,3)*F(3,1)*F(2,1)*Fbar(2,2)*Fbar(1,3)+C(3,1,1,1)*F(3,3)*F(2,1)*Fbar(2,1)*Fbar(1,1)+C(3,2,1,3)*F(3,3)*F(2,2)*Fbar(2,1)*Fbar(1,3)+C(2,2,1,2)*F(3,2)*F(2,2)*Fbar(2,1)*Fbar(1,2)+C(2,3,2,2)*F(3,2)*F(2,3)*Fbar(2,2)*Fbar(1,2)+C(1,3,3,2)*F(3,1)*F(2,3)*Fbar(2,3)*Fbar(1,2)+C(3,2,1,1)*F(3,3)*F(2,2)*Fbar(2,1)*Fbar(1,1)+C(1,2,1,2)*F(3,1)*F(2,2)*Fbar(2,1)*Fbar(1,2);
D(6,5)=C(3,1,2,3)*F(3,3)*F(2,1)*Fbar(3,2)*Fbar(1,3)+C(1,1,3,2)*F(3,1)*F(2,1)*Fbar(3,3)*Fbar(1,2)+C(2,2,2,1)*F(3,2)*F(2,2)*Fbar(3,2)*Fbar(1,1)+C(3,1,2,1)*F(3,3)*F(2,1)*Fbar(3,2)*Fbar(1,1)+C(1,3,3,2)*F(3,1)*F(2,3)*Fbar(3,3)*Fbar(1,2)+C(1,3,2,3)*F(3,1)*F(2,3)*Fbar(3,2)*Fbar(1,3)+C(3,1,2,2)*F(3,3)*F(2,1)*Fbar(3,2)*Fbar(1,2)+C(1,2,1,2)*F(3,1)*F(2,2)*Fbar(3,1)*Fbar(1,2)+C(3,1,3,3)*F(3,3)*F(2,1)*Fbar(3,3)*Fbar(1,3)+C(1,2,3,1)*F(3,1)*F(2,2)*Fbar(3,3)*Fbar(1,1)+C(3,2,2,2)*F(3,3)*F(2,2)*Fbar(3,2)*Fbar(1,2)+C(3,2,2,1)*F(3,3)*F(2,2)*Fbar(3,2)*Fbar(1,1)+C(2,1,3,2)*F(3,2)*F(2,1)*Fbar(3,3)*Fbar(1,2)+C(2,1,1,1)*F(3,2)*F(2,1)*Fbar(3,1)*Fbar(1,1)+C(1,3,3,1)*F(3,1)*F(2,3)*Fbar(3,3)*Fbar(1,1)+C(1,3,1,1)*F(3,1)*F(2,3)*Fbar(3,1)*Fbar(1,1)+C(3,3,1,1)*F(3,3)*F(2,3)*Fbar(3,1)*Fbar(1,1)+C(2,3,1,2)*F(3,2)*F(2,3)*Fbar(3,1)*Fbar(1,2)+C(3,3,1,3)*F(3,3)*F(2,3)*Fbar(3,1)*Fbar(1,3)+C(2,2,3,3)*F(3,2)*F(2,2)*Fbar(3,3)*Fbar(1,3)+C(1,3,1,2)*F(3,1)*F(2,3)*Fbar(3,1)*Fbar(1,2)+C(3,2,3,3)*F(3,3)*F(2,2)*Fbar(3,3)*Fbar(1,3)+C(1,3,2,2)*F(3,1)*F(2,3)*Fbar(3,2)*Fbar(1,2)+C(1,2,1,1)*F(3,1)*F(2,2)*Fbar(3,1)*Fbar(1,1)+C(1,1,1,3)*F(3,1)*F(2,1)*Fbar(3,1)*Fbar(1,3)+C(3,3,2,3)*F(3,3)*F(2,3)*Fbar(3,2)*Fbar(1,3)+C(3,1,1,2)*F(3,3)*F(2,1)*Fbar(3,1)*Fbar(1,2)+C(3,3,2,2)*F(3,3)*F(2,3)*Fbar(3,2)*Fbar(1,2)+C(3,1,3,2)*F(3,3)*F(2,1)*Fbar(3,3)*Fbar(1,2)+C(2,3,1,1)*F(3,2)*F(2,3)*Fbar(3,1)*Fbar(1,1)+C(1,1,3,3)*F(3,1)*F(2,1)*Fbar(3,3)*Fbar(1,3)+C(1,1,2,1)*F(3,1)*F(2,1)*Fbar(3,2)*Fbar(1,1)+C(3,1,1,3)*F(3,3)*F(2,1)*Fbar(3,1)*Fbar(1,3)+C(2,3,2,2)*F(3,2)*F(2,3)*Fbar(3,2)*Fbar(1,2)+C(1,3,1,3)*F(3,1)*F(2,3)*Fbar(3,1)*Fbar(1,3)+C(3,2,1,3)*F(3,3)*F(2,2)*Fbar(3,1)*Fbar(1,3)+C(2,2,3,2)*F(3,2)*F(2,2)*Fbar(3,3)*Fbar(1,2)+C(2,1,2,1)*F(3,2)*F(2,1)*Fbar(3,2)*Fbar(1,1)+C(3,3,3,1)*F(3,3)*F(2,3)*Fbar(3,3)*Fbar(1,1)+C(2,1,3,3)*F(3,2)*F(2,1)*Fbar(3,3)*Fbar(1,3)+C(3,3,3,2)*F(3,3)*F(2,3)*Fbar(3,3)*Fbar(1,2)+C(1,2,2,2)*F(3,1)*F(2,2)*Fbar(3,2)*Fbar(1,2)+C(3,3,2,1)*F(3,3)*F(2,3)*Fbar(3,2)*Fbar(1,1)+C(2,3,2,3)*F(3,2)*F(2,3)*Fbar(3,2)*Fbar(1,3)+C(1,2,3,2)*F(3,1)*F(2,2)*Fbar(3,3)*Fbar(1,2)+C(1,1,1,2)*F(3,1)*F(2,1)*Fbar(3,1)*Fbar(1,2)+C(3,2,3,2)*F(3,3)*F(2,2)*Fbar(3,3)*Fbar(1,2)+C(3,1,1,1)*F(3,3)*F(2,1)*Fbar(3,1)*Fbar(1,1)+C(2,3,3,2)*F(3,2)*F(2,3)*Fbar(3,3)*Fbar(1,2)+C(1,3,3,3)*F(3,1)*F(2,3)*Fbar(3,3)*Fbar(1,3)+C(2,1,1,3)*F(3,2)*F(2,1)*Fbar(3,1)*Fbar(1,3)+C(3,1,3,1)*F(3,3)*F(2,1)*Fbar(3,3)*Fbar(1,1)+C(3,2,1,1)*F(3,3)*F(2,2)*Fbar(3,1)*Fbar(1,1)+C(2,2,2,3)*F(3,2)*F(2,2)*Fbar(3,2)*Fbar(1,3)+C(2,1,2,3)*F(3,2)*F(2,1)*Fbar(3,2)*Fbar(1,3)+C(1,1,2,2)*F(3,1)*F(2,1)*Fbar(3,2)*Fbar(1,2)+C(1,1,2,3)*F(3,1)*F(2,1)*Fbar(3,2)*Fbar(1,3)+C(1,2,1,3)*F(3,1)*F(2,2)*Fbar(3,1)*Fbar(1,3)+C(2,3,2,1)*F(3,2)*F(2,3)*Fbar(3,2)*Fbar(1,1)+C(2,3,3,1)*F(3,2)*F(2,3)*Fbar(3,3)*Fbar(1,1)+C(2,2,1,2)*F(3,2)*F(2,2)*Fbar(3,1)*Fbar(1,2)+C(3,3,3,3)*F(3,3)*F(2,3)*Fbar(3,3)*Fbar(1,3)+C(2,2,2,2)*F(3,2)*F(2,2)*Fbar(3,2)*Fbar(1,2)+C(2,1,1,2)*F(3,2)*F(2,1)*Fbar(3,1)*Fbar(1,2)+C(2,2,1,3)*F(3,2)*F(2,2)*Fbar(3,1)*Fbar(1,3)+C(1,2,2,1)*F(3,1)*F(2,2)*Fbar(3,2)*Fbar(1,1)+C(2,3,3,3)*F(3,2)*F(2,3)*Fbar(3,3)*Fbar(1,3)+C(3,3,1,2)*F(3,3)*F(2,3)*Fbar(3,1)*Fbar(1,2)+C(1,3,2,1)*F(3,1)*F(2,3)*Fbar(3,2)*Fbar(1,1)+C(2,3,1,3)*F(3,2)*F(2,3)*Fbar(3,1)*Fbar(1,3)+C(2,2,1,1)*F(3,2)*F(2,2)*Fbar(3,1)*Fbar(1,1)+C(2,2,3,1)*F(3,2)*F(2,2)*Fbar(3,3)*Fbar(1,1)+C(3,2,3,1)*F(3,3)*F(2,2)*Fbar(3,3)*Fbar(1,1)+C(2,1,3,1)*F(3,2)*F(2,1)*Fbar(3,3)*Fbar(1,1)+C(1,1,1,1)*F(3,1)*F(2,1)*Fbar(3,1)*Fbar(1,1)+C(1,1,3,1)*F(3,1)*F(2,1)*Fbar(3,3)*Fbar(1,1)+C(3,2,2,3)*F(3,3)*F(2,2)*Fbar(3,2)*Fbar(1,3)+C(2,1,2,2)*F(3,2)*F(2,1)*Fbar(3,2)*Fbar(1,2)+C(1,2,3,3)*F(3,1)*F(2,2)*Fbar(3,3)*Fbar(1,3)+C(1,2,2,3)*F(3,1)*F(2,2)*Fbar(3,2)*Fbar(1,3)+C(3,2,1,2)*F(3,3)*F(2,2)*Fbar(3,1)*Fbar(1,2);
D(6,6)=C(3,1,1,1)*F(3,3)*F(2,1)*Fbar(2,1)*Fbar(3,1)+C(3,3,1,1)*F(3,3)*F(2,3)*Fbar(3,1)*Fbar(2,1)+C(3,1,2,2)*F(3,3)*F(2,1)*Fbar(2,2)*Fbar(3,2)+C(3,3,1,3)*F(3,3)*F(2,3)*Fbar(3,1)*Fbar(2,3)+C(1,2,1,2)*F(3,1)*F(2,2)*Fbar(3,1)*Fbar(2,2)+C(3,2,3,3)*F(3,3)*F(2,2)*Fbar(3,3)*Fbar(2,3)+C(1,1,1,1)*F(3,1)*F(2,1)*Fbar(2,1)*Fbar(3,1)+C(3,3,3,3)*F(3,3)*F(2,3)*Fbar(3,3)*Fbar(2,3)+C(1,2,2,1)*F(3,1)*F(2,2)*Fbar(3,2)*Fbar(2,1)+C(3,2,2,2)*F(3,3)*F(2,2)*Fbar(2,2)*Fbar(3,2)+C(1,3,2,2)*F(3,1)*F(2,3)*Fbar(2,2)*Fbar(3,2)+C(2,2,3,2)*F(3,2)*F(2,2)*Fbar(3,3)*Fbar(2,2)+C(2,3,1,3)*F(3,2)*F(2,3)*Fbar(3,1)*Fbar(2,3)+C(3,1,3,2)*F(3,3)*F(2,1)*Fbar(3,3)*Fbar(2,2)+C(3,3,1,2)*F(3,3)*F(2,3)*Fbar(3,1)*Fbar(2,2)+C(3,3,3,2)*F(3,3)*F(2,3)*Fbar(3,3)*Fbar(2,2)+C(3,3,3,1)*F(3,3)*F(2,3)*Fbar(3,3)*Fbar(2,1)+C(3,3,2,2)*F(3,3)*F(2,3)*Fbar(3,2)*Fbar(2,2)+C(1,1,1,2)*F(3,1)*F(2,1)*Fbar(3,1)*Fbar(2,2)+C(1,2,3,3)*F(3,1)*F(2,2)*Fbar(2,3)*Fbar(3,3)+C(1,2,3,1)*F(3,1)*F(2,2)*Fbar(3,3)*Fbar(2,1)+C(2,3,3,2)*F(3,2)*F(2,3)*Fbar(3,3)*Fbar(2,2)+C(1,3,2,1)*F(3,1)*F(2,3)*Fbar(3,2)*Fbar(2,1)+C(3,2,2,3)*F(3,3)*F(2,2)*Fbar(3,2)*Fbar(2,3)+C(2,1,2,1)*F(3,2)*F(2,1)*Fbar(3,2)*Fbar(2,1)+C(1,3,3,1)*F(3,1)*F(2,3)*Fbar(3,3)*Fbar(2,1)+C(3,1,2,3)*F(3,3)*F(2,1)*Fbar(3,2)*Fbar(2,3)+C(2,1,3,2)*F(3,2)*F(2,1)*Fbar(3,3)*Fbar(2,2)+C(1,1,1,3)*F(3,1)*F(2,1)*Fbar(3,1)*Fbar(2,3)+C(2,2,1,1)*F(3,2)*F(2,2)*Fbar(2,1)*Fbar(3,1)+C(1,3,1,2)*F(3,1)*F(2,3)*Fbar(3,1)*Fbar(2,2)+C(1,3,3,3)*F(3,1)*F(2,3)*Fbar(2,3)*Fbar(3,3)+C(3,1,3,1)*F(3,3)*F(2,1)*Fbar(3,3)*Fbar(2,1)+C(2,1,3,1)*F(3,2)*F(2,1)*Fbar(3,3)*Fbar(2,1)+C(1,1,2,3)*F(3,1)*F(2,1)*Fbar(3,2)*Fbar(2,3)+C(1,3,3,2)*F(3,1)*F(2,3)*Fbar(3,3)*Fbar(2,2)+C(3,3,2,1)*F(3,3)*F(2,3)*Fbar(3,2)*Fbar(2,1)+C(1,2,3,2)*F(3,1)*F(2,2)*Fbar(3,3)*Fbar(2,2)+C(3,1,1,2)*F(3,3)*F(2,1)*Fbar(3,1)*Fbar(2,2)+C(3,2,1,1)*F(3,3)*F(2,2)*Fbar(2,1)*Fbar(3,1)+C(2,2,1,3)*F(3,2)*F(2,2)*Fbar(3,1)*Fbar(2,3)+C(1,3,1,1)*F(3,1)*F(2,3)*Fbar(2,1)*Fbar(3,1)+C(1,1,2,1)*F(3,1)*F(2,1)*Fbar(3,2)*Fbar(2,1)+C(2,1,1,2)*F(3,2)*F(2,1)*Fbar(3,1)*Fbar(2,2)+C(3,2,1,2)*F(3,3)*F(2,2)*Fbar(3,1)*Fbar(2,2)+C(3,2,3,1)*F(3,3)*F(2,2)*Fbar(3,3)*Fbar(2,1)+C(2,1,1,3)*F(3,2)*F(2,1)*Fbar(3,1)*Fbar(2,3)+C(1,3,1,3)*F(3,1)*F(2,3)*Fbar(3,1)*Fbar(2,3)+C(2,1,2,2)*F(3,2)*F(2,1)*Fbar(2,2)*Fbar(3,2)+C(2,2,2,2)*F(3,2)*F(2,2)*Fbar(2,2)*Fbar(3,2)+C(1,1,3,3)*F(3,1)*F(2,1)*Fbar(2,3)*Fbar(3,3)+C(2,3,2,1)*F(3,2)*F(2,3)*Fbar(3,2)*Fbar(2,1)+C(2,3,2,3)*F(3,2)*F(2,3)*Fbar(3,2)*Fbar(2,3)+C(2,3,2,2)*F(3,2)*F(2,3)*Fbar(2,2)*Fbar(3,2)+C(2,3,3,1)*F(3,2)*F(2,3)*Fbar(3,3)*Fbar(2,1)+C(1,1,3,2)*F(3,1)*F(2,1)*Fbar(3,3)*Fbar(2,2)+C(2,2,1,2)*F(3,2)*F(2,2)*Fbar(3,1)*Fbar(2,2)+C(3,2,3,2)*F(3,3)*F(2,2)*Fbar(3,3)*Fbar(2,2)+C(3,1,2,1)*F(3,3)*F(2,1)*Fbar(3,2)*Fbar(2,1)+C(2,3,1,1)*F(3,2)*F(2,3)*Fbar(2,1)*Fbar(3,1)+C(2,1,3,3)*F(3,2)*F(2,1)*Fbar(2,3)*Fbar(3,3)+C(3,2,2,1)*F(3,3)*F(2,2)*Fbar(3,2)*Fbar(2,1)+C(1,2,1,3)*F(3,1)*F(2,2)*Fbar(3,1)*Fbar(2,3)+C(1,3,2,3)*F(3,1)*F(2,3)*Fbar(3,2)*Fbar(2,3)+C(2,3,1,2)*F(3,2)*F(2,3)*Fbar(3,1)*Fbar(2,2)+C(2,1,2,3)*F(3,2)*F(2,1)*Fbar(3,2)*Fbar(2,3)+C(2,2,3,3)*F(3,2)*F(2,2)*Fbar(2,3)*Fbar(3,3)+C(2,2,3,1)*F(3,2)*F(2,2)*Fbar(3,3)*Fbar(2,1)+C(1,2,2,2)*F(3,1)*F(2,2)*Fbar(2,2)*Fbar(3,2)+C(2,2,2,1)*F(3,2)*F(2,2)*Fbar(3,2)*Fbar(2,1)+C(1,1,2,2)*F(3,1)*F(2,1)*Fbar(2,2)*Fbar(3,2)+C(2,2,2,3)*F(3,2)*F(2,2)*Fbar(3,2)*Fbar(2,3)+C(2,1,1,1)*F(3,2)*F(2,1)*Fbar(2,1)*Fbar(3,1)+C(3,1,1,3)*F(3,3)*F(2,1)*Fbar(3,1)*Fbar(2,3)+C(3,2,1,3)*F(3,3)*F(2,2)*Fbar(3,1)*Fbar(2,3)+C(3,1,3,3)*F(3,3)*F(2,1)*Fbar(2,3)*Fbar(3,3)+C(2,3,3,3)*F(3,2)*F(2,3)*Fbar(2,3)*Fbar(3,3)+C(3,3,2,3)*F(3,3)*F(2,3)*Fbar(3,2)*Fbar(2,3)+C(1,1,3,1)*F(3,1)*F(2,1)*Fbar(3,3)*Fbar(2,1)+C(1,2,2,3)*F(3,1)*F(2,2)*Fbar(3,2)*Fbar(2,3)+C(1,2,1,1)*F(3,1)*F(2,2)*Fbar(2,1)*Fbar(3,1);

end